(*====================================================================
    File dbmore.pas

    Components library for Delphi and BCB++.
    Creating TBDEDataset clones.

    Copyright  1998-2008, PSTG
    http://www.polesoft.ru
    mailto:support@polesoft.ru

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

======================================================================
$Id: DBMore.pas,v 1.4 2008/09/05 09:00:45 savchenko Exp $
======================================================================*)

unit DBMore;

{$I POLARIS.INC}

interface

uses
  Classes, Db, DBTables, BDE, rDBConst;

type
  TrCloneTable = class(TTable)
  private
    FSourceTable: TTable;
    NewIndexFieldNames, NewIndexName: string;
    procedure SetSourceTable(Value: TTable);
  protected
    function CreateHandle: HDBICur; override;
    procedure OpenCursor(InfoQuery: Boolean); override;
    procedure DoAfterOpen; override;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
  public
  published
    property SourceTable: TTable read FSourceTable write SetSourceTable;
  end;

  TrCloneQuery = class(TQuery)
  private
    FSourceDataSet: TDBDataSet;
    procedure SetSourceDataSet(Value: TDBDataSet);
  protected
    function CreateHandle: HDBICur; override;
    procedure OpenCursor(InfoQuery: Boolean); override;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
  public
  published
    property SourceDataSet: TDBDataSet read FSourceDataSet write SetSourceDataSet;
  end;

implementation

{ TrCloneTable }

procedure TrCloneTable.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and (FSourceTable = AComponent) then FSourceTable := nil;
end;

function TrCloneTable.CreateHandle: HDBICur;
begin
  if FSourceTable <> nil then
    Check(DbiCloneCursor(FSourceTable.Handle, ReadOnly, False, Result))
  else Result := inherited CreateHandle;
end;

procedure TrCloneTable.OpenCursor(InfoQuery: Boolean);
begin
  NewIndexFieldNames := '';
  NewIndexName := '';
  if FSourceTable <> nil then begin
    if not FSourceTable.Active then FSourceTable.Open;
    DatabaseName := FSourceTable.DatabaseName;
    ReadOnly   := FSourceTable.ReadOnly;
    TableName  := FSourceTable.TableName;
    TableType  := FSourceTable.TableType;
    UpdateMode := FSourceTable.UpdateMode;
    if (IndexFieldNames <> '') and
       (IndexFieldNames <> FSourceTable.IndexFieldNames) then begin
      NewIndexFieldNames := IndexFieldNames;
      IndexFieldNames := '';
    end;
    if (IndexName <> '') and (IndexName <> FSourceTable.IndexName) then begin
      NewIndexName := IndexName;
      IndexName := '';
    end;
  end;
  inherited OpenCursor(InfoQuery);
end;

procedure TrCloneTable.DoAfterOpen;
begin
  if NewIndexName <> '' then IndexName := NewIndexName
  else
    if NewIndexFieldNames <> '' then IndexFieldNames := NewIndexFieldNames;
  inherited;
end;

procedure TrCloneTable.SetSourceTable(Value: TTable);
begin
  if Value <> FSourceTable then begin
    CheckInActive;
    FSourceTable := Value;
    if Value <> nil then Value.FreeNotification(Self);
  end;
end;

{ TrCloneQuery }

procedure TrCloneQuery.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and (FSourceDataSet = AComponent) then FSourceDataSet := nil;
end;

function TrCloneQuery.CreateHandle: HDBICur;
begin
  if FSourceDataSet <> nil then
    Check(DbiCloneCursor(FSourceDataSet.Handle, not RequestLive, False, Result))
  else
    Result := inherited CreateHandle;
end;

procedure TrCloneQuery.OpenCursor(InfoQuery: Boolean);
begin                               
  if FSourceDataSet <> nil then begin
    if not FSourceDataSet.Active then FSourceDataSet.Open;
    DatabaseName := FSourceDataSet.DatabaseName;
    if FSourceDataSet is TQuery then
      SQL.Text := TQuery(FSourceDataSet).Text
    else if FSourceDataSet is TTable then
      SQL.Text := 'SELECT * FROM '+TTable(FSourceDataSet).TableName;
  end;
  inherited OpenCursor(InfoQuery);
end;

procedure TrCloneQuery.SetSourceDataSet(Value: TDBDataSet);
begin
  if Value <> FSourceDataSet then begin
    CheckInActive;
    FSourceDataSet := Value;
    if Value <> nil then Value.FreeNotification(Self);
  end;
end;

end.
