(*====================================================================
    File dbtreereg.pas

    Components library for Delphi and BCB++.
    DesignTime  TrDBxxxxTree-.
    DesignTime for TrDBxxxxTree components.

    Copyright  1998-2008, PSTG
    http://www.polesoft.ru
    mailto:support@polesoft.ru

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

======================================================================
$Id: DBTreeReg.pas,v 1.2 2008/09/05 09:00:45 savchenko Exp $
======================================================================*)


unit DBTreeReg;

{$I POLARIS.INC}

interface

uses Classes, TypInfo, SysUtils, DB, rDBTreeView,
     {$IFDEF POLARIS_DELPHI5}DBReg,{$ELSE}PolarisReg,{$ENDIF}
     {$IFDEF POLARIS_D6}DesignIntf,DesignEditors{$ELSE}DsgnIntf{$ENDIF};

procedure Register;

implementation

type
  TrTreeDataFieldProperty = class(TDBStringProperty)
  public
    procedure GetValueList(List: TStrings); override;
    function GetDataSourcePropName: string; virtual;
  end;

  TrTreeLookupFieldProperty = class(TDBStringProperty)
  public
    procedure GetValueList(List: TStrings); override;
    function GetDataSourcePropName: string; virtual;
  end;

function TrTreeDataFieldProperty.GetDataSourcePropName: string;
begin
//  Result := 'LookupSource';
  Result := 'DataSource';
end;

procedure TrTreeDataFieldProperty.GetValueList(List: TStrings);
var
  Instance: TComponent;
  PropInfo: PPropInfo;
  DataSource: TDataSource;
begin
  Instance := TComponent(GetComponent(0));
  PropInfo := TypInfo.GetPropInfo(Instance.ClassInfo, GetDataSourcePropName);
  if (PropInfo <> nil) and (PropInfo^.PropType^.Kind = tkClass) then
  begin
    DataSource := TObject(GetOrdProp(Instance, PropInfo)) as TDataSource;
    if (DataSource <> nil) and (DataSource.DataSet <> nil) then
      DataSource.DataSet.GetFieldNames(List);
  end;
end;

function TrTreeLookupFieldProperty.GetDataSourcePropName: string;
begin
  Result := 'LookupSource';
end;

procedure TrTreeLookupFieldProperty.GetValueList(List: TStrings);
var
  Instance: TComponent;
  PropInfo: PPropInfo;
  DataSource: TDataSource;
begin
  Instance := TComponent(GetComponent(0));
  PropInfo := TypInfo.GetPropInfo(Instance.ClassInfo, GetDataSourcePropName);
  if (PropInfo <> nil) and (PropInfo^.PropType^.Kind = tkClass) then
  begin
    DataSource := TObject(GetOrdProp(Instance, PropInfo)) as TDataSource;
    if (DataSource <> nil) and (DataSource.DataSet <> nil) then
      DataSource.DataSet.GetFieldNames(List);
  end;
end;

procedure Register;
begin
  RegisterComponents('Polaris DB', [TrDBTreeView, TrDBLookupTree, TrDBLookupComboTree]);

  RegisterPropertyEditor(TypeInfo(string), TrDBTreeView, 'KeyField',
    TrTreeDataFieldProperty);
  RegisterPropertyEditor(TypeInfo(string), TrDBTreeView, 'ChildrenField',
    TrTreeDataFieldProperty);
  RegisterPropertyEditor(TypeInfo(string), TrDBTreeView, 'ParentField',
    TrTreeDataFieldProperty);
  RegisterPropertyEditor(TypeInfo(string), TrDBTreeView, 'DisplayFields',
    TrTreeDataFieldProperty);

  RegisterPropertyEditor(TypeInfo(string), TrDBLookupTree, 'DataField',
    TrTreeDataFieldProperty);
  RegisterPropertyEditor(TypeInfo(string), TrDBLookupTree, 'LookupField',
    TrTreeLookupFieldProperty);
  RegisterPropertyEditor(TypeInfo(string), TrDBLookupTree, 'ChildrenField',
    TrTreeLookupFieldProperty);
  RegisterPropertyEditor(TypeInfo(string), TrDBLookupTree, 'ParentField',
    TrTreeLookupFieldProperty);
  RegisterPropertyEditor(TypeInfo(string), TrDBLookupTree, 'LookupDisplay',
    TrTreeLookupFieldProperty);

  RegisterPropertyEditor(TypeInfo(string), TrDBLookupComboTree, 'DataField',
    TrTreeDataFieldProperty);
  RegisterPropertyEditor(TypeInfo(string), TrDBLookupComboTree, 'LookupField',
    TrTreeLookupFieldProperty);
  RegisterPropertyEditor(TypeInfo(string), TrDBLookupComboTree, 'ChildrenField',
    TrTreeLookupFieldProperty);
  RegisterPropertyEditor(TypeInfo(string), TrDBLookupComboTree, 'ParentField',
    TrTreeLookupFieldProperty);
  RegisterPropertyEditor(TypeInfo(string), TrDBLookupComboTree, 'LookupDisplay',
    TrTreeLookupFieldProperty);
end;

end.
