(*====================================================================
    File ErrorMes.pas

    Components library for Delphi and BCB++.
     .
    Errors handler.

    Copyright  1998-2008, PSTG
    http://www.polesoft.ru
    mailto:support@polesoft.ru

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

======================================================================
$Id: ErrorMes.pas,v 1.3 2008/09/05 09:00:45 savchenko Exp $
======================================================================*)

unit ErrorMes;

{$I POLARIS.INC}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, rUtils,
  ExtCtrls, StdCtrls, Buttons, rErrorUtils, IniFiles, rLabel, rConst, rVCLUtils,
  {$IFDEF POLARIS_D6}RxStrUtils{$ELSE}StrUtils{$ENDIF};

type
  TrEHStyle = (ehCustom, ehInformation, ehWarning, ehQuestion, ehError);
  TrEHExtBtn = (ebNone, ebAlways, ebAsSaveLog, ebDescNotEmpty);

const
  rEHIcons: array[TrEHStyle] of PChar =
               (nil, IDI_INFORMATION, IDI_WARNING, IDI_QUESTION, IDI_ERROR);
  rEHBeeps: array[TrEHStyle] of DWORD =
               ($FFFFFFFF, MB_ICONASTERISK, MB_ICONEXCLAMATION, MB_ICONQUESTION, MB_ICONHAND);

type
  TErrorHandler = class;

  TrOnExtButtonSetVisible =
    procedure (Sender: TObject; var SetVisible: Boolean) of object;

  TrFormErrorMes = class(TForm)
    MError: TMemo;
    BitBtnOk: TBitBtn;
    Image1: TImage;
    LError: TLabel;
    BitBtnAll: TBitBtn;
    LMessage: TrLabel;
    ExtButton: TBitBtn;
    BBAntiDefault: TBitBtn;
    procedure FormCreate(Sender: TObject);
    procedure BitBtnAllClick(Sender: TObject);
    procedure FormActivate(Sender: TObject);
  private
    { Private declarations }
    OriginalHeight: Integer;
  public
    { Public declarations }
  end;

  TErrorHandler = class(TComponent)
  private
    FUserName:    string;
    FCaption:     TCaption;
    FShowForm:    Boolean;
    FCancel:      Boolean;
    FDefault:     Boolean;
    FStyle:       TrEHStyle;
    FPicture:     TPicture;
    FExtBtnCaption: TCaption;
    FExtBtnVisible: TrEHExtBtn;
    FOnExtButtonClick: TNotifyEvent;
    FOnExtButtonSetVisible: TrOnExtButtonSetVisible;
    FErrorHolder: TrError;
    function GetIniFile: string;
    procedure SetIniFile(Value: string);
    function GetIniLocation: TrFileLocation;
    procedure SetIniLocation(Value: TrFileLocation);
    function GetLogFile: string;
    procedure SetLogFile(Value: string);
    function GetLogLocation: TrFileLocation;
    procedure SetLogLocation(Value: TrFileLocation);
    function GetMessage: TCaption;
    procedure SetMessage(Value: TCaption);
    function GetTitleMessage: TCaption;
    procedure SetTitleMessage(Value: TCaption);
    function GetDescription: TStrings;
    procedure SetDescription(Value: TStrings);
    function GetSaveLog: Boolean;
    procedure SetSaveLog(Value: Boolean);
    procedure ExtButtonClick(Sender: TObject);
    procedure SetPicture(Value: TPicture);
    function GetConstant(const aName: string): string;
  protected
    { Protected declarations }
  public
    property ErrorHolder: TrError read FErrorHolder;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Execute;
    procedure ShowMessage(aCaption, aTitleMes, aMes, aDesc: TCaption; aStyle: TrEHStyle);
    function  ConvertMessage(mes: string): string;
    procedure AnalyzeException(E: Exception);
    procedure AppException(Sender: TObject; E: Exception);
    procedure SavingLog;
    property Constant[const aName: string]: string read GetConstant;
  published
    property Caption: TCaption     read FCaption     write FCaption;
    property Cancel: Boolean       read FCancel      write FCancel default True;
    property Default: Boolean      read FDefault     write FDefault default True;
    property IniFile: string       read GetIniFile   write SetIniFile;
    property IniLocation: TrFileLocation read GetIniLocation write SetIniLocation;
    property LogFile: string       read GetLogFile   write SetLogFile;
    property LogLocation: TrFileLocation read GetLogLocation write SetLogLocation;
    property UserName: string      read FUserName    write FUserName;
    property TitleMessage: TCaption read GetTitleMessage  write SetTitleMessage;
    property Message: TCaption     read GetMessage     write SetMessage;
    property Description: TStrings read GetDescription write SetDescription;
    property SaveLog: Boolean      read GetSaveLog     write SetSaveLog;
    property ShowForm: Boolean     read FShowForm    write FShowForm default True;
    property Style: TrEHStyle      read FStyle       write FStyle  default ehError;
    property Picture: TPicture     read FPicture     write SetPicture;
    property ExtBtnCaption: TCaption read FExtBtnCaption   write FExtBtnCaption;
    property ExtBtnVisible: TrEHExtBtn read FExtBtnVisible write FExtBtnVisible default ebNone;
    property OnExtButtonClick: TNotifyEvent read FOnExtButtonClick write FOnExtButtonClick;
    property OnExtButtonSetVisible: TrOnExtButtonSetVisible read FOnExtButtonSetVisible write FOnExtButtonSetVisible;
  end;

var
  rFormErrorMes: TrFormErrorMes;

implementation

{$R *.DFM}

{ TErrorHandler }

constructor TErrorHandler.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FErrorHolder := TrError.Create;
  FPicture := TPicture.Create;
  FCaption := Application.Title;
  FShowForm := True;
  FStyle := ehError;
  FExtBtnVisible := ebNone;
  FCancel := True;
  FDefault := True;
end;

destructor TErrorHandler.Destroy;
begin
  inherited Destroy;
  FPicture.Free;
  FErrorHolder.Free;
end;

function TErrorHandler.GetIniFile: string;
begin
  Result := ErrorHolder.IniFile;
end;

procedure TErrorHandler.SetIniFile(Value: string);
begin
  ErrorHolder.IniFile := Value;
end;

function TErrorHandler.GetIniLocation: TrFileLocation;
begin
  Result := ErrorHolder.IniLocation;
end;

procedure TErrorHandler.SetIniLocation(Value: TrFileLocation);
begin
  ErrorHolder.IniLocation := Value;
end;

function TErrorHandler.GetLogFile: string;
begin
  Result := ErrorHolder.LogFile;
end;

procedure TErrorHandler.SetLogFile(Value: string);
begin
  ErrorHolder.LogFile := Value;
end;

function TErrorHandler.GetLogLocation: TrFileLocation;
begin
  Result := ErrorHolder.LogLocation;
end;

procedure TErrorHandler.SetLogLocation(Value: TrFileLocation);
begin
  ErrorHolder.LogLocation := Value;
end;

function TErrorHandler.GetMessage: TCaption;
begin
  Result := ErrorHolder.Message;
end;

procedure TErrorHandler.SetMessage(Value: TCaption);
begin
  ErrorHolder.Message := Value;
end;

function TErrorHandler.GetTitleMessage: TCaption;
begin
  Result := ErrorHolder.PreMessage;
end;

procedure TErrorHandler.SetTitleMessage(Value: TCaption);
begin
  ErrorHolder.PreMessage := Value;
end;

procedure TErrorHandler.SetPicture(Value: TPicture);
begin
  FPicture.Assign(Value);
end;

function TErrorHandler.GetDescription: TStrings;
begin
  Result := ErrorHolder.Description;
end;

procedure TErrorHandler.SetDescription(Value: TStrings);
begin
  ErrorHolder.Description.Assign(Value);
end;

procedure TErrorHandler.ShowMessage;
begin
  FCaption := aCaption;
  TitleMessage := aTitleMes;
  Message := aMes;
  Description.Clear;
  Description.Add(aDesc);
  FStyle := aStyle;
  Execute;
end;

procedure TErrorHandler.ExtButtonClick(Sender: TObject);
begin
  if Assigned(FOnExtButtonClick) then FOnExtButtonClick(Self);
end;

procedure TErrorHandler.Execute;
var
  b: Boolean;
begin
  MessageBeep(rEHBeeps[FStyle]);
  with TrFormErrorMes.Create(Application) do
  try
    if FStyle = ehCustom then
      Image1.Picture.Assign(FPicture)
    else
      Image1.Picture.Icon.Handle := LoadIcon(0,rEHIcons[FStyle]);
    Caption := FCaption;
    BitBtnOk.Cancel := FCancel;
    BitBtnOk.Default := FDefault;
    if not FDefault then
      ActiveControl := BBAntiDefault;
    LError.Caption := TitleMessage;
    LMessage.Caption := Message;
    MError.Lines.Assign(Description);
    with ExtButton do begin
      Caption := FExtBtnCaption;
      OnClick := ExtButtonClick;
      if Assigned(FOnExtButtonClick) then begin
        case FExtBtnVisible of
          ebAlways: Visible := True;
          ebAsSaveLog: Visible := SaveLog;
          ebDescNotEmpty: Visible := Trim(Description.Text) <> '';
        else
          Visible := False;
        end;
        if Assigned(FOnExtButtonSetVisible) then begin
          b := Visible;
          FOnExtButtonSetVisible(Self, b);
          Visible := b;
        end;
      end;
    end;
    ShowModal;
  finally
    Free;
  end;
end;

function TErrorHandler.ConvertMessage(mes: string): string;
begin
  ErrorHolder.ConvertMessage(mes);
end;

procedure TErrorHandler.AnalyzeException(E: Exception);
begin
  ErrorHolder.AnalyzeException(E);
end;

procedure TErrorHandler.SavingLog;
var
  s: string;
begin
  s := '';
  if Assigned(Screen.ActiveForm) then
    with Screen.ActiveForm do begin
      s := Format(srEHForm, [Name,Caption]);
      if Assigned(ActiveControl) then
        s := s + sLineBreak + Format(srEHControl, [ActiveControl.Name]);
    end;
  ErrorHolder.SavingLog(s);
end;

function TErrorHandler.GetConstant(const aName: string): string;
begin
  ErrorHolder.Constant[aName];
end;

procedure TErrorHandler.AppException(Sender: TObject; E: Exception);
begin
  FCaption := Application.Title;
  ErrorHolder.AnalyzeException(E);
  if SaveLog then
    try
      SavingLog;
    except
      on E: Exception do
        Application.MessageBox(PChar(Format(ConvertCodes(srLogError), [E.Message])),
                               PChar(FCaption), MB_OK+MB_ICONERROR);
    end;
  if FShowForm then Execute;
end;

function TErrorHandler.GetSaveLog: Boolean;
begin
  Result := ErrorHolder.SaveLog;
end;

procedure TErrorHandler.SetSaveLog(Value: Boolean);
begin
  ErrorHolder.SaveLog := Value;
end;

{ TrFormErrorMes }

procedure TrFormErrorMes.FormCreate(Sender: TObject);
begin
  HelpFile := srHelpFile;
  OriginalHeight := Height;
  BitBtnAll.Caption := srEHDetail;
end;

procedure TrFormErrorMes.BitBtnAllClick(Sender: TObject);
begin
  with MError do begin
    Visible := not Visible;
    if Visible then begin
      Self.Height := OriginalHeight;
      BitBtnAll.Caption := srEHSummary;
      BitBtnAll.HelpContext := 10303;
    end else begin
      Self.Height := Self.Height-(Self.ClientHeight-Top);
      BitBtnAll.Caption := srEHDetail;
      BitBtnAll.HelpContext := 10302;
    end;
  end;
end;

procedure TrFormErrorMes.FormActivate(Sender: TObject);
begin
  BitBtnAll.Enabled := (Trim(MError.Text) <> '');
  if not MError.Visible then
    Height := Height-(ClientHeight-MError.Top);
end;

end.

