(*====================================================================
    File Launch.pas

    Components library for Delphi and BCB++.
       .
    File (program) launch.

    Copyright  1998-2008, PSTG
    http://www.polesoft.ru
    mailto:support@polesoft.ru

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

======================================================================
$Id: Launch.pas,v 1.3 2008/09/05 09:00:45 savchenko Exp $
======================================================================*)

unit Launch;

{$I POLARIS.INC}

interface

uses
  Windows, Messages, SysUtils, Classes, Forms,
  ExtCtrls, ShellApi, rConst
{$IFNDEF POLARIS_D4}
  , StrUtils   // RxLib
{$ENDIF};

type
  TrLaunchMethod = (lmCreateProcess, lmShellExecute);

  TrCreationFlag = (cfDebugProcess,cfDebugOnlyThisProcess,cfSuspended,cfDetachedProcess,
                    cfNewConsole,cfNewProcessGroup,cfUnicodeEnvironment,
                    cfSeparateWOW_VDM,cfSharedWOW_VDM,cfForceDOS,cfDefaultErrorMode,
                    cfNoWindow,cfProfileUser,cfProfileKernel,cfProfileServer);
  TrCreationFlags = set of TrCreationFlag;
  TrPriorityClass = (pcIdle, pcNormal, pcHigh, pcRealTime);

  TrStartupFlag = (sfUseShowWindow,sfUsePosition,sfUseSize,sfUseConsoleChars,sfUseFillAttributes,
                   sfForceOnFeedback,sfForceOffFeedback,sfRunFullScreen,sfStdHandles,sfUseHotKey);
  TrStartupFlags = set of TrStartupFlag;
  TShowMode = (smHide, smNormal, smMaximized, smMinimized, smMinNoActive, smNoActivate);

  TrFillAttribute = (faForeBlue,faForeGreen,faForeRed,faForeIntensity,
                     faBackBlue,faBackGreen,faBackRed,faBackIntensity);
  TrFillAttributes = set of TrFillAttribute;

  TrShellExecuteMask =
     (semClassName, semClassKey, semIDList, semInvokeIDList, semIcon, semHotKey,
      semNoCloseProcess, semConnectNetDrv, semFlagDDEWait, semDoEnvSubst,
      semFlagNoUI, semUnicode, semNoConsole, semAsyncOK);
  TrShellExecuteMasks = set of TrShellExecuteMask;

  TrBeforeLaunch = procedure(Sender: TObject; var StartupInfo: TStartupInfo;
                                              var ShellExecuteInfo: TShellExecuteInfo) of object;
  TrOnTimeoutElapsed = procedure(Sender: TObject; WaitInit: Boolean) of object;

const
  rCreationFlags: array[TrCreationFlag] of DWORD =
     (DEBUG_PROCESS,DEBUG_ONLY_THIS_PROCESS,CREATE_SUSPENDED,DETACHED_PROCESS,
      CREATE_NEW_CONSOLE,CREATE_NEW_PROCESS_GROUP,CREATE_UNICODE_ENVIRONMENT,
      CREATE_SEPARATE_WOW_VDM,CREATE_SHARED_WOW_VDM,CREATE_FORCEDOS,CREATE_DEFAULT_ERROR_MODE,
      CREATE_NO_WINDOW,PROFILE_USER,PROFILE_KERNEL,PROFILE_SERVER);

  rPriorityClasses: array[TrPriorityClass] of DWORD =
     (IDLE_PRIORITY_CLASS, NORMAL_PRIORITY_CLASS, HIGH_PRIORITY_CLASS, REALTIME_PRIORITY_CLASS);

  rStartupFlags: array[TrStartupFlag] of DWORD =
     (STARTF_USESHOWWINDOW,STARTF_USEPOSITION,STARTF_USESIZE,STARTF_USECOUNTCHARS,
      STARTF_USEFILLATTRIBUTE,STARTF_FORCEONFEEDBACK,STARTF_FORCEOFFFEEDBACK,
      STARTF_RUNFULLSCREEN,STARTF_USESTDHANDLES,STARTF_USEHOTKEY);

  ShowWindowModes: array[TShowMode] of Word =
     (SW_HIDE, SW_SHOWNORMAL, SW_SHOWMAXIMIZED, SW_SHOWMINIMIZED, SW_SHOWMINNOACTIVE, SW_SHOWNOACTIVATE);

  rFillAttributes: array[TrFillAttribute] of DWORD =
     (FOREGROUND_BLUE,FOREGROUND_GREEN,FOREGROUND_RED,FOREGROUND_INTENSITY,
      BACKGROUND_BLUE,BACKGROUND_GREEN,BACKGROUND_RED,BACKGROUND_INTENSITY);

  rShellExecuteMask: array[TrShellExecuteMask] of ULONG =
     (SEE_MASK_CLASSNAME, SEE_MASK_CLASSKEY, SEE_MASK_IDLIST, SEE_MASK_INVOKEIDLIST,
      SEE_MASK_ICON, SEE_MASK_HOTKEY, SEE_MASK_NOCLOSEPROCESS, SEE_MASK_CONNECTNETDRV,
      SEE_MASK_FLAG_DDEWAIT, SEE_MASK_DOENVSUBST, SEE_MASK_FLAG_NO_UI,
      SEE_MASK_UNICODE, SEE_MASK_NO_CONSOLE, SEE_MASK_ASYNCOK);

type
  TLauncher = class(TComponent)
  private
    FFirstTick, FLastTick: DWORD;
    FProcessInfo: TProcessInformation;
    FFullProgramName: string;
    FParameters: string;
    FWorkDir: string;
    FLaunchMethod: TrLaunchMethod;
    FCreationFlags: TrCreationFlags;
    FPriority: TrPriorityClass;
    FShowMode: TShowMode;
    FDesktop: string;
    FConsoleTitle: string;
    FWindowLeft, FWindowTop: Integer;
    FWindowWidth, FWindowHeight: Cardinal;
    FConsoleCharsX, FConsoleCharsY: Cardinal;
    FConsoleFillAttributes: TrFillAttributes;
    FEnvironment: TStrings;
    FStartupFlags: TrStartupFlags;
    FTimer: TTimer;
    FTimerInterval: Cardinal;
    FShellOperation: string;
    FShellExecuteMask: TrShellExecuteMasks;
    FTimeoutWaitInit: Cardinal;
    FTimeoutWaitTerminate: Cardinal;
    FWaitTerminate: Boolean;
    FOnFinished: TNotifyEvent;
    FOnTimeoutElapsed: TrOnTimeoutElapsed;
    FBeforeLaunch: TrBeforeLaunch;
    procedure SetEnvironment(Value: TStrings);
    function GetActive: Boolean;
  protected
    procedure RaiseError(ErrCode: Integer);
    procedure Finished; dynamic;
    procedure TimerExpired(Sender: TObject);
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Launch;
    procedure Terminate(ExitCode: Integer);
    property ProcessInfo: TProcessInformation read FProcessInfo;
    property Active: Boolean read GetActive;
  published
    property ProgramName: string read FFullProgramName write FFullProgramName;
    property Parameters: string read FParameters write FParameters;
    property WorkDir: string read FWorkDir write FWorkDir;
    property LaunchMethod: TrLaunchMethod read FLaunchMethod write FLaunchMethod default lmCreateProcess;
    property CreationFlags: TrCreationFlags read FCreationFlags write FCreationFlags
       default [];
    property Priority: TrPriorityClass read FPriority write FPriority default pcNormal;
    property ShowMode: TShowMode read FShowMode write FShowMode default smNormal;
    property Desktop: string read FDesktop write FDesktop;
    property ConsoleTitle: string read FConsoleTitle write FConsoleTitle;
    property WindowLeft: Integer read FWindowLeft write FWindowLeft default 0;
    property WindowTop: Integer read FWindowTop write FWindowTop default 0;
    property WindowWidth: Cardinal read FWindowWidth write FWindowWidth default 0;
    property WindowHeight: Cardinal read FWindowHeight write FWindowHeight default 0;
    property ConsoleCharsX: Cardinal read FConsoleCharsX write FConsoleCharsX default 0;
    property ConsoleCharsY: Cardinal read FConsoleCharsY write FConsoleCharsY default 0;
    property ConsoleFillAttributes: TrFillAttributes read FConsoleFillAttributes write FConsoleFillAttributes
       default [];
    property Environment: TStrings read FEnvironment write SetEnvironment;
    property StartupFlags: TrStartupFlags read FStartupFlags write FStartupFlags
       default [sfUseShowWindow];
    property TimerInterval: Cardinal read FTimerInterval write FTimerInterval default 1000;
    property ShellOperation: string read FShellOperation write FShellOperation;
    property ShellExecuteMask: TrShellExecuteMasks read FShellExecuteMask write FShellExecuteMask
       default [semNoCloseProcess];
    property TimeoutWaitInit: Cardinal read FTimeoutWaitInit write FTimeoutWaitInit default 0;
    property TimeoutWaitTerminate: Cardinal read FTimeoutWaitTerminate write FTimeoutWaitTerminate default 0;
    property WaitTerminate: Boolean read FWaitTerminate write FWaitTerminate default False;
    property OnFinished: TNotifyEvent read FOnFinished write FOnFinished;
    property OnTimeoutElapsed: TrOnTimeoutElapsed read FOnTimeoutElapsed write FOnTimeoutElapsed;
    property BeforeLaunch: TrBeforeLaunch read FBeforeLaunch write FBeforeLaunch;
  end;

implementation

procedure TLauncher.RaiseError(ErrCode: Integer);
var
  ErrMsg: string;
begin
  case ErrCode of
    ERROR_FILE_NOT_FOUND: ErrMsg := Format(srLaunchNotFound,[AnsiUpperCase(FFullProgramName)]);
    ERROR_BAD_FORMAT, ERROR_BAD_EXE_FORMAT: ErrMsg := Format(srLaunchNotExe,[AnsiUpperCase(FFullProgramName)]);
    ERROR_DIRECTORY: ErrMsg := Format(srLaunchNoWorkDir,[AnsiUpperCase(FWorkDir)]);
  else
{$IFDEF POLARIS_D6}
    RaiseLastOSError;
{$ELSE}
    RaiseLastWin32Error;
{$ENDIF}
  end;
  raise {$IFDEF POLARIS_D6}EOSError{$ELSE}EWin32Error{$ENDIF}.Create(ErrMsg);
end;

constructor TLauncher.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FLaunchMethod := lmCreateProcess;
  FPriority := pcNormal;
  FShowMode := smNormal;
  FStartupFlags := [sfUseShowWindow];
  FEnvironment := TStringList.Create;
  FTimerInterval := 1000;
  FShellExecuteMask := [semNoCloseProcess];
  FillChar(FProcessInfo,SizeOf(FProcessInfo),0);
  WaitTerminate := False;
end;

destructor TLauncher.Destroy;
begin
  if Assigned(FTimer) then
    with FTimer do begin
      Enabled := False;
      Free;
    end;
  FEnvironment.Free;
  inherited Destroy;
end;

procedure TLauncher.SetEnvironment(Value: TStrings);
begin
  FEnvironment.Assign(Value);
end;

procedure TLauncher.Finished;
begin
  if (FTimeoutWaitTerminate > 0) and (FFirstTick+FTimeoutWaitTerminate <= FLastTick) then
    if Assigned(FOnTimeoutElapsed) then FOnTimeoutElapsed(Self,False)
    else
  else
    if Assigned(FOnFinished) then FOnFinished(Self);
end;

procedure TLauncher.TimerExpired(Sender: TObject);
var
  eCode: DWORD;
begin
  FLastTick := GetTickCount;
  // if Windows is run continuously for 49.7 days
  if FLastTick < FFirstTick then FFirstTick := FLastTick;
  if not GetExitCodeProcess(FProcessInfo.hProcess,eCode) or
     (eCode <> STILL_ACTIVE) or
     ((FTimeoutWaitTerminate > 0) and (FFirstTick+FTimeoutWaitTerminate <= FLastTick)) then begin
    FillChar(FProcessInfo,SizeOf(FProcessInfo),0);
    FTimer.Enabled := False;
    Finished;
  end;
end;

procedure TLauncher.Launch;
var
  StartupInfo: TStartupInfo;
  ShellExecuteInfo: TShellExecuteInfo;
  dwCreationFlags: DWORD;
  Success: BOOL;
  PWorkDir, lpEnv: PChar;
  I: TrStartupFlag;
  J: TrFillAttribute;
  K: TrCreationFlag;
  L: TrShellExecuteMask;
  wt: DWORD;
  s: string;

begin
  if FWorkDir = '' then PWorkDir := NIL
                   else PWorkDir := PChar(FWorkDir);

  if FLaunchMethod = lmCreateProcess then begin
    dwCreationFlags := rPriorityClasses[FPriority];
    for K := Low(TrCreationFlag) to High(TrCreationFlag) do
      if K in FCreationFlags then dwCreationFlags := dwCreationFlags or rCreationFlags[K];
    if FEnvironment.Text = '' then lpEnv := NIL
{$IFDEF POLARIS_D4}
                              else lpEnv := PChar(StringReplace(FEnvironment.Text,#13#10,#0,[rfReplaceAll])+#0);
{$ELSE}
                              else lpEnv := PChar(ReplaceStr(FEnvironment.Text,#13#10,#0)+#0);
{$ENDIF}

    FillChar(StartupInfo,SizeOf(StartupInfo),0);
    with StartupInfo do begin
      cb := SizeOf(StartupInfo);
      if FDesktop = '' then lpDesktop := NIL
                       else lpDesktop := PChar(FDesktop);
      if FConsoleTitle = '' then lpTitle := NIL
                            else lpTitle := PChar(FConsoleTitle);
      dwX := FWindowLeft;
      dwY := FWindowTop;
      dwXSize := FWindowWidth;
      dwYSize := FWindowHeight;
      dwXCountChars := FConsoleCharsX;
      dwYCountChars := FConsoleCharsY;
      for J := Low(TrFillAttribute) to High(TrFillAttribute) do
        if J in FConsoleFillAttributes then
          dwFillAttribute := dwFillAttribute or rFillAttributes[J];
      for I := Low(TrStartupFlag) to High(TrStartupFlag) do
        if I in FStartupFlags then dwFlags := dwFlags or rStartupFlags[I];
      wShowWindow := ShowWindowModes[FShowMode];
    end;

    if Assigned(FBeforeLaunch) then FBeforeLaunch(Self,StartupInfo,ShellExecuteInfo);
    Success := CreateProcess(NIL, PChar(FFullProgramName+' '+FParameters),
                             NIL, NIL, False, dwCreationFlags, lpEnv,
                             PWorkDir, StartupInfo, FProcessInfo);
  end else begin  //--- ShellExecute
    FillChar(ShellExecuteInfo, SizeOf(ShellExecuteInfo), 0);
    with ShellExecuteInfo do begin
      cbSize := SizeOf(ShellExecuteInfo);
      for L := Low(TrShellExecuteMask) to High(TrShellExecuteMask) do
        if L in FShellExecuteMask then fMask := fMask or rShellExecuteMask[L];
      Wnd := Application.Handle;
      if FShellOperation = '' then lpVerb := NIL
                              else lpVerb := PChar(FShellOperation);

      s := Trim(FFullProgramName);
      // quoted path is not working on Windows NT4
      if (Length(s) > 2) and (s[1] = '"') and (s[Length(s)] = '"') then
        s := Copy(s, 2, Length(s) - 2);
      lpFile := PChar(s);

      if FParameters = '' then lpParameters := NIL
                          else lpParameters := PChar(FParameters);
      lpDirectory := PWorkDir;
      nShow := ShowWindowModes[FShowMode];
    end;

    if Assigned(FBeforeLaunch) then FBeforeLaunch(Self,StartupInfo,ShellExecuteInfo);
    Success := ShellExecuteEx(@ShellExecuteInfo);
    FProcessInfo.hProcess := ShellExecuteInfo.hProcess;
  end;

  if not Success then RaiseError(GetLastError)
  else begin
    if FTimeoutWaitInit > 0 then
      case WaitForInputIdle(FProcessInfo.hProcess, FTimeoutWaitInit) of
        0: ;
        WAIT_TIMEOUT: if Assigned(FOnTimeoutElapsed) then FOnTimeoutElapsed(Self,True);
      else
        RaiseError(GetLastError);
      end;

    if FWaitTerminate then begin
      if FTimeoutWaitTerminate <= 0 then
        wt := INFINITE
      else
        wt := FTimeoutWaitTerminate;
      case WaitForSingleObject(FProcessInfo.hProcess, wt) of
        WAIT_FAILED:   RaiseError(GetLastError);
        WAIT_TIMEOUT:  if Assigned(FOnTimeoutElapsed) then FOnTimeoutElapsed(Self,False);
        WAIT_OBJECT_0: if Assigned(FOnFinished) then FOnFinished(Self);
      end;
    end else
    if (FTimeoutWaitTerminate > 0) or Assigned(FOnFinished) then begin
      FFirstTick := GetTickCount;
      FLastTick := 0;
      if not Assigned(FTimer) then
        FTimer := TTimer.Create(Self);
      with FTimer do begin
        Interval := FTimerInterval;
        OnTimer := TimerExpired;
        FTimer.Enabled := True;
      end;
    end;
  end;
end;

procedure TLauncher.Terminate(ExitCode: Integer);
begin
  if Active and (FProcessInfo.hProcess <> 0) then
    if TerminateProcess(FProcessInfo.hProcess, ExitCode) then
      RaiseError(GetLastError);
end;

function TLauncher.GetActive: Boolean;
begin
  Result := Assigned(FTimer) and FTimer.Enabled;
end;

end.
