(*====================================================================
    File rBkgnd.pas

    Components library for Delphi and BCB++.
    Save and config TCustomForm properties.

    Copyright  1998-2008, PSTG
    http://www.polesoft.ru
    mailto:support@polesoft.ru

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

======================================================================
$Id: rBkgnd.pas,v 1.3 2008/07/10 08:32:13 khmelevskiy Exp $
======================================================================*)
unit rBkgnd;

{$I POLARIS.INC}

{$IFDEF POLARIS_D6}
  {$WARN SYMBOL_PLATFORM OFF}
  {$WARN UNIT_PLATFORM OFF}
{$ENDIF}

interface

uses Windows, SysUtils, Classes, Graphics, Forms, Controls, StdCtrls,
  Buttons, ExtCtrls, Dialogs, FileCtrl, ComCtrls, rButtons, Mask,
  RXSpin, RxCombos, Registry, Messages, Consts, rVCLUtils, ClipBrd;

type

//rHookWindow
  PClass = ^TClass;
  THookMessageEvent = procedure (Sender: TObject; var Msg: TMessage;
    var Handled: Boolean) of object;

  TrWindowHook = class(TComponent)
  private
    FActive: Boolean;
    FControl: TWinControl;
    FControlHook: TObject;
    FBeforeMessage: THookMessageEvent;
    FAfterMessage: THookMessageEvent;
    FClientBeforeMessage: THookMessageEvent;
    FClientAfterMessage: THookMessageEvent;
    function GetWinControl: TWinControl;
    function GetHookHandle: HWnd;
    procedure SetActive(Value: Boolean);
    procedure SetWinControl(Value: TWinControl);
    function IsForm: Boolean;
    function NotIsForm: Boolean;
    function DoUnhookControl: Pointer;
    procedure ReadForm(Reader: TReader);
    procedure WriteForm(Writer: TWriter);
  protected
    procedure DefineProperties(Filer: TFiler); override;
    procedure DoAfterMessage(var Msg: TMessage; var Handled: Boolean); dynamic;
    procedure DoBeforeMessage(var Msg: TMessage; var Handled: Boolean); dynamic;
    procedure DoClientAfterMessage(var Msg: TMessage; var Handled: Boolean); dynamic;
    procedure DoClientBeforeMessage(var Msg: TMessage; var Handled: Boolean); dynamic;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure HookControl;
    procedure UnhookControl;
    property HookWindow: HWnd read GetHookHandle;
  published
    property Active: Boolean read FActive write SetActive default True;
    property WinControl: TWinControl read GetWinControl write SetWinControl
      stored NotIsForm;
    property BeforeMessage: THookMessageEvent read FBeforeMessage write FBeforeMessage;
    property AfterMessage: THookMessageEvent read FAfterMessage write FAfterMessage;
    property ClientBeforeMessage: THookMessageEvent read FClientBeforeMessage write FClientBeforeMessage;
    property ClientAfterMessage: THookMessageEvent read FClientAfterMessage write FClientAfterMessage;
  end;

  TrViewMode = (rvmNone, rvmTile, rvmCenter, rvmStretch);
  TrViewWindowMode = (rvwmNone, rvwmMaximized, rvwmCentered);

  TrPosSizeEnabled = (rpsPosEnabled, rpsSizeEnabled);
  TrPosSizeSet = Set of TrPosSizeEnabled;

type TGraphicFileName = string;

//        
type TrBackgroundValue = class
     private
       FOwner: TComponent;
       FMaxValue: array[0..3] of integer;
       FGlyph: Graphics.TPicture;
       FColor: TColor;
       FViewMode: TrViewMode;
       FXPos, FYPos, FWidth, FHeight: integer;
       FState: TrViewWindowMode;
       FFileName: TGraphicFileName;
       procedure SetColor( AValue: TColor );
       procedure SetViewMode( AValue: TrViewMode );
       procedure SetXPos( AValue: integer );
       procedure SetYPos( AValue: integer );
       procedure SetWidth( AValue: integer );
       procedure SetHeight( AValue: integer );
       procedure SetState( AValue: TrViewWindowMode );
       function IsCenteredState( Form: TCustomForm ): Boolean;
       procedure MakeMax(Sender: TObject);
       procedure SetFileName(const AValue: TGraphicFileName);
     public
       constructor Create( AOwner: TComponent ); dynamic;
       destructor Destroy; override;
       //     
       procedure LoadfromForm( Form: TCustomForm );
       //    
       procedure StoreToForm( Form: TCustomForm );
       //    
       procedure StoreToRegistry( const RegKey, ValueName: String );
       //    
       procedure LoadFromRegistry( const RegKey, ValueName: String );
       //   
       function TestValue( Value: TrBackGroundValue ): boolean;
       //    
       procedure SetDefault;
       procedure Assign( Value: TrBackgroundValue );
       //  
       property Glyph: TPicture read FGlyph;
       //  
       property Color: TColor read FColor write SetColor;
       property ViewMode: TrViewMode read FViewMode write SetViewMode;
       property XPos: integer read FXPos write SetXPos;
       property YPos: integer read FYPos write SetYPos;
       property Width: integer read FWidth write SetWidth;
       property Height: integer read FHeight write SetHeight;
       property State: TrViewWindowMode read FState write SetState;
       property FileName: TGraphicFileName read FFileName write SetFileName;
     end;

  THackForm = class(TCustomForm);

  TBkgValueNameType = (bvntName, bvntClassName);

  TrBackground = class(TComponent)
  private
    BackgroundLoaded: boolean;
    FAlignToolBars: boolean;
    FAfterLoadBackground: TNotifyEvent;
    FBeforeSaveBackground: TNotifyEvent;
    FFileName: TGraphicFileName;
    FAutoLoad: Boolean;
    FBackgroundControls: TList;
    FValue: TrBackGroundValue;
    FBindControl: TCustomForm;
    FPrevRect: TRect;
    FPosSizeEnabled : TrPosSizeSet;
    FValueName: TBkgValueNameType;
    FSaveSplitterPos,
    FSaveToolBar,
    FRedrawBkg,
    FAutoSave,
    FUpdate: Boolean;
    FLastFileName,
    FProgramKey: String;
    FFileHistory: TStringList;
    FMaxValue: Array[0..3] of Integer;
    FHook: TrWindowHook;
    FCloseEvent: TCloseEvent;
    FOnHideEvent,
    FCreateEvent: TNotifyEvent;
    procedure SetProgramKey(Value: String);
    procedure SetBindControl(Value: TCustomForm);
    function  GetBindControl: TCustomform;
    //procedure SetCurrentBkgnd(Value: TrBackgroundValue);
    //procedure SetBkgndColor(Value: TColor);
    //procedure SetBkgndFileName(Value: TFileName);
    //procedure SetBkgndViewMode(Value: TrViewMode);
    procedure SetRedrawBkg(Value: Boolean);
    procedure SetColor( AValue: TColor );
    procedure SetViewMode( AValue: TrViewMode );
    function GetColor: TColor;
    function GetViewMode: TrViewMode;
    procedure SetXPos( AValue: integer );
    procedure SetYPos( AValue: integer );
    procedure SetWidth( AValue: integer );
    procedure SetHeight( AValue: integer );
    procedure SetState( AValue: TrViewWindowMode );
    function GetXPos: integer;
    function GetYPos: integer;
    function GetWidth: integer;
    function GetHeight: integer;
    function GetState: TrViewWindowMode;
    procedure SetAutoLoad( AValue: Boolean );
    //      ToolBar-
    function LoadOldToolBar( Form: TCustomForm; DeleteOldValue, StoreNewValue: boolean ): boolean;
    procedure SetAfterLoadBackground( AValue: TNotifyEvent );
    procedure SetBeforeSaveBackground( AValue: TNotifyEvent );
    procedure PrepareCurrentToolBars;
  protected
    { Protected declarations }
    procedure BeforeHook(Sender: TObject; var Msg: TMessage; var Handled: Boolean);
    procedure AfterHook(Sender: TObject; var Msg: TMessage; var Handled: Boolean);
    procedure ClientBeforeHook(Sender: TObject; var Msg: TMessage; var Handled: Boolean);
    procedure ClientAfterHook(Sender: TObject; var Msg: TMessage; var Handled: Boolean);

    function GetCurrentRect: TRect;
    procedure Loaded; override;
    procedure _DrawBackground( DC: HDC );
    procedure DrawBackGround( Canvas: TCanvas );
    procedure MakeMax(Sender: TObject);
    procedure ChangeSizePos;
    procedure Notification(AComponent: TComponent;
      Operation: TOperation); override;

    procedure LoadOtherControl;

    function LoadBackground(Value: TrBackgroundValue): Boolean;
    function TestBackground(Value1, Value2: TrBackgroundValue): Boolean;
    property BindControl: TCustomForm read GetBindControl write SetBindControl;
    function IsNotDefaultRegKey: Boolean;
    function IsNotDefaultFileName: Boolean;
    procedure RefreshForm;
    procedure DestroyMessage(Sender: TObject);

    procedure OnFormClose(Sender: TObject; var Action: TCloseAction);
    procedure OnFormCreate(Sender: TObject);
    procedure OnFormHide(Sender: TObject);
    procedure InternalSave(Sender: TObject);
    function GetValueSaveName(Sender: TObject): String;
    function GetGlyph: TPicture;
    procedure SetFileName( const AValue: TGraphicFileName );
    function GetValue: TrBackgroundValue;
  public
    NewToolBarLoaded: Boolean;
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function CurrentForm: THackForm;
    function Execute: Boolean;
    procedure SaveBackground;
    //property CurrentBkgnd: TrBackgroundStruct read FCurrentBkgnd write SetCurrentBkgnd;
    property Value: TrBackgroundValue read GetValue;
    property Glyph: TPicture read GetGlyph;
    property LeftPos: integer read GetXPos write SetXPos stored False;
    property TopPos: integer read GetYPos write SetYPos stored False;
    property Width: integer read GetWidth write SetWidth stored False;
    property Height: integer read GetHeight write SetHeight stored False;
    property State: TrViewWindowMode read GetState write SetState stored False;
  published
    { Published declarations }
    property AlignToolbars: boolean read FAlignToolBars write FAlignToolBars default True;
    property RegKey: String read FProgramKey write SetProgramKey stored IsNotDefaultRegKey;
    property Color: TColor read GetColor write SetColor stored False default clBtnFace ;
    //property FileName: TFileName read FCurrentBkgnd.FileName write SetBkgndFileName stored IsNotDefaultFileName;
    property ViewMode: TrViewMode read GetViewMode write SetViewMode default rvmNone;
    property PosSizeEnabled: TrPosSizeSet read FPosSizeEnabled write FPosSizeEnabled default [rpsPosEnabled, rpsSizeEnabled];
    property AutoSave: Boolean read FAutoSave write FAutoSave default False;
    property AutoLoad: Boolean read FAutoLoad write SetAutoLoad default True;
    property SaveSplitterPos: Boolean read FSaveSplitterPos write FSaveSplitterPos default True;
    property SaveToolBar: Boolean read FSaveToolBar write FSaveToolBar default True;
    property RedrawBkg: Boolean read FRedrawBkg write SetRedrawBkg default True;
    property ValueName: TBkgValueNameType read FValueName write FValueName default bvntName;
    property FileName: TGraphicFileName read FFileName write SetFileName;
    property AfterLoadBackground: TNotifyEvent read FAfterLoadBackground
                                               write SetAfterLoadBackground;
    property BeforeSaveBackground: TNotifyEvent read FBeforeSaveBackground
                                                write SetBeforeSaveBackground;
  end;

  TrBackgroundDlg = class(TForm)
    TestPanel: TPanel;
    GroupBox1: TGroupBox;
    GroupBox2: TGroupBox;
    FileComboBox: TComboBox;
    FileBtn: TButton;
    ComboBox2: TComboBox;
    Label1: TLabel;
    PosBox: TGroupBox;
    RLeft: TRxSpinEdit;
    Label2: TLabel;
    Label3: TLabel;
    RTop: TRxSpinEdit;
    OKBtn: TrBitBtn;
    CancelBtn: TrBitBtn;
    RestoreBtn: TrBitBtn;
    Bevel1: TBevel;
    SizeBox: TGroupBox;
    Label4: TLabel;
    Label5: TLabel;
    RHeight: TRxSpinEdit;
    RWidth: TRxSpinEdit;
    Panel1: TPanel;
    rBitBtn1: TrBitBtn;
    RPosition: TComboBox;
    Label6: TLabel;
    ColorComboBox1: TColorComboBox;
    PaintBox1: TPaintBox;
    procedure ColorBtnClick(Sender: TObject);
    procedure FileComboBoxChange(Sender: TObject);
    procedure FileBtnClick(Sender: TObject);
    procedure RestoreBtnClick(Sender: TObject);
    procedure ComboBox2Change(Sender: TObject);
    function FormHelp(Command: Word; Data: Integer;
      var CallHelp: Boolean): Boolean;
    procedure RHeightChange(Sender: TObject);
    procedure ColorComboBox1Change(Sender: TObject);
    procedure FileComboBoxClick(Sender: TObject);
    procedure PaintBox1Paint(Sender: TObject);
  private
    { Private declarations }
    FFileName: String;
    FGlyph: TPicture;
    FDefBkgnd: TrBackgroundValue;
    procedure SetDefBkgnd(const Value: TrBackgroundValue);
    function GetGlyph: TPicture;
    procedure SetColor( AValue: TColor );
    function GetColor: TColor;
    procedure SetFileName( const AValue: TGraphicFileName );
    function GetFileName: TGraphicFileName;
    procedure SetViewMode( AValue: TrViewMode );
    function GetViewMode: TrViewMode;
    procedure SetXPos( AValue: integer );
    function GetXPos: integer;
    procedure SetYPos( AValue: integer );
    function GetYPos: integer;
    procedure SetFormWidth( AValue: integer );
    procedure SetFormHeight( AValue: integer );
    function GetFormWidth: integer;
    function GetFormHeight: integer;
    procedure SetFormState( AValue: TrViewWindowMode );
    function GetFormState: TrViewWindowMode;
    procedure SetBkgFileName(Value: TGraphicFileName);
  protected
    procedure DoChangeSizePos;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure SetmaxValue(Index: Integer; Value: Integer);
    procedure AssignFrom( Value: TrBackgroundValue );
    procedure AssignToValue( Value: TrBackgroundValue );
    property DefBkgnd: TrBackgroundValue read FDefBkgnd write SetDefBkgnd;
    property Color: TColor read GetColor write SetColor;
    property FileName: TGraphicFileName read GetFileName write SetBkgFileName;
    property ViewMode: TrViewMode read GetViewMode write SetViewMode;
    property XPos: Integer read GetXPos write SetXPos;
    property YPos: Integer read GetYPos write SetYPos;
    property FormWidth: Integer read GetFormWidth write SetFormWidth;
    property FormHeight: Integer read GetFormHeight write SetFormHeight;
    property FormState: TrViewWindowMode read GetFormState write SetFormState;
    property Glyph: TPicture read GetGlyph;
  end;

const
  CM_RECREATEWINDOW  = CM_BASE + 102;
  CM_DESTROYHOOK     = CM_BASE + 103;

var
  rBackgroundDlg: TrBackgroundDlg;

//  ,   ,    
//  ,   
//        Initialization
//    RegisterBackgroundControl
Type TComponentClass = class of TComponent;
Type TBackGroundControl = class
     private
       FControlClass: TComponentClass;
       FAddRegValueName: String;
       FOwner: TrBackground;
       FCurrentForm: TCustomForm;
       procedure LoadBigString( const S: String );
       procedure SetCurrentForm( AValue: TCustomForm );
     public
       //    
       procedure StoreToRegistry( Form: TCustomForm; const RegKey, ValueName: string );
       //    
       procedure LoadFromRegistry( Form: TCustomForm; const RegKey, ValueName: string );
       // ====================================================================
       // ,    
       //     Control-
       // ControlClass    -  ,  TSplitter
       // AddRegValueName -    ValueName,  '_Splitters'
       procedure DefineProperties( var ControlClass: TComponentClass;
                                   var AddRegValueName: string ); dynamic; abstract;
       //   ,     
       //     '&'
       //   Abort,     
       procedure StoreString( Sender: TComponent; var Text: string ); dynamic; abstract;
       //      
       procedure LoadString( Sender: TComponent; const Text: String ); dynamic; abstract;
       //    
       procedure LoadOldString( const Text: String ); dynamic; abstract;
       // ====================================================================
       constructor Create( AOwner: TrBackground );
       property CurrentForm: TCustomForm read FCurrentForm write SetCurrentForm;
       property ControlClass: TComponentClass read FControlClass;
       property Owner: TrBackground read FOwner;
     end;

Type TBackgroundControlClass = class of TBackGroundControl;
procedure RegisterBackgroundControl( AValue: TBackGroundControlClass );

Var BackgroundControlClasses: TList = Nil;

//  
Type TBackgroundSplitter = class( TBackgroundControl )
     private
       function FindControl(SPL: TSplitter): TControl;
     public
       procedure DefineProperties( var ControlClass: TComponentClass;
                                   var AddRegValueName: string ); override;
       procedure StoreString( Sender: TComponent; var Text: string ); override;
       procedure LoadString( Sender: TComponent; const Text: String ); override;
       procedure LoadOldString( const Text: String ); override;
     end;


//  ToolBar
Type TBackgroundToolbar = class( TBackgroundControl )
     private
     public
       procedure DefineProperties( var ControlClass: TComponentClass;
                                   var AddRegValueName: string ); override;
       procedure StoreString( Sender: TComponent; var Text: string ); override;
       procedure LoadString( Sender: TComponent; const Text: String ); override;
       procedure LoadOldString( const Text: String ); override;
     end;



implementation

uses
  extdlgs, rConst, rUtils, rErrorUtils,
  {$IFDEF POLARIS_D6} RxStrUtils {$ELSE} StrUtils {$ENDIF};

type
  THack = class(TWinControl);
  THookOrder = (hoBeforeMsg, hoAfterMsg);

{ TControlHook }

  TControlHook = class(TObject)
  private
    FControl: TWinControl;
    FNewWndProc: Pointer;
    FPrevWndProc: Pointer;
    FNewClientWndProc: Pointer;
    FPrevClientWndProc: Pointer;
    FList: TList;
    FDestroying: Boolean;
    procedure SetWinControl(Value: TWinControl);
    procedure HookWndProc(var AMsg: TMessage);
    procedure HookClientWndProc(var AMsg: TMessage);
    procedure NotifyHooks(Order: THookOrder; var Msg: TMessage;
      var Handled: Boolean);
    procedure NotifyClientHooks(Order: THookOrder; var Msg: TMessage;
      var Handled: Boolean);
  public
    constructor Create;
    destructor Destroy; override;
    procedure HookControl;
    procedure UnhookControl;
    procedure AddHook(AHook: TrWindowHook);
    procedure RemoveHook(AHook: TrWindowHook);
    property WinControl: TWinControl read FControl write SetWinControl;
  end;

{ THookList }

  THookList = class(TList)
  private
    FHandle: HWnd;
    procedure WndProc(var Msg: TMessage);
  public
    constructor Create;
    destructor Destroy; override;
    function FindControlHook(AControl: TWinControl): TControlHook;
    function GetControlHook(AControl: TWinControl): TControlHook;
    property Handle: HWnd read FHandle;
  end;

var
  HookList: THookList;

//====================================
function GetHookList: THookList;
begin
  if HookList = nil then HookList := THookList.Create;
  Result := HookList;
end;

procedure DropHookList; far;
begin
  HookList.Free;
  HookList := nil;
end;

{function StreamToString(Stream: TStream): String;
var
  Size: Integer;
  p: PChar;
begin
  Size := Stream.Size;
  SetLength(Result, Size * 2);
  GetMem(p, Size);

  Stream.Position := 0;
  Stream.Read(p^, Size);

  BinToHex(p, @Result[1], Size);

  FreeMem(p, Size);
end;

procedure StringToStream(const s: String; Stream: TStream);
var
  Size: Integer;
  p: PChar;
begin
  Size := Length(s) div 2;
  GetMem(p, Size);

  HexToBin(@s[1], p, Size * 2);

  Stream.Position := 0;
  Stream.Write(p^, Size);

  FreeMem(p, Size);
end;}

{ TControlHook }

constructor TControlHook.Create;
begin
  inherited Create;
  FList := TList.Create;
  FNewWndProc := {$IFDEF POLARIS_D6}Classes.{$ENDIF}MakeObjectInstance(HookWndProc);
  FPrevWndProc := nil;
  FNewClientWndProc := {$IFDEF POLARIS_D6}Classes.{$ENDIF}MakeObjectInstance(HookClientWndProc);
  FPrevClientWndProc := nil;
  FControl := nil;
end;

destructor TControlHook.Destroy;
begin
  FDestroying := True;
  if Assigned(HookList) then
    if HookList.IndexOf(Self) >= 0 then HookList.Remove(Self);
  while FList.Count > 0 do RemoveHook(TrWindowHook(FList.Last));
  FControl := nil;
  FList.Free;
  {$IFDEF POLARIS_D6}Classes.{$ENDIF}FreeObjectInstance(FNewWndProc);
  FNewWndProc := nil;
  {$IFDEF POLARIS_D6}Classes.{$ENDIF}FreeObjectInstance(FNewClientWndProc);
  FNewClientWndProc := nil;
  inherited Destroy;
end;

procedure TControlHook.AddHook(AHook: TrWindowHook);
begin
  if FList.IndexOf(AHook) < 0 then begin
    FList.Add(AHook);
    AHook.FControlHook := Self;
    WinControl := AHook.FControl;
  end;
  HookControl;
end;

procedure TControlHook.RemoveHook(AHook: TrWindowHook);
begin
  AHook.FControlHook := nil;
  FList.Remove(AHook);
  if FList.Count = 0 then UnhookControl;
end;

procedure TControlHook.NotifyHooks(Order: THookOrder; var Msg: TMessage;
  var Handled: Boolean);
var
  I: Integer;
begin
  if (FList.Count > 0) and Assigned(FControl) and
    not FDestroying then begin
    for I := FList.Count - 1 downto 0 do begin
      if not (csDestroying in FControl.ComponentState)
      then try
        if Order = hoBeforeMsg then
          TrWindowHook(FList[I]).DoBeforeMessage(Msg, Handled)
        else if Order = hoAfterMsg then
          TrWindowHook(FList[I]).DoAfterMessage(Msg, Handled);
      except
        Application.HandleException(Self);
      end;
      if Handled then Break;
    end;
  end;
end;

procedure TControlHook.NotifyClientHooks(Order: THookOrder; var Msg: TMessage;
  var Handled: Boolean);
var
  I: Integer;
begin
  if (FList.Count > 0) and Assigned(FControl) and
    not (FDestroying or (csDestroying in FControl.ComponentState)) then
    for I := FList.Count - 1 downto 0 do begin
      try
        if Order = hoBeforeMsg then
          TrWindowHook(FList[I]).DoClientBeforeMessage(Msg, Handled)
        else if Order = hoAfterMsg then
          TrWindowHook(FList[I]).DoClientAfterMessage(Msg, Handled);
      except
        Application.HandleException(Self);
      end;
      if Handled then Break;
    end;
end;

procedure TControlHook.HookControl;
var
  P: Pointer;
  Frm: THackForm;
begin
  if Assigned(FControl) and not ((csDesigning in FControl.ComponentState) or
    (csDestroying in FControl.ComponentState) or FDestroying) then
  begin
    FControl.HandleNeeded;
    P := Pointer(GetWindowLong(FControl.Handle, GWL_WNDPROC));
    if (P <> FNewWndProc) then begin
      FPrevWndProc := P;
      SetWindowLong(FControl.Handle, GWL_WNDPROC, LongInt(FNewWndProc));
    end;
    if (FControl is TCustomForm) and (THackForm(FControl).FormStyle = fsMDIForm) then begin
      Frm := THackForm(FControl);
      P := Pointer(GetWindowLong(Frm.ClientHandle, GWL_WNDPROC));
      if (P <> FNewClientWndProc) then begin
        FPrevClientWndProc := P;
        SetWindowLong(Frm.ClientHandle, GWL_WNDPROC, LongInt(FNewClientWndProc));
      end;
    end;
  end;
end;

procedure TControlHook.UnhookControl;
var
  Frm: THackForm;
begin
  if Assigned(FControl) then begin
    if Assigned(FPrevWndProc) and FControl.HandleAllocated
    then begin
      if (Pointer(GetWindowLong(FControl.Handle, GWL_WNDPROC)) = FNewWndProc)
      then SetWindowLong(FControl.Handle, GWL_WNDPROC, LongInt(FPrevWndProc));
      if Assigned(FPrevClientWndProc) and (FControl is TCustomForm) and (THackForm(FControl).FormStyle = fsMDIForm)
      then begin
        Frm := THackForm(FControl);
        if (Pointer(GetWindowLong(Frm.ClientHandle, GWL_WNDPROC)) = FNewClientWndProc)
        then SetWindowLong(Frm.ClientHandle, GWL_WNDPROC, LongInt(FPrevClientWndProc));
      end;
    end;
  end;
  FPrevWndProc := nil;
  FPrevClientWndProc := nil;
end;

procedure TControlHook.HookWndProc(var AMsg: TMessage);
var
  Handled: Boolean;
begin
  Handled := False;
  if Assigned(FControl) then begin
    if (AMsg.Msg <> WM_QUIT) then NotifyHooks(hoBeforeMsg, AMsg, Handled);
    with AMsg do begin
      if (not Handled) or (Msg = WM_DESTROY) then
        try
          if AMsg.Result = 0 then
            if Assigned(FPrevWndProc) then
              Result := CallWindowProc(FPrevWndProc, FControl.Handle, Msg,
                WParam, LParam)
            else
              Result := CallWindowProc(THack(FControl).DefWndProc,
                FControl.Handle, Msg, WParam, LParam);
        finally
          NotifyHooks(hoAfterMsg, AMsg, Handled);
        end;
      if Msg = WM_DESTROY then begin
        UnhookControl;
        if Assigned(HookList) and not (FDestroying or
          (csDestroying in FControl.ComponentState)) then
          PostMessage(HookList.FHandle, CM_RECREATEWINDOW, 0, Longint(Self));
      end;
    end;
  end;
end;

procedure TControlHook.HookClientWndProc(var AMsg: TMessage);
var
  Handled: Boolean;
begin
  Handled := False;
  if Assigned(FControl) then begin
    if (AMsg.Msg <> WM_QUIT) then NotifyClientHooks(hoBeforeMsg, AMsg, Handled);
    with AMsg do begin
      if (not Handled) or (Msg = WM_DESTROY) then
        try
          if AMsg.Result = 0 then
            if Assigned(FPrevClientWndProc) then
              Result := CallWindowProc(FPrevClientWndProc, FControl.Handle, Msg,
                WParam, LParam)
            else
              Result := CallWindowProc(THack(FControl).DefWndProc,
                FControl.Handle, Msg, WParam, LParam);
        finally
          NotifyClientHooks(hoAfterMsg, AMsg, Handled);
        end;
    end;
  end;
end;

procedure TControlHook.SetWinControl(Value: TWinControl);
begin
  if Value <> FControl then begin
    UnhookControl;
    FControl := Value;
    if FList.Count > 0 then HookControl;
  end;
end;

{ THookList }

constructor THookList.Create;
begin
  inherited Create;
  FHandle := {$IFDEF POLARIS_D6}Classes.{$ENDIF}AllocateHWnd(WndProc);
end;

destructor THookList.Destroy;
begin
  while Count > 0 do TControlHook(Last).Free;
  {$IFDEF POLARIS_D6}Classes.{$ENDIF}DeallocateHWnd(FHandle);
  inherited Destroy;
end;

procedure THookList.WndProc(var Msg: TMessage);
var
  Hook: TControlHook;
begin
  try
    with Msg do begin
      if Msg = CM_RECREATEWINDOW then begin
        Hook := TControlHook(LParam);
        if (Hook <> nil) and (IndexOf(Hook) >= 0) then
          Hook.HookControl;
      end
      else if Msg = CM_DESTROYHOOK then begin
        Hook := TControlHook(LParam);
        if Assigned(Hook) and (IndexOf(Hook) >= 0) and
          (Hook.FList.Count = 0) then Hook.Free;
      end
      else Result := DefWindowProc(FHandle, Msg, wParam, lParam);
    end;
  except
    Application.HandleException(Self);
  end;
end;

function THookList.FindControlHook(AControl: TWinControl): TControlHook;
var
  I: Integer;
begin
  if Assigned(AControl) then
    for I := 0 to Count - 1 do
      if (TControlHook(Items[I]).WinControl = AControl) then begin
        Result := TControlHook(Items[I]);
        Exit;
      end;
  Result := nil;
end;

function THookList.GetControlHook(AControl: TWinControl): TControlHook;
begin
  Result := FindControlHook(AControl);
  if Result = nil then begin
    Result := TControlHook.Create;
    try
      Add(Result);
      Result.WinControl := AControl;
    except
      Result.Free;
      raise;
    end;
  end;
end;

{ TrWindowHook }

constructor TrWindowHook.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FActive := True;
end;

destructor TrWindowHook.Destroy;
begin
  Active := False;
  WinControl := nil;
  inherited Destroy;
end;

procedure TrWindowHook.SetActive(Value: Boolean);
begin
  if FActive <> Value then
    if Value then HookControl else UnhookControl;
end;

function TrWindowHook.GetHookHandle: HWnd;
begin
  if Assigned(HookList) then Result := HookList.Handle
  else
    Result := INVALID_HANDLE_VALUE;
end;

procedure TrWindowHook.HookControl;
begin
  if Assigned(FControl) and not (csDestroying in ComponentState) then
    GetHookList.GetControlHook(FControl).AddHook(Self);
  FActive := True;
end;

function TrWindowHook.DoUnhookControl: Pointer;
begin
  Result := FControlHook;
  if Result <> nil then TControlHook(Result).RemoveHook(Self);
  FActive := False;
end;

procedure TrWindowHook.UnhookControl;
begin
  DoUnhookControl;
  FActive := False;
end;

function TrWindowHook.NotIsForm: Boolean;
begin
  Result := (WinControl <> nil) and not (WinControl is TCustomForm);
end;

function TrWindowHook.IsForm: Boolean;
begin
  Result := (WinControl <> nil) and ((WinControl = Owner) and
    (Owner is TCustomForm));
end;

procedure TrWindowHook.ReadForm(Reader: TReader);
begin
  if Reader.ReadBoolean then
    if Owner is TCustomForm then WinControl := TWinControl(Owner);
end;

procedure TrWindowHook.WriteForm(Writer: TWriter);
begin
  Writer.WriteBoolean(IsForm);
end;

procedure TrWindowHook.DefineProperties(Filer: TFiler);
  function DoWrite: Boolean;
  begin
    if Assigned(Filer.Ancestor) then
      Result := IsForm <> TrWindowHook(Filer.Ancestor).IsForm
    else Result := IsForm;
  end;
begin
  inherited DefineProperties(Filer);
  Filer.DefineProperty('IsForm', ReadForm, WriteForm, DoWrite);
end;

function TrWindowHook.GetWinControl: TWinControl;
begin
  if Assigned(FControlHook) then Result := TControlHook(FControlHook).WinControl
  else Result := FControl;
end;

procedure TrWindowHook.DoAfterMessage(var Msg: TMessage; var Handled: Boolean);
begin
  if Assigned(FAfterMessage) then FAfterMessage(Self, Msg, Handled);
end;

procedure TrWindowHook.DoBeforeMessage(var Msg: TMessage; var Handled: Boolean);
begin
  if Assigned(FBeforeMessage) then FBeforeMessage(Self, Msg, Handled);
end;

procedure TrWindowHook.DoClientAfterMessage(var Msg: TMessage; var Handled: Boolean);
begin
  if Assigned(FClientAfterMessage) then FClientAfterMessage(Self, Msg, Handled);
end;

procedure TrWindowHook.DoClientBeforeMessage(var Msg: TMessage; var Handled: Boolean);
begin
  if Assigned(FClientBeforeMessage) then FClientBeforeMessage(Self, Msg, Handled);
end;

procedure TrWindowHook.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (AComponent = WinControl) and (Operation = opRemove) then
    WinControl := nil
  else if (Operation = opRemove) and ((Owner = AComponent) or
    (Owner = nil)) then WinControl := nil;
end;

procedure TrWindowHook.SetWinControl(Value: TWinControl);
var
  SaveActive: Boolean;
  Hook: TControlHook;
begin
  if Value <> WinControl then begin
    SaveActive := FActive;
    Hook := TControlHook(DoUnhookControl);
    FControl := Value;
    if Value <> nil then Value.FreeNotification(Self);
    if Assigned(Hook) and (Hook.FList.Count = 0) and Assigned(HookList) then
      PostMessage(HookList.Handle, CM_DESTROYHOOK, 0, Longint(Hook));
    if SaveActive then HookControl;
  end;
end;
//====================================

{$R *.DFM}

constructor TrBackgroundDlg.Create(AOwner: TComponent);
var FullName: String;
    S: String;
    i,n: integer;
    fExt: string;
    Result: integer;
    SearchRec: TSearchRec;
begin
  inherited Create(AOwner);
  FGlyph := TPicture.Create;
  FDefBkgnd := TrBackgroundValue.Create( Nil );
  //S := ExtractWord(2,GraphicFilter(TGraphic),['|']);
  HelpFile := srHelpFile;
  SizeBox.Visible := True;
  ComboBox2.ItemIndex := 0;
  SetColor(clBtnFace);
  FileName := crEmptyStr;
  ViewMode := rvmNone;

  FullName := ExtractFileDir(ExpandFileName(Application.ExeName))+'\';
  with FileComboBox do begin
    Items.Clear;
    Items.AddObject(srNone, nil);
    ItemIndex := 0;
    if Trim(S) = crEmptyStr then S := '*.bmp';
    N := WordCount(S,[';']);
    for I := 1 to N do begin
      fExt := ExtractWord(I,S,[';']);
      Result := FindFirst(ExtractFilePath(Application.ExeName)+fExt, faArchive, SearchRec);
      while Result = 0 do
        begin
         Items.AddObject(ExtractFileName(SearchRec.Name),
                  TObject(StrNew(PChar(FullName+ExtractFileName(SearchRec.Name)))));
         Result := FindNext(SearchRec);
        end;
      FindClose(SearchRec);
    end;
  end;


end;

procedure TrBackgroundDlg.SetDefBkgnd(const Value: TrBackgroundValue);
begin
  FDefBkgnd.Assign( Value );
end;



procedure TrBackgroundDlg.ColorBtnClick(Sender: TObject);
var
  ColorDialog1: TColorDialog;
begin
  ColorDialog1 := TColorDialog.Create(Self);
  try
    ColorDialog1.Color := Color;
    if ColorDialog1.Execute then begin
      SetColor(ColorDialog1.Color);
      Repaint;
    end;
  finally
    ColorDialog1.Free;
  end;
end;

procedure TrBackgroundDlg.FileComboBoxChange(Sender: TObject);
begin
  with FileComboBox do
   if Items.Objects[ItemIndex] <> nil then
     SetFileName(StrPas(PChar(Items.Objects[ItemIndex])))
   else SetFileName(crEmptyStr);
end;

procedure TrBackgroundDlg.FileBtnClick(Sender: TObject);
Var D: TOpenPictureDialog;
begin
  D := TOpenPictureDialog.Create(Self);
  try
    D.Options := D.Options+[ofNoChangeDir];
    D.Title := srBkgndPic;
    D.InitialDir := ExtractFilePath(FileName);
    D.FileName   := FileName;
    if D.Execute then
      FileName := D.FileName;
  finally
    D.Free;
  end;
end;


procedure TrBackgroundDlg.RestoreBtnClick(Sender: TObject);
begin
  AssignFrom(FDefBkgnd);
end;

procedure TrBackgroundDlg.ComboBox2Change(Sender: TObject);
begin
  if GetViewMode <> TrViewMode(ComboBox2.ItemIndex+1)
       then SetViewMode(TrViewMode(ComboBox2.ItemIndex+1));
  PaintBox1.Refresh;     
end;

procedure TrBackgroundDlg.DoChangeSizePos;
begin
  if PosBox.Visible or SizeBox.Visible
  then ClientHeight := PosBox.Top+PosBox.Height+8+Bevel1.Height+Panel1.Height
  else ClientHeight := PosBox.Top+Bevel1.Height+Panel1.Height
end;

procedure TrBackgroundDlg.SetmaxValue(Index: Integer; Value: Integer);
begin
  if Value > 0 then
  case Index of
  0: begin
       RLeft.MaxValue  := Value;
       RLeft.MaxLength := Length(IntToStr(Value));
     end;
  1: begin
       RTop.MaxValue   := Value;
       RTop.MaxLength := Length(IntToStr(Value));
     end;
  2: begin
       RWidth.MaxValue := Value;
       RWidth.MaxLength := Length(IntToStr(Value));
     end;
  3: begin
       RHeight.MaxValue:= Value;
       RHeight.MaxLength := Length(IntToStr(Value));
     end;
  end;        
end;

function TrBackgroundDlg.FormHelp(Command: Word; Data: Integer;
  var CallHelp: Boolean): Boolean;
begin
  CallHelp := not ((Command = HELP_CONTEXTPOPUP) and (Data = HelpContext));
  Result := True;
end;

procedure TrBackgroundDlg.RHeightChange(Sender: TObject);
var
  X1, X0, M: Integer;
begin
  if TRxSpinEdit(Sender).Text = crEmptyStr then Exit;
  X1 := TRxSpinEdit(Sender).AsInteger;
  M  := Trunc(TRxSpinEdit(Sender).MaxValue);
  if Sender = RHeight then X0 := YPos
  else if Sender = RWidth then X0 := XPos
  else if Sender = RLeft then X0 := FormWidth
  else if Sender = RTop then X0 := FormHeight
  else X0 := 0;
  if (X1+X0) > M
  then begin
    X0 := M-X0;
    if (X0 < 0) or (Sender = RLeft) or (Sender = RTop) then X0 := 0;
    TRxSpinEdit(Sender).Value := X0;
    TRxSpinEdit(Sender).SelectAll;
    raise EAnyError.CreateFmt(srBkgndHeit,[X1]);
  end;
  case RPosition.ItemIndex of
  1: if ((RWidth.Value - RLeft.Value) <> RLeft.MaxValue) and
        ((RHeight.Value - RTop.Value) <> RTop.MaxValue)
     then SetFormState(rvwmNone);
  2: begin
        X0 := Round(RLeft.MaxValue - Round(RLeft.MaxValue / 2));
        X1 := Round(RTop.MaxValue - Round(RTop.MaxValue / 2));
        if (RLeft.Value <> X0) and (RTop.Value <> X1) then  SetFormState(rvwmNone);
     end;
  end;
end;



{TrBackground}

constructor TrBackground.Create(AOwner: TComponent);
Var i: integer;
begin
  BackgroundLoaded := false;
  FAlignToolBars := true;
  FUpdate := False;
  FValueName := bvntName;
  inherited Create(AOwner);
  FBackgroundControls := TList.Create;
  for i := 0 to BackgroundControlClasses.Count - 1 do
    FBackgroundControls.Add(
      TBackgroundControlClass( BackgroundControlClasses[i] ).Create( Self ) );


  FValue := TrBackGroundValue.Create( Self );
  FHook := TrWindowHook.Create(Self);
  FHook.BeforeMessage := BeforeHook;
  FHook.AfterMessage := AfterHook;
  FHook.ClientBeforeMessage := ClientBeforeHook;
  FHook.ClientAfterMessage := ClientAfterHook;
  FCloseEvent  := nil;
  FCreateEvent := nil;
  FOnHideEvent := nil;
  FPrevRect := Rect(0,0,0,0);
  FRedrawBkg := True;
  FAutoSave := False;
  FAutoLoad := True;
  FSaveSplitterPos := True;
  FSaveToolBar := True;
  FPosSizeEnabled := [rpsPosEnabled, rpsSizeEnabled];
  FillChar(FMaxValue, SizeOf(FMaxValue), 0);
  FFileHistory := TStringList.Create;
  //FCurrentBkgnd := DefaultBackground;
  //FCurrentBkgnd.Color := TCustomForm(AOwner).Color;

  //Value.LoadfromForm( TCustomForm( AOwner ) );

  FProgramKey := crEmptyStr;
  FLastFileName := crEmptyStr;
  BindControl := TCustomForm(AOwner);
end;

destructor TrBackground.Destroy;
Var i: integer;
begin
  for i := 0 to FBackgroundControls.Count - 1 do
    TBackgroundControl( FBackgroundControls[i] ).Free;
  FBackgroundControls.Free;

  if (FBindControl <> nil) and not (csDestroying in FBindControl.ComponentState) then
    begin
      DestroyMessage(FBindControl);
    end;
  FFileHistory.Free;
  //FPicture.Free;
  FHook.Free;
  FValue.Free;
  inherited Destroy;
end;

procedure TrBackground.DestroyMessage(Sender: TObject);
begin
  if FAutoSave and (Sender <> nil) then Value.LoadfromForm( TCustomForm( Sender ) );
  InternalSave(Sender);
end;

procedure TrBackground.BeforeHook(Sender: TObject; var Msg: TMessage; var Handled: Boolean);
begin
  case Msg.Msg of
    WM_SHOWWINDOW: if TWMSHOWWINDOW(Msg).Show and not FUpdate then
                              FUpdate := TWMSHOWWINDOW(Msg).Show and not FUpdate;
    WM_ERASEBKGND: if (THackForm(FHook.WinControl).FormStyle <> fsMDIForm) and FRedrawBkg
                       then begin
                              _DrawBackGround( TWMEraseBkGnd(Msg).Dc );
                              Msg.Result := 1;
                            end;
    WM_PAINT: if IsWindowVisible(THackForm(FHook.WinControl).Handle)
                then THackForm(FHook.WinControl).Invalidate;
    //WM_SIZE: if BackgroundLoaded then PrepareCurrentToolBars;
  end;
end;

procedure TrBackground.AfterHook(Sender: TObject; var Msg: TMessage;
  var Handled: Boolean);
begin
  case Msg.Msg of
    WM_SIZE: if BackgroundLoaded then PrepareCurrentToolBars;
  end;
end;

procedure TrBackground.ClientAfterHook(Sender: TObject; var Msg: TMessage; var Handled: Boolean);
var
  R: TRect;
begin
  case Msg.Msg of
  WM_VSCROLL,
  WM_HSCROLL:
    if FUpdate and Assigned(FHook.WinControl)
    then begin
      GetWindowRect(THackForm(FHook.WinControl).ClientHandle, R);
      FPrevRect := R;
      InvalidateRect(THackForm(FHook.WinControl).ClientHandle, @R, TRUE);
      RedrawWindow(THackForm(FHook.WinControl).ClientHandle, nil, 0,
            RDW_ERASENOW or RDW_UPDATENOW or RDW_FRAME or RDW_NOCHILDREN or RDW_INVALIDATE);
    end;
  $3F:
    if FUpdate and Assigned(FHook.WinControl)
    then begin
      GetWindowRect(THackForm(FHook.WinControl).ClientHandle, R);
      if not CompareMem(@R,@FPrevRect, SizeOf(TRect)) then begin
        FPrevRect := R;
        InvalidateRect(THackForm(FHook.WinControl).ClientHandle, @R, TRUE);
        RedrawWindow(THackForm(FHook.WinControl).ClientHandle, nil, 0,
              RDW_ERASENOW or RDW_UPDATENOW or RDW_FRAME or RDW_NOCHILDREN or RDW_INVALIDATE);
      end;
    end;
  end;
end;

procedure TrBackground.ClientBeforeHook(Sender: TObject; var Msg: TMessage; var Handled: Boolean);
begin
  case Msg.Msg of
  WM_ERASEBKGND: if (THackForm(FHook.WinControl).FormStyle = fsMDIForm) and FRedrawBkg
  then begin       
    _DrawBackGround( TWMEraseBkGnd(Msg).Dc );
    Msg.Result := 1;
  end;
  end;
end;

procedure TrBackground.SetRedrawBkg(Value: Boolean);
var
  TF: TCustomForm;
begin
  TF := BindControl;
  if FRedrawBkg <> Value then begin
    FRedrawBkg := Value;
    if (FHook.WinControl = TF) and not Value
    then begin
      FBindControl := TF;
      FHook.WinControl := nil;
    end;
    if (FBindControl = TF) and Value
    then begin
      FBindControl := nil;
      FHook.WinControl := TF;
    end;
    if FRedrawBkg and
     FUpdate
    then RefreshForm;
  end;
end;

procedure TrBackground.DrawBackGround( Canvas: TCanvas );

  procedure ClearClient( Canvas: TCanvas; const R: TRect );
  begin
    Canvas.Brush.Color := FValue.Color;
    Canvas.Brush.Style := bsSolid;
    Canvas.Pen.Color := FValue.Color;
    Canvas.Pen.Style := psSolid;
    Canvas.Pen.Width := 1;
    Canvas.FillRect( R );
  end;

  procedure DrawTiled( Canvas: TCanvas; const R: TRect );
  var FollowX, FollowY: integer;
      R2: TRect;
  begin
    FollowX := R.Left;
    FollowY := R.Top;
    While FollowY < R.Bottom do
      begin
        R2.Left := FollowX;
        R2.Top := FollowY;
        R2.Right := R2.Left + Glyph.Width;
        R2.Bottom := R2.Top + Glyph.Height;
        Canvas.StretchDraw( R2, Glyph.Graphic );
        FollowX := FollowX + Glyph.Width;
        if FollowX > R.Right then
          begin
            FollowX := R.Left;
            FollowY := FollowY + Glyph.Height;
          end;
        Application.ProcessMessages;
      end;
  end;

  procedure DrawCenter( Canvas: TCanvas; const R: TRect );
  var
    dX, dY: integer;
    R2: TRect;
  begin

    dX := ((R.Right - R.Left) - Glyph.Width) div 2;
    dY := ((R.Bottom - R.Top) - Glyph.Height) div 2;

    R2 := R;
    R2.Left := R2.Left + dX;
    R2.Top := R2.Top + dY;
    R2.Right := R2.Right - dX;
    R2.Bottom := R2.Bottom - dY;

    Canvas.StretchDraw( R2, Glyph.Graphic );
  end;

  procedure DrawStretch( Canvas: TCanvas; const R: TRect );
  begin
    Canvas.StretchDraw( R, Glyph.Graphic );
  end;

var
  R, R2: TRect;
  B: TBitmap;
begin
  R := CurrentForm.ClientRect;

  if Glyph.Height = 0 then
    begin
      ClearClient( Canvas, R );
      Exit;
    end;

  B := TBitmap.Create;
  try
    B.Width := R.Right - R.Left;
    B.Height := R.Bottom - R.Top;
    R2.Left := 0;
    R2.Top := 0;
    R2.Right := B.Width;
    R2.Bottom := B.Height;

    ClearClient( B.Canvas, R2 );

    Case Value.ViewMode of
      rvmNone,
        rvmTile: DrawTiled( B.Canvas, R2 );
      rvmCenter:  DrawCenter( B.Canvas, R2 );
      rvmStretch: DrawStretch( B.Canvas, R2 );
    end;

    Canvas.CopyRect( R, B.Canvas, R2 );
  finally
    B.Free;
  end;
end;


procedure TrBackground.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and (AComponent = CurrentForm)
  then BindControl := nil;
end;

function TrBackground.GetBindControl: TCustomform;
begin
  if FRedrawBkg
  then Result := TCustomForm(FHook.WinControl)
  else Result := FBindControl;
end;

procedure TrBackground.SetBindControl(Value: TCustomForm);
var
  TF: TCustomForm;
begin
  TF := GetBindControl;
  if {TCustomForm(FHook.WinControl)}TF <> Value then begin
    if (Value <> nil) and not (Value is TCustomForm)
    then raise Exception.Create(srBkgndBind);
    if FRedrawBkg then begin
      if (TCustomForm(FHook.WinControl) <> nil) and not (csDesigning in ComponentState) then begin
        THackForm(FHook.WinControl).OnCreate := FCreateEvent;
        THackForm(FHook.WinControl).OnClose := FCloseEvent;
        THackForm(FHook.WinControl).OnHide := FOnHideEvent;
      end;
      FHook.WinControl := Value;
      FBindControl := nil;
    end
    else begin
      if (FBindControl <> nil) and not (csDesigning in ComponentState) then begin
        THackForm(FBindControl).OnCreate := FCreateEvent;
        THackForm(FBindControl).OnClose := FCloseEvent;
        THackForm(FBindControl).OnHide := FOnHideEvent;
      end;
      FBindControl := Value;
      FHook.WinControl := nil;
    end;
    if (Value <> nil) then begin
      if ({TCustomForm(FHook.WinControl)}Value <> nil) and not (csDesigning in ComponentState) then begin
        FCreateEvent := THackForm(Value).OnCreate;
        FCloseEvent := THackForm(Value).OnClose;
        FOnHideEvent := THackForm(Value).OnHide;
        THackForm(Value).OnCreate := OnFormCreate;
        THackForm(Value).OnClose := OnFormClose;
        THackForm(Value).OnHide := OnFormHide;
      end;
      MakeMax(Value);
      
      if LoadBackground( FValue )
        then begin
               ChangeSizePos;
               LoadOtherControl;
             end;
    end;
  end;
end;

procedure TrBackground.OnFormHide(Sender: TObject);
begin
  DestroyMessage(Sender);
  if Assigned(FOnHideEvent) then FOnHideEvent(Sender);
end;

procedure TrBackground.OnFormClose(Sender: TObject; var Action: TCloseAction);
begin
  DestroyMessage(Sender);
  if Assigned(FCloseEvent) then FCloseEvent(Sender, Action);
end;

procedure TrBackground.OnFormCreate(Sender: TObject);
begin
  if Assigned(FCreateEvent) then FCreateEvent(Sender);
  if LoadBackground(FValue) and (BindControl <> nil) then
    begin
      FValue.StoreToForm( BindControl );
      LoadOtherControl;
    end;
end;

procedure TrBackground.SetProgramKey(Value: String);
begin
  Value := Trim(Value);
  if Value <> FProgramKey then begin
    if (FProgramKey <> EmptyStr) and FAutoSave then SaveBackground;
    FProgramKey := Value;
    if LoadBackground(FValue)
    then begin
      ChangeSizePos;
      LoadOtherControl;
    end;
  end;
end;

function TrBackground.CurrentForm: THackForm;
begin
  Result := THackForm(BindControl)
end;

procedure TrBackground.RefreshForm;
begin
  if IsWindowVisible(CurrentForm.Handle) then RefreshWindow(BindControl,nil);
end;

procedure TrBackground.MakeMax(Sender: TObject);
var
  R: TRect;
begin
  if THackForm(Sender) <> nil then begin
    if not (csDesigning in ComponentState) then
      if THackForm(Sender).FormStyle <> fsMDIChild
      then SystemParametersInfo(SPI_GETWORKAREA, 0, @R, 0)
      else GetClientRect(Application.MainForm.ClientHandle, R)
    else R := Rect(0,0, Screen.Width, Screen.Height);
    FMaxValue[0] := R.Right-R.Left;
    FMaxValue[2] := R.Right-R.Left;
    FMaxValue[1] := R.Bottom-R.Top;
    FMaxValue[3] := R.Bottom-R.Top;
  end;
end;

function TrBackground.Execute: Boolean;
var
  Dlg: TrBackgroundDlg;
  Df: TrBackgroundValue;
  //R: TRect;
begin
  Result := False;
  Dlg := TrBackgroundDlg.Create(Application);
  try
    Df := TrBackgroundValue.Create( Self );
    try
      //SaveBackground;
      Value.LoadfromForm( CurrentForm );
      //LoadBackground(Df);
      Dlg.DefBkgnd.Assign( Value );

      Dlg.AssignFrom( Value );
      if Dlg.ShowModal = mrOK then
        begin
          Dlg.AssignToValue( Value );
          Value.StoreToForm( CurrentForm );
          PostMessage( CurrentForm.Handle, WM_ERASEBKGND, 0, 0 );
          RefreshForm;
          SaveBackground;
          Result := True;
        end;
    finally
      Df.Free;
    end;    
  finally
    Dlg.Free;
  end;
end;

function TrBackground.GetValueSaveName(Sender: TObject): String;
begin
  Result := crEmptyStr;
  if Assigned(Sender) then
    if FValueName = bvntName
    then Result := THackForm(Sender).Name
    else Result := THackForm(Sender).ClassName;
end;

procedure TrBackground.InternalSave(Sender: TObject);
Var i: integer;
begin
  if Sender = Nil then Exit;
  if csDesigning in ComponentState then Exit;
  if FProgramKey = crEmptyStr then Exit;
  if Assigned( FBeforeSaveBackground ) then FBeforeSaveBackground( Self );

  Value.StoreToRegistry( RegKey, GetValueSaveName(CurrentForm) );

  //     
  for i := 0 to FBackgroundControls.Count - 1 do
    TBackgroundControl( FBackgroundControls[i] ).StoreToRegistry(
       CurrentForm,
       FProgramKey, GetValueSaveName(CurrentForm) );

end;

procedure TrBackground.SaveBackground;
begin
  DestroyMessage( CurrentForm );
end;

type
  TSPL = class(TSplitter);

procedure TrBackground.LoadOtherControl;
Var i: integer;
begin
  for i := 0 to FBackgroundControls.Count - 1 do
    TBackgroundControl( FBackgroundControls[i] ).LoadFromRegistry(
       CurrentForm, FProgramKey, GetValueSaveName(CurrentForm) );
end;

function TrBackground.LoadBackground(Value: TrBackgroundValue): Boolean;
begin
  Result := False;
  if csLoading in ComponentState then Exit;
  if csDesigning in ComponentState then Exit;
  if CurrentForm = nil then Exit;
  if not FAutoLoad then Exit;
  Value.LoadFromRegistry( RegKey, GetValueSaveName(CurrentForm) );
  Value.StoreToForm( CurrentForm );
  //     ToolBar-
  //    ,    
  //      
  if RegKey <> crEmptyStr then
    begin
      BackgroundLoaded := true;
      LoadOtherControl;
      if not NewToolBarLoaded then LoadOldToolBar( CurrentForm, false, true );
      //PrepareCurrentToolBars;
      if Assigned( FAfterLoadBackground ) then FAfterLoadBackground( Self );
    end;
end;

function TrBackground.TestBackground(Value1, Value2: TrBackgroundValue): Boolean;
begin
  Result := Value1.TestValue( Value2 );
end;

function TrBackground.GetCurrentRect: TRect;
var
  X, Y, W, H: Integer;
begin
  Result := Rect(0,0,0,0);
  if (CurrentForm <> nil) then Result := CurrentForm.BoundsRect;
  if (CurrentForm <> nil) and ((rpsPosEnabled in FPosSizeEnabled)
  or (rpsSizeEnabled in FPosSizeEnabled))then begin
    if rpsSizeEnabled in FPosSizeEnabled
    then begin
      W := FValue.Width; H := FValue.Height;
    end
    else begin
      W := CurrentForm.Width; H := CurrentForm.Height;
    end;
    if rpsPosEnabled in FPosSizeEnabled
    then begin
      X := FValue.XPos; Y := FValue.YPos;
    end
    else begin
      X := CurrentForm.Left;  Y := CurrentForm.Top;
    end;
    MakeMax(CurrentForm);
    if FValue.State = rvwmCentered then begin
      X := (FMaxValue[0] - W) shr 1; Y := (FMaxValue[1] - H) shr 1;
    end;
    Result := Rect(X, Y, X+W, Y+H);
  end;
end;

procedure TrBackground.ChangeSizePos;
var
  X, Y,
  W, H: Integer;
  R: TRect;
  P: TWindowPlacement;
begin
  if not (csDesigning in ComponentState) and not (csLoading  in ComponentState) then begin
    R := GetCurrentRect;
    W := R.Right-R.Left;
    H := R.Bottom-R.Top;
    p.length := SizeOf(p);
    GetWindowPlacement(CurrentForm.Handle, @p);
    p.rcNormalPosition := R;
    P.flags := WPF_RESTORETOMAXIMIZED;
    P.showCmd := SW_RESTORE;
    case FValue.State of
    rvwmNone,
    rvwmCentered : begin
      if FValue.State = rvwmCentered then begin
        X := (FMaxValue[0] - W) div 2;
        Y := (FMaxValue[1] - H) div 2;
        p.rcNormalPosition := Rect(X,Y, X+W,Y+H);
      end;
      if FUpdate then SetWindowPlacement(CurrentForm.Handle, @p);
    end;
    rvwmMaximized: if CurrentForm.WindowState <> wsMaximized then begin
        P.showCmd := SW_MAXIMIZE;
        if FUpdate then SetWindowPlacement(CurrentForm.Handle, @p);
      end;
    end;
    if FUpdate then RefreshForm;
  end;
end;

procedure TrBackground.Loaded;
begin
  inherited Loaded;
  if LoadBackground(FValue) then
    begin
      ChangeSizePos;
      LoadOtherControl;
    end;
end;

function TrBackground.IsNotDefaultRegKey: Boolean;
begin
  Result := Trim(FProgramKey) <> crEmptyStr;
end;

function TrBackground.IsNotDefaultFileName: Boolean;
begin
  Result := false;
end;

function GetNormalSize( F: TCustomForm ): TRect;
Var R: TWindowPlacement;
begin
  Result.Left := 0;
  Result.Top := 0;
  Result.Bottom := 0;
  Result.Right := 0;
  if GetWindowPlacement( F.Handle, @R ) then
    begin
      Result.Left := R.rcNormalPosition.Left;
      Result.Top := R.rcNormalPosition.Top;
      Result.Right := R.rcNormalPosition.Right;
      Result.Bottom := R.rcNormalPosition.Bottom;
    end;
end;

function TrBackground.GetColor: TColor;
begin
  if Value <> Nil then Result := Value.Color else Result := clBtnFace;
end;

function TrBackground.GetViewMode: TrViewMode;
begin
  if Value <> Nil then Result := Value.ViewMode else Result := rvmNone;
end;

procedure TrBackground.SetColor(AValue: TColor);
begin
  if Value = Nil then Exit;
  Value.LoadfromForm( CurrentForm );
  Value.Color := AValue;
  Value.StoreToForm( CurrentForm );
end;


procedure TrBackground.SetViewMode(AValue: TrViewMode);
begin
  if Value = Nil then Exit;
  Value.LoadfromForm( CurrentForm );
  Value.ViewMode := AValue;
  Value.StoreToForm( CurrentForm );
end;

function TrBackground.GetGlyph: TPicture;
begin
  if Value <> Nil then Result := FValue.Glyph else Result := Nil;
end;

function TrBackground.GetHeight: integer;
begin
  if Value <> Nil then Result := Value.Height else Result := 100;
end;

function TrBackground.GetState: TrViewWindowMode;
begin
  if Value <> Nil then Result := Value.State else Result := rvwmNone;
end;

function TrBackground.GetWidth: integer;
begin
  if Value <> Nil then Result := Value.Width else Result := 100;
end;

function TrBackground.GetXPos: integer;
begin
  if Value <> Nil then Result := Value.XPos else Result := 100;
end;

function TrBackground.GetYPos: integer;
begin
  if Value <> Nil then Result := Value.YPos else Result := 100;
end;

procedure TrBackground.SetHeight(AValue: integer);
begin
  if Value = Nil then Exit;
  //Value.LoadfromForm( CurrentForm );
  Value.Height := AValue;
  //Value.StoreToForm( CurrentForm );
end;

procedure TrBackground.SetState(AValue: TrViewWindowMode);
begin
  if Value = Nil then Exit;
  //Value.LoadfromForm( CurrentForm );
  Value.State := AValue;
  //Value.StoreToForm( CurrentForm );
end;

procedure TrBackground.SetWidth(AValue: integer);
begin
  if Value = Nil then Exit;
  //Value.LoadfromForm( CurrentForm );
  Value.Width := AValue;
  //Value.StoreToForm( CurrentForm );
end;

procedure TrBackground.SetXPos(AValue: integer);
begin
  if Value = Nil then Exit;
  //Value.LoadfromForm( CurrentForm );
  Value.XPos := AValue;
  //Value.StoreToForm( CurrentForm );
end;

procedure TrBackground.SetYPos(AValue: integer);
begin
  if Value = Nil then Exit;
  //Value.LoadfromForm( CurrentForm );
  Value.YPos := AValue;
  //Value.StoreToForm( CurrentForm );
end;

procedure TrBackground.SetAutoLoad(AValue: Boolean);
begin
  FAutoLoad := AValue;
end;

procedure TrBackground.SetFileName(const AValue: TGraphicFileName);
begin
  if Value = Nil then Exit;
  Value.LoadFromForm( CurrentForm );
  Value.FileName := AValue;
  Value.StoreToForm( CurrentForm );
end;

function TrBackground.LoadOldToolBar( Form: TCustomForm; DeleteOldValue, StoreNewValue: boolean ): boolean;
Var i: integer;
    Reg: TRegistry;
    r: HKEY;
    S: String;
    OwnList: TStringList;

  function Execute_ToolBarParent( const S: String ): boolean;
  Var R: String;
      i: integer;
      C: TControl;
      RLeft, RHeight, RTop, RWidth, RVisible: integer;
  begin
    Result := false;
    R := Reg.ReadString(GetValueSaveName(Form) + S );
    if R = crEmptyStr then Exit;
    for i := 1 to (WordCount( R, [';'] ) - 1) div 6 do
      begin
        C := TControl( Form.FindComponent( ExtractWord( (i-1)*6 + 2, R, [';'] ) ) );
        if C = Nil then continue;

        RLeft := StrToIntDef( ExtractWord( (i-1)*6 + 3, R, [';'] ), -1 );
        RTop := StrToIntDef( ExtractWord( (i-1)*6 + 4, R, [';'] ), -1 );
        RHeight := StrToIntDef( ExtractWord( (i-1)*6 + 5, R, [';'] ), -1 );
        RWidth := StrToIntDef( ExtractWord( (i-1)*6 + 6, R, [';'] ), -1 );
        RVisible := StrToIntDef( ExtractWord( (i-1)*6 + 7, R, [';'] ), -1 );

        if RTop < 0 then continue;
        if RLeft < 0 then continue;
        if RHeight < 0 then continue;
        if RWidth < 0 then continue;
        if RVisible < 0 then continue;

        C.Left := RLeft;
        C.Top := RTop;
        C.Height := RHeight;
        C.Width := RWidth;
        C.Visible := Boolean( RVisible );

        Result := true;

      end;
    if DeleteOldValue then Reg.DeleteValue( GetValueSaveName(Form) + S );
  end;

begin
  Result := false;
  Reg := TRegistry.Create;
  try
    SeparateRegKey(RegKey, r, S);
    Reg.RootKey := r;
    if not Reg.OpenKey(S, False) then Exit;
    S := Reg.ReadString( GetValueSaveName( Form ) + srBkgndTool );
    if S = crEmptyStr then Exit;
    OwnList := TStringList.Create;
    try
      OwnList.CommaText := S;
      for i := 0 to OwnList.Count - 1 do
        if Execute_ToolBarParent( OwnList[i] ) then
          Result := true;
    finally
      OwnList.Free;
    end;

  finally
    Reg.Free;
  end;

  if StoreNewValue and Result then Self.InternalSave( Form );

end;

procedure TrBackground.SetAfterLoadBackground(AValue: TNotifyEvent);
begin
  FAfterLoadBackground := AValue;
end;

procedure TrBackground.SetBeforeSaveBackground(AValue: TNotifyEvent);
begin
  FBeforeSaveBackground := AValue;
end;

procedure TrBackground._DrawBackground(DC: HDC);
Var C: TCanvas;
    SaveIndex: integer;
    MyDC: HDC;
begin
  if DC = 0 then MyDC := CurrentForm.Canvas.Handle else MyDC := DC;
  C := TCanvas.Create;
  try
    SaveIndex := SaveDC(MyDC);
    try
      C.Handle := MyDC;
      DrawBackground( C );
    finally
      C.Handle := 0;
      RestoreDC(MyDC, SaveIndex)
     end;
  finally
    C.Free;
  end;
end;

function TrBackground.GetValue: TrBackgroundValue;
begin
  Result := FValue;
end;




{ TrBackgroundValue }

constructor TrBackgroundValue.Create( AOwner: TComponent );
begin
  FGlyph := TPicture.Create;
  FOwner := AOwner;
  FColor := clBtnFace;
  FViewMode := rvmNone;
  FXPos := -1;
  FYPos := -1;
  FWidth := -1;
  FHeight := -1;
  FState := rvwmNone;
end;

destructor TrBackgroundValue.Destroy;
begin
  FGlyph.Free;
  inherited;
end;

function TrBackgroundValue.IsCenteredState(Form: TCustomForm): Boolean;
begin
  MakeMax( Form );
  if (THackForm(Form).Left = (FMaxValue[0] - THackForm(Form).Width) shr 1)
     and (THackForm(Form).Top = (FMaxValue[1] - THackForm(Form).Height) shr 1)
        then Result := true else Result := false;
end;

procedure TrBackgroundValue.LoadfromForm(Form: TCustomForm);
Var R: TRect;
begin
  if Form.WindowState = wsMaximized then
    begin
      FState := rvwmMaximized;
      R := GetNormalSize( Form );
      FXPos := R.Left;
      FYPos := R.Top;
      FWidth := R.Right - R.Left;
      FHeight := R.Bottom - R.Top;
    end else
    begin
      FXPos := Form.Left;
      FYPos := Form.Top;
      FWidth := Form.Width;
      FHeight := Form.Height;
      if IsCenteredState( Form ) then FState := rvwmCentered else FState := rvwmNone;
    end;
  Color := Form.Color;
end;

procedure TrBackgroundValue.StoreToForm(Form: TCustomForm);
  function MaximizedChildren(aMDIParentForm: TForm): Boolean;
  var
    I: Integer;
  begin
    for I := 0 to aMDIParentForm.MDIChildCount - 1 do
      if aMDIParentForm.MDIChildren[I].WindowState = wsMaximized then begin
        Result := True;
        Exit;
      end;
    Result := False;
  end;

  function AnalyzeState(aState: TrViewWindowMode; aForm: TCustomForm): TrViewWindowMode;
  var
    MDIForm: TForm;
  begin
    Result := aState; // 
    if (aForm is TForm)and(TForm(aForm).FormStyle = fsMDIChild)then begin
      MDIForm := Application.MainForm; // FormStyle = fsMDIForm      ??
      if Assigned(MDIForm)and(MDIForm.FormStyle = fsMDIForm)and(MDIForm.MDIChildCount > 1) then begin // > 1, ..     Form
        if (MaximizedChildren(MDIForm)) then Result := rvwmMaximized
        else Result := rvwmNone;
      end;
    end;
  end;
begin
  Form.Color := Color;
  if FXPos <> -1 then begin
    FState := AnalyzeState(FState, Form); //    MDIChild- ,   .
                                          //  ,     ,    
                                          //   
    Case FState of
      rvwmNone: begin
        Form.Left := FXPos;
        Form.Top := FYPos;
        if FHeight <> -1 then Form.Height := FHeight;
        if FWidth <> -1 then Form.Width := FWidth;
        //Form.SetBounds(FXPos, FYPos, FWidth, FHeight);
        //makeMax( Form );
        Form.WindowState := wsNormal;
      end;

      rvwmCentered: begin
        Form.WindowState := wsNormal;
        makeMax( Form );
        THackForm(Form).Left := (FMaxValue[0] - FWidth) div 2;
        THackForm(Form).Top  := (FMaxValue[1] - FHeight) div 2;
        if THackForm(Form).Left < 0 then THackForm(Form).Left := 0;
        if THackForm(Form).Top < 0 then THackForm(Form).Top  := 0;
        if (THackForm(Form).Left+THackForm(Form).Width) > FMaxValue[0]
          then THackForm(Form).Width := FMaxValue[0]-THackForm(Form).Left;
        if (THackForm(Form).Top+THackForm(Form).Height) > FMaxValue[1]
          then THackForm(Form).Height := FMaxValue[1]-THackForm(Form).Top;
      end;
      
      rvwmMaximized: begin
        Form.WindowState := wsMaximized;
      end;
    end;
  end;
  if FOwner <> Nil then with TrBackground( FOwner ) do
    begin
      _DrawBackGround( 0 );
    end;
end;


procedure TrBackgroundValue.SetColor(AValue: TColor);
begin
  FColor := AValue;
end;

procedure TrBackgroundValue.SetHeight(AValue: integer);
begin
  FHeight := AValue;
end;

procedure TrBackgroundValue.SetState(AValue: TrViewWindowMode);
begin
  FState := AValue;
end;

procedure TrBackgroundValue.SetViewMode(AValue: TrViewMode);
begin
  FViewMode := AValue;
end;

procedure TrBackgroundValue.SetWidth(AValue: integer);
begin
  FWidth := AValue;
end;

procedure TrBackgroundValue.SetXPos(AValue: integer);
begin
  FXPos := AValue;
end;

procedure TrBackgroundValue.SetYPos(AValue: integer);
begin
  FYPos := AValue;
end;

procedure TrBackgroundValue.MakeMax(Sender: TObject);
Var R: TRect;
begin
  if THackForm(Sender) <> nil then begin
    if not (csDesigning in TComponent(Sender).ComponentState) then
      if THackForm(Sender).FormStyle <> fsMDIChild
      then SystemParametersInfo(SPI_GETWORKAREA, 0, @R, 0)
      else GetClientRect(Application.MainForm.ClientHandle, R)
    else R := Rect(0,0, Screen.Width, Screen.Height);
    FMaxValue[0] := R.Right-R.Left;
    FMaxValue[2] := R.Right-R.Left;
    FMaxValue[1] := R.Bottom-R.Top;
    FMaxValue[3] := R.Bottom-R.Top;
  end;
end;

procedure TrBackgroundValue.StoreToRegistry(const RegKey,
  ValueName: String);
Var Reg: TRegistry;
    r: HKEY;
    S: String;
    IsSizeable: boolean;

    function NoSizeable( i: integer ): string;
    begin
      if not IsSizeable then Result := crEmptyStr else
                             Result := IntToStr( i );
    end;

begin
  if ValueName = crEmptyStr then Exit;
  if FOwner <> Nil then
    IsSizeable := (TrBackground( FOwner ).CurrentForm.BorderStyle = bsSizeable); 

  Reg := TRegistry.Create;
  try
    SeparateRegKey(RegKey, r, S);
    Reg.RootKey := r;
    if Reg.OpenKey(S, True) then
      begin
        Reg.WriteString( ValueName,
          FFileName + ';' +
          IntToStr( FColor ) + ';' +
          IntToStr( Integer( FViewMode ) ) + ';' +
          IntToStr( FXPos ) + ';' +
          IntToStr( FYPos ) + ';' +
          NoSizeable( FWidth ) + ';' +
          NoSizeable( FHeight ) + ';' +
          //IntToStr( FWidth ) + ';' +
          //IntToStr( FHeight ) + ';' +
          IntToStr( Integer( FState ) ) );
      end;
  finally
    Reg.Free;
  end;
end;

procedure TrBackgroundValue.LoadFromRegistry(const RegKey,
  ValueName: String);
Var Reg: TRegistry;
    r: HKEY;
    S: String;

    //     
    procedure LoadFromString( const S: String );
    Var i: integer;
        St: String;
    begin
      for i := 1 to GetDelimCount(S, [';']) + 1 do
        begin
          St := ExtractDelimited(I,S,[';']);
          try
            Case i of
              1: FileName := St;
              2: Color := StrToIntDef( St, FColor );
              3: FViewMode := trViewMode( StrToIntDef( St, Integer(FViewMode) ) );
              4: FXPos := StrToIntDef( St, FXPos );
              5: FYPos := StrToIntDef( St, FYPos );
              6: FWidth := StrToIntDef( St, FWidth );
              7: FHeight := StrToIntDef( St, FHeight );
              8: FState := TrViewWindowMode( StrToIntDef( St, Integer(FState) ) );
            end;
          except
            Case i of
              1: FileName := crEmptyStr;
              2: Color := clBtnFace;
              3: FViewMode := rvmNone;
              4: FXPos := -1;
              5: FYPos := -1;
              6: FWidth := -1;
              7: FHeight := -1;
              8: FState := rvwmNone;
            end;
          end;
        end;
    end;

begin
  if ValueName = crEmptyStr then Exit;
  Reg := TRegistry.Create;
  try
    SeparateRegKey(RegKey, r, S);
    Reg.RootKey := r;
    if not Reg.OpenKey(S, False) then Exit;
    // 
    S := Reg.ReadString( ValueName );
    if S = crEmptyStr then Exit;
    LoadFromString( S );
  finally
    Reg.Free;
  end;
end;



procedure TrBackgroundValue.SetDefault;
begin
  FXPos := 0;
  FYPos := 0;
  FViewMode := rvmNone;
end;

function TrBackgroundValue.TestValue(Value: TrBackGroundValue): boolean;
begin
  Result :=
    (Value.Color = Color) and
    (Value.ViewMode = ViewMode) and
    (Value.XPos = XPos) and
    (Value.YPos = YPos) and
    (Value.Width = Width) and
    (Value.Height = Height) and
    (Value.State = State);
end;


procedure TrBackgroundDlg.AssignFrom(Value: TrBackgroundValue);
begin
  Color := Value.Color;
  FileName := Value.FileName;
  ViewMode := Value.ViewMode;
  XPos := Value.XPos;
  YPos := Value.YPos;
  FormWidth := Value.Width;
  FormHeight := Value.Height;
  FormState := Value.State;
end;

procedure TrBackgroundValue.Assign(Value: TrBackgroundValue);
begin
  Color := Value.Color;
  FileName := Value.FileName;
  ViewMode := Value.ViewMode;
  XPos := Value.XPos;
  YPos := Value.YPos;
  Width := Value.Width;
  Height := Value.Height;
  State := Value.State;
end;

procedure TrBackgroundDlg.AssignToValue(Value: TrBackgroundValue);
begin
  Value.Color := Color;
  Value.FileName := FileName;
  Value.ViewMode := ViewMode;
  Value.XPos := XPos;
  Value.YPos := YPos;
  Value.Width := FormWidth;
  Value.Height := FormHeight;
  Value.State := FormState;
end;


procedure TrBackgroundValue.SetFileName(const AValue: TGraphicFileName);
begin
  if AValue = crEmptyStr then
    begin
      Glyph.Assign( Nil );
      FFileName := crEmptyStr;
    end;  
  if not FileExists( AValue ) then Exit;
  Glyph.LoadFromFile( AValue );
  FFileName := AValue;
end;


destructor TrBackgroundDlg.Destroy;
var
  i: Integer;
begin
  FGlyph.Free;
  for i := 0 to FileComboBox.Items.Count -1 do
    StrDispose(PChar(FileComboBox.Items.Objects[i]));
  FDefBkgnd.Free;
  inherited;
end;

function TrBackgroundDlg.GetColor: TColor;
begin
  Result := ColorComboBox1.ColorValue;
end;

function TrBackgroundDlg.GetFileName: TGraphicFileName;
begin
  //Result := FileComboBox.Text;
  Result := FFileName;
end;

function TrBackgroundDlg.GetFormHeight: integer;
begin
  Result := Round( RHeight.Value );
end;

function TrBackgroundDlg.GetFormState: TrViewWindowMode;
begin
  Case RPosition.ItemIndex of
    1: Result := rvwmMaximized;
    2: Result := rvwmCentered;
  else
       Result := rvwmNone;
  end;
end;

function TrBackgroundDlg.GetFormWidth: integer;
begin
  Result := Round( RWidth.Value );
end;

function TrBackgroundDlg.GetGlyph: TPicture;
begin
  Result := FGlyph;
end;

function TrBackgroundDlg.GetViewMode: TrViewMode;
begin
  Result := TrViewMode(ComboBox2.ItemIndex+1);
end;

function TrBackgroundDlg.GetXPos: integer;
begin
  Result := Round( RLeft.Value );
end;

function TrBackgroundDlg.GetYPos: integer;
begin
  Result := Round( RTop.Value );
end;

procedure TrBackgroundDlg.SetColor(AValue: TColor);
begin
  ColorComboBox1.ColorValue := AValue;
  PaintBox1.Refresh;
end;

procedure TrBackgroundDlg.SetFileName(const AValue: TGraphicFileName);
begin
  if GetFileName = aValue then Exit;
  if AValue = crEmptyStr then
    begin
      FGlyph.Bitmap.FreeImage;
      FileComboBox.Text := crEmptyStr;
      PaintBox1.Refresh;
      Exit;
    end;
  FGlyph.LoadFromFile( AValue );
  FileComboBox.Text := AValue;
  PaintBox1.Refresh;
end;

procedure TrBackgroundDlg.SetFormHeight(AValue: integer);
begin
  RHeight.Value := AValue;
end;

procedure TrBackgroundDlg.SetFormState(AValue: TrViewWindowMode);
begin
  Case AValue of
    rvwmMaximized: RPosition.ItemIndex := 1;
    rvwmCentered: RPosition.ItemIndex := 2;
  else
    RPosition.ItemIndex := 0;
  end;
end;

procedure TrBackgroundDlg.SetFormWidth(AValue: integer);
begin
  RWidth.Value := AValue;
end;

procedure TrBackgroundDlg.SetViewMode(AValue: TrViewMode);
begin
  ComboBox2.ItemIndex := ord(AValue)-1;
end;

procedure TrBackgroundDlg.SetXPos(AValue: integer);
begin
  RLeft.Value := AValue;
end;

procedure TrBackgroundDlg.SetYPos(AValue: integer);
begin
  RTop.Value := AValue;
end;


procedure TrBackgroundDlg.ColorComboBox1Change(Sender: TObject);
begin
  PaintBox1.Refresh;
end;

procedure TrBackgroundDlg.SetBkgFileName(Value: TGraphicFileName);
var
  I  : Integer;
  C,S: String;
begin
  S := ExpandFileName(Value);

  if Length(S) > 0 then
    begin
      FFileName := S;
      C := ExtractFileName(Value);
      I := FileComboBox.Items.IndexOf(C);
      if I < 1 then
        begin
          I := FileComboBox.Items.IndexOf(S);
          if I < 1 then I := FileComboBox.Items.AddObject(S,TObject(StrNew(PChar(S))));
        end;
      FGlyph.LoadFromFile(S);
    end else
    begin
      I := 0;
      FFileName := crEmptyStr;
      FGlyph.Assign( Nil );
    end;

  ComboBox2.Enabled := FFileName <> crEmptyStr;
  Label1.Enabled := ComboBox2.Enabled;

  FileComboBox.ItemIndex := I;
  PaintBox1.Refresh;
end;

procedure TrBackgroundDlg.FileComboBoxClick(Sender: TObject);
begin
  with FileComboBox do
   if Items.Objects[ItemIndex] <> nil then
     SetBkgFileName(StrPas(PChar(Items.Objects[ItemIndex])))
   else SetBkgFileName(crEmptyStr);
end;

procedure TrBackgroundDlg.PaintBox1Paint(Sender: TObject);
Var R: TRect;
    dX, dY: integer;
    FollowX, FollowY: integer;
    R2: TRect;
begin
  //  ,   
  R := PaintBox1.ClientRect;
  //   
  with PaintBox1.Canvas do
    begin
      Brush.Color := GetColor;
      Brush.Style := bsSolid;

      Pen.Color := GetColor;
      Pen.Width := 1;
      Pen.Style := psSolid;

      FillRect( R );
    end;

  if FGlyph.Width = 0 then Exit;

  Case ViewMode of
    rvmNone,
      rvmCenter:
        begin
          dX := ((R.Right - R.Left) - FGlyph.Width) div 2;
          dY := ((R.Bottom - R.Top) - FGlyph.Height) div 2;
          R.Left := R.Left + dX;
          R.Top := R.Top + dY;
          R.Right := R.Right - dX;
          R.Bottom := R.Bottom - dY;

          PaintBox1.Canvas.StretchDraw( R, FGlyph.Graphic );
        end;
    rvmTile:
        begin
          FollowX := R.Left;
          FollowY := R.Top;
          While FollowY < R.Bottom do
            begin
              R2.Left := FollowX;
              R2.Top := FollowY;
              R2.Right := R2.Left + FGlyph.Width;
              R2.Bottom := R2.Top + FGlyph.Height;

              PaintBox1.Canvas.StretchDraw( R2, FGlyph.Graphic );

              FollowX := FollowX + FGlyph.Width;
              if FollowX > R.Right then
                begin
                  FollowX := R.Left;
                  FollowY := FollowY + FGlyph.Height;
                end;
            end;
        end;
    rvmStretch:
        begin
          PaintBox1.Canvas.StretchDraw( R, FGlyph.Graphic );
        end;
  end;
end;

procedure RegisterBackgroundControl( AValue: TBackGroundControlClass );
Var i: integer;
begin
  if BackgroundControlClasses = Nil then BackgroundControlClasses := TList.Create;
  // ,     
  for i := 0 to BackgroundControlClasses.Count - 1 do
    if TBackgroundControlClass( BackgroundControlClasses[i] ) = AValue then
      Exit;
  BackgroundControlClasses.Add( AValue );
end;

{ TBackGroundControl }

constructor TBackGroundControl.Create(AOwner: TrBackground);
begin
  FOwner := AOwner;
  FCurrentForm := AOwner.CurrentForm;
  DefineProperties( FControlClass, FAddRegValueName );
end;


procedure TBackGroundControl.LoadBigString(const S: String);
Var C: TComponent;
    i: integer;
    S1, S2: String;
begin
  if Self.FControlClass = TToolBar then Owner.NewToolBarLoaded := false;
  if S = crEmptyStr then Exit;
  if Self.FControlClass = TToolBar then Owner.NewToolBarLoaded := true;
  if CurrentForm = Nil then Exit;
  //    
  if S[1] <> '&' then
    begin
      LoadOldString( S );
      Exit;
    end;
  //    
  for i := 1 to WordCount( S, ['&'] ) div 2 do
    begin
      S1 := ExtractWord( i * 2 - 1, S, ['&'] );
      S2 := ExtractWord( i * 2, S, ['&'] );
      C := CurrentForm.FindComponent( S1 );
      if C = Nil then continue;
      LoadString( C, S2 );
    end;
  // -----------------------
end;

procedure TBackGroundControl.LoadFromRegistry(Form: TCustomForm; const RegKey,
  ValueName: string);
Var Reg: TRegistry;
    S: String;
    r: HKEY;
begin
  if Form = Nil then Exit;
  if ValueName = crEmptyStr then Exit;
  if RegKey = crEmptyStr then Exit;
  FCurrentForm := Form;
  Reg := TRegistry.Create;
  try
    SeparateRegKey(RegKey, r, S);
    Reg.RootKey := r;
    if Reg.OpenKey(S, False) then
         LoadBigString( Reg.ReadString( ValueName + FAddRegValueName ) );
  finally
    Reg.Free;
  end;
end;

procedure TBackGroundControl.SetCurrentForm(AValue: TCustomForm);
begin
  FCurrentForm := AValue;
end;

procedure TBackGroundControl.StoreToRegistry(Form: TCustomForm;
  const RegKey, ValueName: string);
Var i: integer;
    S: String;
    StoredString: String;
    Reg: TRegistry;
    r: HKEY;
begin
  //     
  if Form = Nil then Exit;
  if ValueName = crEmptyStr then Exit;
  StoredString := crEmptyStr;
  for i := 0 to Form.ComponentCount - 1 do
    if Form.Components[i] is FControlClass then
      begin
        S := crEmptyStr;
        StoreString( Form.Components[i], S );
        if S = crEmptyStr then continue;
        StoredString := StoredString + '&' + Form.Components[i].Name + '&' + S;
      end;
  //     
  Reg := TRegistry.Create;
  try
    SeparateRegKey(RegKey, r, S);
    Reg.RootKey := r;
    if Reg.OpenKey(S, True) then
      begin
        if StoredString = crEmptyStr then
          Reg.DeleteValue( ValueName + FAddRegValueName ) else
          Reg.WriteString( ValueName + FAddRegValueName, StoredString );
      end;
  finally
    Reg.Free;
  end;
  // ----------------------------
end;

{ TBackgroundToolbar }

procedure TBackgroundToolbar.DefineProperties(
  var ControlClass: TComponentClass; var AddRegValueName: string);
begin
  ControlClass := TToolBar;
  AddRegValueName := srBkgndTool + '_NEW';
end;

procedure TBackgroundToolbar.LoadString(Sender: TComponent;
  const Text: String);

  function GetInt( const S: String; I: integer ): integer;
  begin
    Result := StrToIntDef( S, I );
    if Result = 0 then Result := I;
  end;

Var S: String;
    C: TComponent;
begin
  if not Owner.SaveToolBar then Exit;
  with TToolBar( Sender ) do
    begin
      S := ExtractWord( 6, Text, [';'] );
      if AnsiCompareText( S, Parent.Name ) <> 0 then
        begin
          C := CurrentForm.FindComponent( S );
          if C <> Nil then TToolBar( Sender ).Parent := TWinControl( C );
        end;
      (*
      Left := StrToIntDef( ExtractWord( 1, Text, [';'] ), Left );
      Top := StrToIntDef( ExtractWord( 2, Text, [';'] ), Top );
      Height := StrToIntDef( ExtractWord( 3, Text, [';'] ), Height );
      Width := StrToIntDef( ExtractWord( 4, Text, [';'] ), Width );
      Visible := Boolean( StrToIntDef( ExtractWord( 5, Text, [';'] ), Integer(true) ) );
      *)

      Left := GetInt( ExtractWord( 1, Text, [';'] ), Left );
      Top := GetInt( ExtractWord( 2, Text, [';'] ), Top );
      Height := GetInt( ExtractWord( 3, Text, [';'] ), Height );
      Width := GetInt( ExtractWord( 4, Text, [';'] ), Width );
      Visible := Boolean( StrToIntDef( ExtractWord( 5, Text, [';'] ), Integer(true) ) );
    end;
end;

procedure TBackgroundToolbar.StoreString(Sender: TComponent;
  var Text: string);
begin
  if not Owner.SaveToolBar then Exit;
  with TToolBar( Sender ) do
    Text := Format('%d;%d;%d;%d;%d;%s',
       [Left, Top, Height, Width, Integer(Visible), Parent.Name] );
end;


procedure TBackgroundToolbar.LoadOldString(const Text: String);
begin
end;

{ TBackgroundSplitter }

procedure TBackgroundSplitter.DefineProperties(
  var ControlClass: TComponentClass; var AddRegValueName: string);
begin
  ControlClass := TSplitter;
  AddRegValueName := srBkgndSlit + '_NEW';
end;

function TBackgroundSplitter.FindControl(SPL: TSplitter): TControl;
var
  P: TPoint;
  I: Integer;
begin
  Result := nil;
  P := Point(SPL.Left, SPL.Top);
  case SPL.Align of
    alLeft: Dec(P.X);
    alRight: Inc(P.X, SPL.Width);
    alTop: Dec(P.Y);
    alBottom: Inc(P.Y, SPL.Height);
  else
    Exit;
  end;
  for I := 0 to SPL.Parent.ControlCount - 1 do
  begin
    Result := SPL.Parent.Controls[I];
    if PtInRect(Result.BoundsRect, P) then Exit;
  end;
  Result := nil;
end;

procedure TBackgroundSplitter.LoadOldString(const Text: String);
Var N, i, j: integer;
    SPL: TControl;
    SS: String;
begin
  {inherited;}
  if not Owner.SaveSplitterPos then Exit;
  N := GetDelimCount(Text, [';']);
  if N <= 0 then Exit;
  N := N+1;
  SPL := nil;
  for I := 1 to N do
    begin
      SS := ExtractDelimited(I,Text,[';']);
      if I = 1 then
        begin
          J := StrToIntDef(SS,0);
          if J = 0 then Break;
        end else
        begin
          J := 0;
          if (I-2) mod 3 <> 0 then
            begin
              J := StrToIntDef(SS,0);
              if J <= 0 then Continue;
            end;
            case (I-2) mod 3 of
              0: begin
                   SPL := nil;
                   if SS <> EmptyStr then SPL := TControl(CurrentForm.FindComponent(SS));
                 end;
              1: if SPL <> nil then SPL.Height := J;
              2: if SPL <> nil then SPL.Width := J;
            end;
        end;
    end;
end;

procedure TBackgroundSplitter.LoadString(Sender: TComponent;
  const Text: String);
Var FC: TControl;
begin
  if not Owner.SaveSplitterPos then Exit;
  FC := TControl( CurrentForm.FindComponent( ExtractWord( 1, Text, [';'] )) );
  if FC = Nil then Exit;
  FC.Height := StrToIntDef( ExtractWord( 2, Text, [';'] ), FC.Height );
  FC.Width := StrToIntDef( ExtractWord( 3, Text, [';'] ), FC.Width );
end;

procedure TBackgroundSplitter.StoreString(Sender: TComponent;
  var Text: string);
Var FC: TControl;
begin
  if not Owner.SaveSplitterPos then Exit;
  FC := FindControl( TSplitter( Sender ) );
  if FC = Nil then Exit;
  Text := Format( '%s;%d;%d', [FC.Name, FC.Height, FC.Width] );
end;

procedure PrepareToolBars( ControlBar1: TControlBar );

  procedure ResizeToolBar(T: TToolBar);
  Var i: integer;
      MaxTop: integer;
      MaxLeft, MaxWidth: integer;
      NewPos: boolean;
  begin
    //    
    if (T.Left + T.Width) < ControlBar1.ClientWidth then Exit;
    //    ToolBar,     
    if ControlBar1.ControlCount = 1 then
      begin
        T.Left := 0;
        Exit;
      end;
    //   
    //      ToolBar-
    MaxTop := 0; MaxLeft := 0; MaxWidth := 0;
    for i := 0 to ControlBar1.ControlCount - 1 do
      begin
        if ControlBar1.Controls[i].Top > MaxTop then
          begin
            MaxTop := ControlBar1.Controls[i].Top;
            MaxLeft := 0;
          end;
        if (ControlBar1.Controls[i].Top = MaxTop) and (ControlBar1.Controls[i].Left > MaxLeft) then
          begin
            MaxLeft := ControlBar1.Controls[i].Left;
            MaxWidth := ControlBar1.Controls[i].Width;
          end;
      end;
    // , ,        
    NewPos := false; MaxLeft := MaxLeft + MaxWidth;
    if ((MaxLeft + T.Width) >= ControlBar1.ClientWidth) or (T.Width = 0) then
      begin
        NewPos := true;
        MaxTop := MaxTop + ControlBar1.RowSize;
        MaxLeft := 0;
      end;
    // ,   
    if not ControlBar1.AutoSize and (ControlBar1.ClientHeight < (MaxTop + ControlBar1.RowSize)) then
      ControlBar1.Height := ControlBar1.Height + ControlBar1.RowSize;
    T.Top := MaxTop;
    T.Left := MaxLeft;
    //     .
    // ,        
    if NewPos then Exit;
    if (T.Left + T.Width) < ControlBar1.ClientWidth then Exit;
    T.Top := T.Top + ControlBar1.RowSize;
    T.Left := 0;
  end;

var
  i: integer;
begin
  for i := 0 to ControlBar1.ControlCount - 1 do
    if ControlBar1.Controls[i] is TToolBar then
       ResizeToolBar(TToolBar(ControlBar1.Controls[i]));
end;

procedure TrBackground.PrepareCurrentToolBars;
Var i: integer;
    F: TCustomForm;
begin
  if not FAlignToolBars then Exit;
  if not BackgroundLoaded then Exit;
  F := CurrentForm;
  for i := 0 to F.ComponentCount - 1 do if F.Components[i] is TControlBar then
    PrepareToolBars( TControlBar( F.Components[i] ) );
end;

initialization
  HookList := nil;
  BackgroundControlClasses := TList.Create;
  RegisterBackgroundControl( TBackgroundSplitter );
  RegisterBackgroundControl( TBackgroundToolBar );


finalization
  DropHookList;
  BackgroundControlClasses.Free;

end.
