(*====================================================================
    File rDSExcel.pas

    Components library for Delphi and BCB++.
    rDSExcel component.

    Copyright  1998-2008, PSTG
    http://www.polesoft.ru
    mailto:support@polesoft.ru

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

======================================================================
$Id: rDSExcel.pas,v 1.2 2008/09/05 09:00:45 savchenko Exp $
======================================================================*)
unit rDSExcel;

interface

uses
  {$ifndef nographics}
    {$ifdef mswindows}
    Windows, Graphics,
    {$endif}
    {$ifdef linux}
    QGraphics,
    {$endif}
  {$endif nographics}
  Types, SysUtils, Classes, DB, RxStrUtils, SQLTimSt;

type

  TrXlsRecordAttr = packed array[0..2] of byte;
  TrDSExcel = class;
  TrXlsCellAttr = class(TPersistent)
  private
    FOwner: TrDSExcel;
    FBorders: array[0..4] of Boolean;
    FAlignment: TAlignment;
    FFormatNumber: Word;
    FFontNumber: Byte;
    function GetBorders(Index: Integer): Boolean;
    procedure SetBorders(Index: Integer; Value: Boolean);
    procedure SetFormatNumber(Value: Word);
  protected
    procedure AssignTo(Dest: TPersistent); override;
    function GetXLSAttr: DWORD;
    property Owner: TrDSExcel read FOwner write FOwner;
  public
    constructor Create(AOwner: TrDSExcel);
    destructor Destroy; override;
    function GetCellAttr: TrXlsRecordAttr;
    property FormatNumber: Word read FFormatNumber write SetFormatNumber default 0;
    property FontNumber: Byte read FFontNumber write FFontNumber default 0;
  published
    property Alignment: TAlignment read FAlignment write FAlignment default taLeftJustify;
    property Shaded: Boolean index 0 read GetBorders write SetBorders default False;
    property LineBottom: Boolean index 1 read GetBorders write SetBorders default False;
    property LineTop: Boolean index 2 read GetBorders write SetBorders default False;
    property LineRight: Boolean index 3 read GetBorders write SetBorders default False;
    property LineLeft: Boolean index 4 read GetBorders write SetBorders default False;
  end;

  TrStaticXlsCellAttr = class(TrXlsCellAttr)
  published
    property Alignment;
    property Shaded;
    property LineBottom;
    property LineTop;
    property LineRight;
    property LineLeft;
  end;

  TXlsProgressEvent=procedure (Sender: TObject; Position: Integer) of object;

  TrDSExcel = class(TComponent)
  private
    { Private declarations }
    {$ifndef nographics}
    FFonts: array[0..3] of TFont;
    {$endif}
    FHeaderFooter: array[0..1] of string;
    FFileName: string;
    FExportOnlyVisible: Boolean;
    FFormats: TStringList;
    FXlsColWidth,
    FXlsAttrs: TList;
    FProgressSteps,
    FProgress: Integer;
    FBeforeExport: TNotifyEvent;
    FOnProgress: TXlsProgressEvent;
    FExportFieldTitle: Boolean;
    {$ifndef nographics}
    function GetFont(Index: Integer): TFont;
    procedure SetFont(Index: Integer; Value: TFont);
    {$endif}
    function GetHeadFoot(Index: integer): string;
    procedure SetHeadFoot(Index: Integer; Value: string);
    function GetDefAttr(Index: Integer): TrStaticXlsCellAttr;
    procedure SetDefAttr(Index: Integer; Value: TrStaticXlsCellAttr);
    function GetPercent: Integer;
//    function GetColWidth(Index: Integer): Integer;
    procedure SetColWidth(Index: Integer; Value: Integer);
    procedure SetFileName(Value: String);
    function GetAttributes(Index: Integer): TrXlsCellAttr;
    function GetAttrCount: Integer;
    function GetDataSet(Index: Integer): TDataSet;
    function GetDataSetCount: Integer;
  protected
    { Protected declarations }
    function CompareAttr(Attr1,Attr2: TrXlsCellAttr): Boolean;
    function IndexOfAttr(FNumber: Byte; FrNumber: Word; FAlign: TAlignment;
                         lShaded, lBottom, lTop, lRight, lLeft: Boolean): Integer;
    function AddAttr(FNumber: Byte; FrNumber: Word; FAlign: TAlignment;
                     lShaded, lBottom, lTop, lRight, lLeft: Boolean): Integer;
    procedure CalcMaxStep;
    procedure IncProgress;
    property AttrCount: Integer read GetAttrCount;
    property Attributes[Index: Integer]: TrXlsCellAttr read GetAttributes;
  public
    AllDataSets: TStringList;
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

    procedure Convert;
    property DataSet[Index: Integer]: TDataSet read GetDataSet;
    property DataSetCount: Integer read GetDataSetCount;
    property Percent: Integer read GetPercent;
  published
    { Published declarations }
    property BeforeExport: TNotifyEvent read fBeforeExport write fBeforeExport;
    property OnProgress: TXlsProgressEvent read FOnProgress write FOnProgress;
    property FileName: string read FFileName write SetFileName;
    {$ifndef nographics}
    property HeaderFont: TFont index 0 read GetFont write SetFont;
    property FooterFont: TFont index 1 read GetFont write SetFont;
    property CellFont: TFont index 2 read GetFont write SetFont;
    property TitleFont: TFont index 3 read GetFont write SetFont;
    {$endif}
    property HeaderAttr: TrStaticXlsCellAttr index 0 read GetDefAttr write SetDefAttr;
    property FooterAttr: TrStaticXlsCellAttr index 1 read GetDefAttr write SetDefAttr;
    property TitleAttr: TrStaticXlsCellAttr index 2 read GetDefAttr write SetDefAttr;
    property DefCellAttr: TrStaticXlsCellAttr index 3 read GetDefAttr write SetDefAttr;
    property Header: string index 0 read GetHeadFoot write SetHeadFoot;
    property Footer: string index 1 read GetHeadFoot write SetHeadFoot;
    property ExportFieldTitle: Boolean read FExportFieldTitle write FExportFieldTitle default True;
    property ExportOnlyVisible: Boolean read FExportOnlyVisible write FExportOnlyVisible default true;
  end;

implementation

const
  FileHeader    : packed array[0..4] of Word = ($809, $6, $0, $10, $0);
//  DimRecordProto: packed array[0..5] of Word = (0,$8,0,0,0,0);
  XlsEOF        : packed array[0..1] of Word = ($A,0);

  MAXWORD = 65535;

  {$ifdef nographics}
  cFontName = 'Arial';
  cFontSize = 8;
  {$endif}

type
  TrXlsRecordHeader = packed record
    Opcode,
    TotalLength,
    Row, Col: Word;
    Attr: TrXlsRecordAttr;
  end;

const
  StrRecHeader: TrXlsRecordHeader = (Opcode: $4; TotalLength: $0;
                                     Row: $0; Col: $0; Attr: (0,0,0));
  IntRecHeader: TrXlsRecordHeader = (Opcode: $2; TotalLength: 9;
                                     Row: $0; Col: $0; Attr: (0,0,0));
  DoubleRecHeader: TrXlsRecordHeader = (Opcode: $3; TotalLength: 15;
                                     Row: $0; Col: $0; Attr: (0,0,0));
//  DateRecHeader: TrXlsRecordHeader = (Opcode: $3; TotalLength: 15;
//                                     Row: $0; Col: $0; Attr: (0,0,0));
  BoolRecHeader: TrXlsRecordHeader = (Opcode: $5; TotalLength: 9;
                                     Row: $0; Col: $0; Attr: (0,0,0));
  BlankRecHeader: TrXlsRecordHeader = (Opcode: $1; TotalLength: 7;
                                     Row: $0; Col: $0; Attr: (0,0,0));

procedure WriteDataCell(Stream: TStream; X: Word; var Y: Word;
                        Attr: TrXlsCellAttr; Value: String); overload;
var
  BufRec: packed record
    Header: TrXlsRecordHeader;
    Data: ShortString;
  end;
  P, I, nY: Integer;
  C: Char;
  S: String;
  R: String;

  function GetNextStr(St: String;var P: Integer): shortString;
  var
    I, L,
    EC: Integer;

  begin
    L := Length(St);
    Result := '';
    Inc(P);
    if P > L then Exit;
    EC := 0;
    if L > 255 then begin
      for I := 255 downto 1 do
        if (P+I=L) or ((P+I<L) and (St[P+I] = ' ')) then begin
          EC := I;
          Break;
        end;
    end;
    if EC = 0 then EC := 255;
    Result := DelESpace(Copy(St,P,EC));
    Inc(P,EC);
  end;

begin
  R := Value;
  P := 0;
  nY := Y-1;
  S := StringReplace(R, #10, ' ', [rfReplaceAll]);
  for C := #0 to #31 do S := StringReplace(S, C, '', [rfReplaceAll]);
  I := 0;
  repeat
    if nY>65534 then break;
    BufRec.Header := StrRecHeader;
    BufRec.Header.Row  := Y+I;
    BufRec.Header.Col  := X;
    BufRec.Header.Attr := Attr.GetCellAttr;
    BufRec.Data        := GetNextStr(S,P);
    if (BufRec.Data <> '') or (I=0) then begin
      BufRec.Header.TotalLength := Length(BufRec.Data)+8;
      Stream.Write(BufRec, 12+Length(BufRec.Data));
      Inc(nY);
    end;
    Inc(I);
  until BufRec.Data = '';
  Y := nY;
end;

procedure WriteDataCell(Stream: TStream; X: Word; var Y: Word;
                        Attr: TrXlsCellAttr; Value: Smallint); overload;
var
  BufRec: packed record
    Header: TrXlsRecordHeader;
    Data: Smallint;
  end;
begin
  if Y>65534 then Exit;
  BufRec.Header      := IntRecHeader;
  BufRec.Header.Row  := Y;
  BufRec.Header.Col  := X;
  BufRec.Header.Attr := Attr.GetCellAttr;
  BufRec.Data        := Value;
  Stream.Write(BufRec, SizeOf(BufRec));
end;

procedure WriteDataCell(Stream: TStream; X: Word; var Y: Word;
                       Attr: TrXlsCellAttr; Value: Double); overload;
var
  BufRec: packed record
    Header: TrXlsRecordHeader;
    Data: Double;
  end;
begin
  if Y>65534 then Exit;
  BufRec.Header      := DoubleRecHeader;
  BufRec.Header.Row  := Y;
  BufRec.Header.Col  := X;
  BufRec.Header.Attr := Attr.GetCellAttr;
  BufRec.Data        := Value;
  Stream.Write(BufRec, SizeOf(BufRec));
end;

procedure WriteDataCell(Stream: TStream; X: Word; var Y: Word;
                        Attr: TrXlsCellAttr; Value: Boolean); overload;
var
  BufRec: packed record
    Header: TrXlsRecordHeader;
    Data: ByteBool;
    Specifies: Byte;
  end;
begin
  if Y>65534 then Exit;
  BufRec.Header      := BoolRecHeader;
  BufRec.Header.Row  := Y;
  BufRec.Header.Col  := X;
  BufRec.Data        := Value;
  BufRec.Specifies   := 0;
  BufRec.Header.Attr := Attr.GetCellAttr;
  Stream.Write(BufRec, SizeOf(BufRec));
end;

procedure WriteBlankCell(Stream: TStream; X: Word; var Y: Word;
                        Attr: TrXlsCellAttr);
var
  BufRec: packed record
    Header: TrXlsRecordHeader;
  end;
begin
  if Y>65534 then Exit;
  BufRec.Header      := BlankRecHeader;
  BufRec.Header.Row  := Y;
  BufRec.Header.Col  := X;
  BufRec.Header.Attr := Attr.GetCellAttr;
  Stream.Write(BufRec, SizeOf(BufRec));
end;

{TrXlsCellAttr}
constructor TrXlsCellAttr.Create(AOwner: TrDSExcel);
begin
  inherited Create;
  FOwner := AOwner;
  FillChar(FBorders, SizeOf(FBorders), False);
end;

destructor TrXlsCellAttr.Destroy;
begin
  FOwner := nil;
  inherited Destroy;
end;

function TrXlsCellAttr.GetBorders(Index: Integer): Boolean;
begin
  Result := False;
  if Index in [0..4] then Result := FBorders[Index]
end;

procedure TrXlsCellAttr.SetBorders(Index: Integer; Value: Boolean);
begin
  if (Index in [0..4]) and (Value <> FBorders[Index]) then FBorders[Index] := Value;
end;

procedure TrXlsCellAttr.SetFormatNumber(Value: Word);
begin
  if (Value <> FFormatNumber) and (Value in [0..63]) and (Value  <FOwner.FFormats.Count)
  then FFormatNumber := Value;
end;

procedure TrXlsCellAttr.AssignTo(Dest: TPersistent);
begin
  if Dest is TrXlsCellAttr then
    with Dest as TrXlsCellAttr do begin
      FOwner := Self.FOwner;
      FBorders := Self.FBorders;
      FAlignment := Self.FAlignment;
      FFormatNumber := Self.FFormatNumber;
      FFontNumber := Self.FFontNumber;
    end
  else inherited AssignTo(Dest);
end;

function TrXlsCellAttr.GetXLSAttr: DWORD;
const
  Bits: Array[0..4] of Byte = ($80,$40,$20,$10,$8);
var
  DOpt: DWORD;
  BOpt: array[0..3] of byte absolute DOpt;
  I: Integer;
begin
  FillChar(DOpt,SizeOf(DOpt),0);
  BOpt[0] := FFontNumber;
  BOpt[2] := $40 or FFormatNumber;
  for I := 0 to 4 do
    if FBorders[I] then BOpt[3] := BOpt[3] or Bits[I];
  case Alignment of
  taLeftJustify : BOpt[3] := BOpt[3] or 1;
  taCenter      : BOpt[3] := BOpt[3] or 2;
  taRightJustify: BOpt[3] := BOpt[3] or 3;
  end;
  Result := DOpt;
end;

function TrXlsCellAttr.GetCellAttr: TrXlsRecordAttr;
begin
  FillChar(Result,SizeOf(TrXlsRecordAttr),0);
  Result[0] := $40 or FOwner.FXlsAttrs.IndexOf(Self);
end;

{TrDSExcel}

constructor TrDSExcel.Create(AOwner: TComponent);
{$ifndef nographics}
var
  I: Integer;
{$endif}
begin
  inherited Create(AOwner);
  {$ifndef nographics}
  for I := 0 to 3 do begin
    FFonts[I] := TFont.Create;
    FFonts[I].Name := 'Arial';
    FFonts[I].Charset := RUSSIAN_CHARSET;
    if I in [0,1] then FHeaderFooter[I] := '';
  end;
  {$endif}
  FExportOnlyVisible := True;
  FExportFieldTitle := True;
  FProgressSteps := 0;
  FProgress      := 0;
  FFormats := TStringList.Create;
  FXlsColWidth := TList.Create;
  FFormats.Add('General');
  AllDataSets := TStringList.Create;
  FXlsAttrs := TList.Create;
  AddAttr(6, 0, taLeftJustify, False, False, False, False, False);
  AddAttr(7, 0, taLeftJustify, False, False, False, False, False);
  AddAttr(8, 0, taLeftJustify, False, False, False, False, False);
  AddAttr(5, 0, taLeftJustify, False, False, False, False, False);
  FOnProgress := nil;
end;

destructor TrDSExcel.Destroy;
var
  I: Integer;
begin
  {$ifndef nographics}
  for I := 0 to 3 do
    if FFonts[I] <> nil then begin
      FFonts[I].Free;
      FFonts[I] := nil;
    end;
  {$endif}
  FFormats.Free;
  AllDataSets.Free;
  FXlsColWidth.Free;
  for I := 0 to FXlsAttrs.Count-1 do
  if FXlsAttrs.Items[I] <> nil then begin
    TrStaticXlsCellAttr(FXlsAttrs.Items[I]).Free;
    FXlsAttrs.Items[I] := nil;
  end;
  FXlsAttrs.Free;
  inherited Destroy;
end;

{$ifndef nographics}
function TrDSExcel.GetFont(Index: Integer): TFont;
begin
  Result := nil;
  if Index in [0..3] then Result := FFonts[Index];
end;

procedure TrDSExcel.SetFont(Index: Integer; Value: TFont);
begin
  if Index in [0..3] then begin
    if FFonts[Index] = nil then FFonts[Index] := TFont.Create;
    FFonts[Index].Assign(Value);
  end;
end;
{$endif}

function TrDSExcel.GetHeadFoot(Index: integer): String;
begin
  Result := '';
  if Index in [0,1] then Result := FHeaderFooter[Index];
end;

procedure TrDSExcel.SetHeadFoot(Index: Integer; Value: String);
begin
  if (Index in [0..1]) and (FHeaderFooter[Index] <> Trim(Value))
  then FHeaderFooter[Index] := Trim(Value);
end;

function TrDSExcel.CompareAttr(Attr1,Attr2: TrXlsCellAttr): Boolean;
begin
  Result := False;
  if (Attr1 <> nil) and (Attr2 <> nil)
  then
     Result := (Attr1.FontNumber=Attr2.FontNumber) and
               (Attr1.Alignment=Attr2.Alignment) and
               (Attr1.FormatNumber=Attr2.FormatNumber) and
               (Attr1.Shaded=Attr2.Shaded) and
               (Attr1.LineBottom=Attr2.LineBottom) and
               (Attr1.LineTop=Attr2.LineTop) and
               (Attr1.LineRight=Attr2.LineRight) and
               (Attr1.LineLeft=Attr2.LineLeft);
end;

function TrDSExcel.IndexOfAttr(FNumber: Byte; FrNumber: Word; FAlign: TAlignment;
                               lShaded, lBottom, lTop,
                               lRight, lLeft: Boolean): Integer;
var
  I: Integer;
begin
  Result := -1;
  for I := 0 to FXlsAttrs.Count-1 do
    with TrXlsCellAttr(FXlsAttrs.Items[I]) do
      if (FontNumber=FNumber) and
         (FormatNumber=FrNumber) and
         (Alignment=FAlign) and
         (lShaded=Shaded) and
         (lBottom=LineBottom) and
         (lTop=LineTop) and
         (lRight=LineRight) and
         (lLeft=LineLeft)
      then begin
        Result := I;
        Break;
      end;
end;

function TrDSExcel.AddAttr(FNumber: Byte; FrNumber: Word; FAlign: TAlignment;
                          lShaded, lBottom, lTop, lRight, lLeft: Boolean): Integer;
var
  RA: TrXlsCellAttr;
begin
  Result := IndexOfAttr(FNumber, FrNumber, FAlign, lShaded, lBottom, lTop, lRight, lLeft);
  if Result = -1 then begin
    RA := TrXlsCellAttr.Create(Self);
    RA.FAlignment    := FAlign;
    RA.FFontNumber   := FNumber;
    RA.FFormatNumber := FrNumber;
    RA.Shaded        := lShaded;
    RA.LineBottom    := lBottom;
    RA.LineTop       := lTop;
    RA.LineRight     := lRight;
    RA.LineLeft      := lLeft;
    Result := FXlsAttrs.Add(RA);
  end;
end;

function TrDSExcel.GetDefAttr(Index: Integer): TrStaticXlsCellAttr;
begin
  Result := nil;
  if Index < FXlsAttrs.Count then Result := TrStaticXlsCellAttr(FXlsAttrs.Items[Index]);
end;

procedure TrDSExcel.SetDefAttr(Index: Integer; Value: TrStaticXlsCellAttr);
begin
  if (Value <> nil) and (Index < FXlsAttrs.Count) and not
     CompareAttr(Value, TrStaticXlsCellAttr(FXlsAttrs.Items[Index]))
  then TrStaticXlsCellAttr(FXlsAttrs.Items[Index]).Assign(Value);
end;

function _RecordCount( D: TDataset ): integer;
begin
  if D.IsUniDirectional then Result := 1 else
                             Result := D.RecordCount;
end;


procedure TrDSExcel.CalcMaxStep;
var
  I: Integer;
  D: TDataSet;
begin
  FProgressSteps := 0;
  for I := 0 to DataSetCount-1 do begin
    D := DataSet[I];
    if (D <> nil) and D.Active
    then if FProgressSteps+_RecordCount( D ) <= MaxWord
      then Inc(FProgressSteps,_RecordCount(D))
      else FProgressSteps := MaxWord;
    if FProgressSteps = MaxWord then break;
  end;
  FProgressSteps := FProgressSteps+9;
end;

procedure TrDSExcel.IncProgress;
var
  Per1, Per2: Integer;
begin
  Per1 := Percent;
  Inc(FProgress);
  Per2 := Percent;
  if (Per1 <> Per2) or (Per2 = 0) then begin
    if Assigned(FOnProgress) then FOnProgress(Self, Per2);
  end;
end;

function TrDSExcel.GetDataSetCount: Integer;
begin
  Result := AllDatasets.Count;
end;

function TrDSExcel.GetDataSet(Index: Integer): TDataSet;
begin
  Result := nil;
  if ((Index < AllDatasets.Count) and (Index >= 0))
  and (AllDatasets.Objects[Index] <> nil)
  then
    Result := AllDatasets.Objects[Index] as TDataSet;
end;

function TrDSExcel.GetPercent: Integer;
begin
  Result := 0;
  if FProgressSteps <> 0
  then Result := Round(FProgress/FProgressSteps*100);
end;

{function TrDSExcel.GetColWidth(Index: Integer): Integer;
begin
  Result := -1;
  if Index < FXlsColWidth.Count then
    Result := Integer(FXlsColWidth.Items[Index]) shr 8;
end;}

procedure TrDSExcel.SetColWidth(Index: Integer; Value: Integer);
begin
  if Value > 255 then Value := 255;
  Value := Value shl 8;
  while Index > FXlsColWidth.Count-1 do FXlsColWidth.Add(Pointer(-1));
  if (Index < FXlsColWidth.Count) then
    begin
      if (Integer(FXlsColWidth.Items[Index])<>Value) then
          FXlsColWidth.Items[Index] := Pointer(Value);
    end
end;

procedure TrDSExcel.Convert;

const
  OKFields: Set of TFieldType = [ftString, ftSmallint, ftWord, ftFloat, ftInteger, ftCurrency,
                                 ftBCD, ftFMTBCD, ftDate, ftTime, ftDateTime, ftMemo, ftTimeStamp];
var
  Tmp: TStream;

  procedure CreateHeaderFooter(Value: String; var Y: Word);
  var
    At: Integer;

  begin
    if Trim(Value) <> '' then begin
      if Y > 0
      then
          At := IndexOfAttr( 7, 0, FooterAttr.Alignment, FooterAttr.Shaded,
                       FooterAttr.LineBottom, FooterAttr.LineTop,
                       FooterAttr.LineRight, FooterAttr.LineLeft)
      else
        At := IndexOfAttr( 6, 0, HeaderAttr.Alignment, HeaderAttr.Shaded,
                       HeaderAttr.LineBottom, HeaderAttr.LineTop,
                       HeaderAttr.LineRight, HeaderAttr.LineLeft);
      if At <> -1 then begin
        WriteDataCell(Tmp, 0, Y, TrXlsCellAttr(FXlsAttrs.Items[At]), Value);
        Inc(Y);
      end;
    end;
  end;

  function GetFieldFormat(Fld: TField): String;
  begin
     Result := '';
     case Fld.DataType of
     ftSmallint: Result := TSmallintField(Fld).DisplayFormat;
     ftWord    : Result := TWordField(Fld).DisplayFormat;
     ftFloat   : Result := TFloatField(Fld).DisplayFormat;
     ftInteger : Result := TIntegerField(Fld).DisplayFormat;
     ftCurrency: Result := TCurrencyField(Fld).DisplayFormat;
     ftBCD     : Result := TBCDField(Fld).DisplayFormat;
     ftFMTBCD  : Result := TFMTBCDField(Fld).DisplayFormat;
     ftDate    : begin
                   Result := TDateField(Fld).DisplayFormat;
                   if Result = '' then Result := ShortDateFormat;
                 end;
     ftTime    : begin
                   Result := TTimeField(Fld).DisplayFormat;
                   if Result = '' then Result := ShortTimeFormat;
                 end;
     ftDateTime: begin
                   Result := TDateTimeField(Fld).DisplayFormat;
                   if Result = '' then Result := ShortDateFormat+' '+ShortTimeFormat;
                 end;
     ftTimeStamp: begin
                   Result := TSQLTimeStampField(Fld).DisplayFormat;
                   if Result = '' then Result := ShortDateFormat+' '+ShortTimeFormat;
                   Result := ShortDateFormat+' '+ShortTimeFormat;
                 end;
     end;
  end;

  procedure CreateCellAttr;
  var
    Fld : TField;
    Fms: String;
    Cnt, Fm,
    I, J, N: Integer;
  begin
    Cnt := 0;
    for I := 0 to 1 do if FHeaderFooter[0] <> '' then
      if I = 0 then
        AddAttr( 7, 0, FooterAttr.Alignment, FooterAttr.Shaded,
                 FooterAttr.LineBottom, FooterAttr.LineTop,
                 FooterAttr.LineRight, FooterAttr.LineLeft)
      else
        AddAttr( 6, 0, HeaderAttr.Alignment, HeaderAttr.Shaded,
                 HeaderAttr.LineBottom, HeaderAttr.LineTop,
                 HeaderAttr.LineRight, HeaderAttr.LineLeft);
    for I := 0 to DataSetCount-1 do begin
      if (DataSet[I] <> nil) and DataSet[I].Active then
      for J := 0 to DataSet[I].FieldCount-1 do begin
        Fld := DataSet[I].Fields[J];
        if not (Fld.DataType in OKFields) then Continue;
        if not Fld.Visible then Continue
        else Inc(Cnt);
        if Cnt > 256 then break;
        if Cnt > FXlsColWidth.Count then
          if Fld.DataType = ftMemo
          then SetColWidth(Cnt-1, 255)
          else
            begin
              N := Fld.DisplayWidth;
              SetColWidth(Cnt-1, N);
            end;  

        Fms := GetFieldFormat(Fld);
        if Fms = '' then Fm := 0
        else begin
          Fm := FFormats.IndexOf(Fms);
          if Fm = -1 then Fm := FFormats.Add(Fms);
        end;
        AddAttr( 8, 0, TitleAttr.Alignment, TitleAttr.Shaded,
                 TitleAttr.LineBottom, TitleAttr.LineTop, TitleAttr.LineRight,
                 TitleAttr.LineLeft);
        AddAttr( 5, Fm, Fld.Alignment, DefCellAttr.Shaded,
                 DefCellAttr.LineBottom, DefCellAttr.LineTop, DefCellAttr.LineRight,
                 DefCellAttr.LineLeft);
      end;
      if Cnt > 256 then break;
    end;
  end;

  procedure WriteDimension(Stream: TStream; X, Y: Word);
  var
    RecBuf: packed record
      Opcode, TotalLength,
      FirstRow, LastRow,
      FirstCol, LastCol: Word;
    end;
  begin
    if Stream = nil then Exit;
    FillChar(RecBuf, SizeOf(RecBuf),0);
    RecBuf.TotalLength := 8;
    RecBuf.LastRow     := Y;
    RecBuf.LastCol     := X;
    Stream.Write(RecBuf, SizeOf(RecBuf));
    IncProgress;
  end;

  procedure WriteFonts(Stream: TStream);
  {$ifndef nographics}
  const _FontStyle: Array [fsBold..fsStrikeOut] of Word = ($1,$2, $4, $8);
  {$endif}
  var
    RecBuf: packed record
      Opcode, TotalLength,
      Height, FontAttr: Word;
      FontName: ShortString;
    end;
    I: Word;

    {$ifndef nographics}
    function GetXlsFontAttr(Fnt: TFont): Word;
    var
      J: TFontStyle;
    begin
      Result := 0;
      for J := fsBold to fsStrikeOut do
        if J in Fnt.Style then Result := Result or _FontStyle[J];
    end;
    {$endif}

  {$ifndef nographics}
  var
    Fn: Byte;
  {$endif}
  begin
    IncProgress;
    FillChar(RecBuf,SizeOf(RecBuf),0);
    RecBuf.Opcode      := $31;
    RecBuf.TotalLength := 5+Length({$ifndef nographics}FFonts[2].Name{$else}cFontName{$endif});
    RecBuf.Height      := {$ifndef nographics}FFonts[2].Size{$else}cFontSize{$endif}*20;
    RecBuf.FontAttr    := {$ifndef nographics}GetXlsFontAttr(FFonts[2]){$else}0{$endif};
    RecBuf.FontName    := {$ifndef nographics}FFonts[2].Name{$else}cFontName{$endif};
    for I := 0 to 4 do Stream.Write(RecBuf, 9+Length({$ifndef nographics}FFonts[2].Name{$else}cFontName{$endif}));
    for I := 0 to 2 do begin
      {$ifndef nographics}
      Fn := I;
      if Fn = 2 then Fn := 3;
      {$endif}
      FillChar(RecBuf,SizeOf(RecBuf),0);
      RecBuf.Opcode  := $31;
      RecBuf.TotalLength := Length({$ifndef nographics}FFonts[Fn].Name{$else}cFontName{$endif})+5;
      RecBuf.Height  := {$ifndef nographics}FFonts[Fn].Size{$else}cFontSize{$endif}*20;
      RecBuf.FontAttr:= {$ifndef nographics}GetXlsFontAttr(FFonts[Fn]){$else}0{$endif};
      RecBuf.FontName    := {$ifndef nographics}FFonts[Fn].Name{$else}cFontName{$endif};
      Stream.Write(RecBuf,  9+Length({$ifndef nographics}FFonts[Fn].Name{$else}cFontName{$endif}));
    end;
  end;

  procedure WriteFormats(Stream: TStream);
  var
    RecBuf: packed record
      Opcode, TotalLength: Word;
      Format: ShortString;
    end;
    I, L: Integer;
  begin
    if Stream = nil then Exit;
    IncProgress;
    for I := 0 to FFormats.Count-1 do begin
      FillChar(RecBuf, SizeOf(RecBuf),0);
      RecBuf.Opcode := $1E;
      L := 5+Length(FFormats.Strings[I]);
      RecBuf.TotalLength := Length(FFormats.Strings[I])+1;
      RecBuf.Format      := FFormats.Strings[I];
      Stream.Write(RecBuf,  L);
    end
  end;

  procedure WriteAttributes(Stream: TStream);
  var
   RecBuf: packed record
     Opcode, TotalLength: Word;
     Attrs: DWORD;
   end;
   C: Integer;
   I: Byte;
  begin
    if Stream = nil then exit;
    IncProgress;
    C := AttrCount-1;
    if C > 63 then C := 63;
    for I := 0 to C do begin
      RecBuf.Opcode := $43;
      RecBuf.TotalLength := $04;
      RecBuf.Attrs := Attributes[I].GetXLSAttr;
      Stream.Write(RecBuf, SizeOf(RecBuf));
    end;
  end;

  procedure WriteColWidth(Stream: TStream);
  var
   RecBuf: packed record
     Opcode, TotalLength: Word;
     FirstCol, LastCol: Byte;
     ColWidth: Word;
   end;
   C: Integer;
   I: Byte;
  begin
    C := FXlsColWidth.Count-1;
    if C > 255 then C := 255;
    for I := 0 to C do begin
      FillChar(RecBuf, SizeOf(RecBuf),0);
      RecBuf.Opcode := $24;
      RecBuf.TotalLength := $4;
      RecBuf.FirstCol := I;
      RecBuf.LastCol  := I;
      RecBuf.ColWidth := Integer(FXlsColWidth.Items[I]);
      Stream.Write(RecBuf, SizeOf(RecBuf));
    end;
    IncProgress;
  end;

  procedure WriteData(var MaxY, Y: Word);
  var
    LastY, I: Integer;

    procedure CreateTitle(DataSet: TDataSet; var Y: Word);
    var
      Fld: TField;
      I, lY, mY: Word;
//      FmS: String;
      {Fm, }Cnt,
      At: Integer;
      S: String;

    //CreateTitle
    begin
      mY := Y;
      Cnt := 0;
      if DataSet <> nil then
        for I := 0 to DataSet.FieldCount-1 do begin
          Fld := DataSet.Fields[I];
          if (not FExportOnlyVisible or Fld.Visible) and (Fld.DataType in OKFields)
          then begin
            Inc(Cnt);
            if Cnt > 256 then break;
{            Fms := GetFieldFormat(Fld);
            if Fms = '' then Fm := 0
            else begin
              Fm := FFormats.IndexOf(Fms);
              if Fm = -1 then Fm := 0;
            end;}
            At := IndexOfAttr( 8, 0, TitleAttr.Alignment, TitleAttr.Shaded,
                    TitleAttr.LineBottom, TitleAttr.LineTop,
                    TitleAttr.LineRight, TitleAttr.LineLeft);
            if At <> -1 then begin
              lY := Y;
              S := Copy(Fld.DisplayName,1,255);
              WriteDataCell(Tmp, Cnt-1, lY, TrXlsCellAttr(FXlsAttrs.Items[At]),S);
              if mY < lY then mY := lY;
            end;
            if mY > 65533 then break;
          end;
        end;
      if mY > Y then Y := mY;
    end;
    //CreateTitle

    procedure CreateRow(DataSet: TDataSet; var Y: Word);
    var
      Fld: TField;
      I, mY, lY: Word;
      FmS: String;
      Fm, Cnt,
      At: Integer;

    //CreateRow
    begin
      if Y > 65533 then Exit;
      mY := Y;
      if DataSet <> nil then
        begin
          Cnt := 0;
          for I := 0 to DataSet.FieldCount-1 do
            begin
              Fld := DataSet.Fields[I];
              if (not FExportOnlyVisible or Fld.Visible) and (Fld.DataType in OKFields) then
                begin
                  Inc(Cnt);
                  if Cnt > 256 then break;
                  Fm := 0;
                  if not Fld.IsNull then
                    begin
                      Fms := GetFieldFormat(Fld);
                      if Fms = '' then Fm := 0 else
                        begin
                          Fm := FFormats.IndexOf(Fms);
                          if Fm = -1 then Fm := 0;
                        end;
                    end;
                  At := IndexOfAttr( 5, Fm, Fld.Alignment, DefCellAttr.Shaded,
                               DefCellAttr.LineBottom, DefCellAttr.LineTop,
                               DefCellAttr.LineRight, DefCellAttr.LineLeft);
                  if At <> -1 then
                    begin
                      lY := Y;
                      if Fld.IsNull
                        then WriteBlankCell(Tmp, Cnt-1, lY, TrXlsCellAttr(FXlsAttrs.Items[At])) else
                        begin
                          case Fld.DataType of
                            ftString, ftMemo:
                              WriteDataCell(Tmp, Cnt-1, lY, TrXlsCellAttr(FXlsAttrs.Items[At]),Fld.AsString);
                            ftSmallint, ftWord:
                              WriteDataCell(Tmp, Cnt-1, lY, TrXlsCellAttr(FXlsAttrs.Items[At]),SmallInt(Fld.AsInteger));
                            ftBoolean:
                              WriteDataCell(Tmp, Cnt-1, lY, TrXlsCellAttr(FXlsAttrs.Items[At]),Fld.AsBoolean);
                            ftDate, ftTime, ftDateTime,
                            ftFloat, ftInteger, ftCurrency, ftBCD, ftFMTBCD:
                              WriteDataCell(Tmp, Cnt-1, lY, TrXlsCellAttr(FXlsAttrs.Items[At]), Fld.AsFloat);
                            ftTimeStamp:
                              WriteDataCell(Tmp, Cnt-1, lY, TrXlsCellAttr(FXlsAttrs.Items[At]),
                                SQLTimeStampToDateTime( Fld.AsSQLTimeStamp ) );
                          end;
                        end;
                      if mY < lY then mY := lY;
                    end;
                end;
                if mY > 65533 then break;

            end;
          //       !!!!
            
        end;
        if mY > Y then Y := mY;
    end;
    //CreateRow

  //WriteData
  begin
    if Y > 65533 then Exit;
    for I := 0 to DataSetCount-1 do begin
      if Y > 65533 then break;
      if (DataSet[I] <> nil) and DataSet[I].Active then begin
        if FExportFieldTitle then CreateTitle(DataSet[I], Y);
        if MaxY < Y then MaxY := Y;
        if MaxY > Y then Y := MaxY;
        Inc(Y);
        if not Dataset[i].IsUniDirectional then Dataset[I].First;
        while not DataSet[I].EOF do begin
          LastY := Y;
          CreateRow(DataSet[I], Y);
          if MaxY < Y then MaxY := Y;
          Y := LastY;
          if MaxY > Y then Y := MaxY;
          DataSet[I].Next;
          inc(Y);
          IncProgress;
          if Y > 65533 then break;
        end;
      end;
      inc(Y)
    end;
  end;
  //WriteData

  function CalcLineCount: Integer;
  var
    I: Integer;
  begin
    Result := 0;
    for I := 0 to 1 do
      if FHeaderFooter[I] <> '' then Inc(Result);
    for I := 0 to DataSetCount-1 do begin
      if (DataSet[I] <> nil) and DataSet[I].Active
      then Inc(Result, _RecordCount(Dataset[i]));
      if Result > 65534 then begin
        Result := 65534;
        Break;
      end;
    end;
  end;

//Convert
var
  Y, MaxY: Word;
  ResultStream: TFileStream;
begin
  if FFileName = '' then Exit;
  if Assigned(fBeforeExport) then fBeforeExport(Self);

  CalcMaxStep;
  Y := 0;
  MaxY := 0;
  CreateCellAttr;
  ResultStream := TFileStream.Create(FFileName, fmCreate or fmOpenReadWrite or fmShareDenyWrite);
  try
    Tmp := ResultStream;
    ResultStream.Write(FileHeader, SizeOf(FileHeader));      //
    IncProgress;
    WriteDimension(ResultStream, FXlsColWidth.Count+1, CalcLineCount+1); //DIMENSION
    WriteFonts(ResultStream);                                //FONT
    WriteFormats(ResultStream);                              //FORMAT
    WriteAttributes(ResultStream);                           //ATTRIBUTES
    WriteColWidth(ResultStream);                             //COLWIDTH
    CreateHeaderFooter(FHeaderFooter[0],Y);
    if MaxY < Y then MaxY := Y;
    WriteData(MaxY, Y);                                      //
    if MaxY < Y then MaxY := Y;
    CreateHeaderFooter(FHeaderFooter[1],Y);
    ResultStream.Write(XlsEOF,SizeOf(XlsEOF));               // 
    IncProgress;
  finally
    ResultStream.Free;
    FProgress := -1;
    IncProgress;
  end;
end;
//Convert

procedure TrDSExcel.SetFileName(Value: String);
begin
  Value := ExpandFileName(Value);
  if ExtractFileExt(Value) = '' then Value := ChangeFileExt(Value,'xls');
  Value := Trim(Value);
  if Value <> FFileName then FFileName := Value;
end;

function TrDSExcel.GetAttrCount: Integer;
begin
  Result := FXlsAttrs.Count;
end;

function TrDSExcel.GetAttributes(Index: Integer): TrXlsCellAttr;
begin
  Result := nil;
  if FXlsAttrs.Count > Index then
    Result := TrXlsCellAttr(FXlsAttrs.Items[Index]);
end;

end.

