(*====================================================================
    File rDataEvents.pas

    Components library for Delphi and BCB++.
    rDataEvents component.

    Copyright  1998-2008, PSTG
    http://www.polesoft.ru
    mailto:support@polesoft.ru

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

======================================================================
$Id: rDataEvents.pas,v 1.2 2008/09/05 09:00:45 savchenko Exp $
======================================================================*)
unit rDataEvents;

interface

uses
  Windows, Messages, SysUtils, Classes, DB;

Type TDEvent = procedure (Sender: TDataset; Event: TDataEvent; Info: longint ) of object;

Type TRDatalink = class;


  TrDataEvents = class(TComponent)
  private
    FOnDataChange: TNotifyEvent;
    FOnEditingChange: TNotifyEvent;
    FOnActiveChange: TNotifyEvent;
    FDataEvent: TDEvent;
    FDatalink: TRDatalink;
    function GetDatasource: TDatasource;
    procedure SetDatasource( Value: TDatasource );
  protected
  public
    constructor Create( AOwner: TComponent ); override;
    destructor Destroy; override;
  published
    property Datasource: TDatasource read GetDatasource write SetDatasource;
    property OnDataEvent: TDEvent read FDataEvent write FDataEvent;
    property OnDataChange: TNotifyEvent read FOnDataChange write FOnDataChange;
    property OnEditingChange: TNotifyEvent read FOnEditingChange write FOnEditingChange;
    property OnActiveChange: TNotifyEvent read FOnActiveChange write FOnActiveChange;
  end;

  TRDatalink = class( TDatalink )
  private
    FParent: TrDataEvents;
  protected
    procedure DataEvent(Event: TDataEvent; Info: Longint); override;
    procedure ActiveChanged; override;
    procedure EditingChanged; override;
    procedure RecordChanged(Field: TField); override;
  public
    constructor Create( AOwner: TrDataEvents );  reintroduce;
  end;


procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Polaris DB', [TrDataEvents]);
end;

{ TrDataEvents }


{ TrDataEvents }

constructor TrDataEvents.Create(AOwner: TComponent);
begin
  inherited;
  FDatalink := TRDatalink.Create( Self );
end;

destructor TrDataEvents.Destroy;
begin
  FDatalink.Free;
  inherited;
end;

function TrDataEvents.GetDatasource: TDatasource;
begin
  Result := FDatalink.DataSource;
end;

procedure TrDataEvents.SetDatasource(Value: TDatasource);
begin
  if not (FDataLink.DataSourceFixed and (csLoading in ComponentState)) then
    FDataLink.DataSource := Value;
end;

{ TRDatalink }

procedure TRDatalink.ActiveChanged;
begin
  inherited;
  if Assigned( FParent.OnActiveChange ) then FParent.OnActiveChange( FParent );
end;

constructor TRDatalink.Create(AOwner: TrDataEvents);
begin
  inherited Create;
  FParent := AOwner;
end;

procedure TRDatalink.DataEvent(Event: TDataEvent; Info: Integer);
begin
  inherited;
  if Assigned( FParent.OnDataEvent ) then
     FParent.OnDataEvent( Datasource.Dataset, Event, Info );
end;

procedure TRDatalink.EditingChanged;
begin
  inherited;
  if Assigned( FParent.OnEditingChange ) then FParent.OnEditingChange( FParent );
end;

procedure TRDatalink.RecordChanged(Field: TField);
begin
  inherited;
  if Assigned( FParent.OnDataChange ) then FParent.OnDataChange( FParent );
end;

end.
