(*====================================================================
    File rSQLQuery.pas

    Components library for Delphi and BCB++.
     WinSock.
    WinSock interface.

    Copyright  1998-2008, PSTG
    http://www.polesoft.ru
    mailto:support@polesoft.ru

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

======================================================================
$Id: rMiniSock.pas,v 1.3 2008/09/05 09:00:45 savchenko Exp $
======================================================================*)

// based on Borland ScktComp.pas

{ *********************************************************************** }
{                                                                         }
{ Delphi Runtime Library                                                  }
{                                                                         }
{ Copyright (c) 1997-2001 Borland Software Corporation                    }
{                                                                         }
{ *********************************************************************** }

{*******************************************************}
{       Windows socket components                       }
{*******************************************************}

unit rMiniSock;

interface

uses
{$IFDEF MSWINDOWS}
  Windows, WinSock;
{$ENDIF}
{$IFDEF LINUX}
  Libc;
{$ENDIF}

type
  TrMiniSock = class
  private
    FSocket: TSocket;
    FAddr: TSockAddrIn;
    FLastError: Integer;
    function LookupName(const aName: string): TInAddr;
    function _GetError: Integer;
  public
    function Connect(const aHost: string; const aPort: WORD): Integer;
    function Disconnect: Integer;
    function ReadString(const aLen: Integer): string;
    function WriteString(const aStr: string): Integer;
    procedure BeforeDestruction; override;
    //
    property LastError: Integer read FLastError;
  end;

implementation

{$IFDEF LINUX}
type
  BOOL = LongBool;
{$ENDIF}

{ TrMiniSock }

function TrMiniSock._GetError: Integer;
begin
  Result :=
    {$IFDEF MSWINDOWS}
    WSAGetLastError
    {$ENDIF}
    {$IFDEF LINUX}
    errno
    {$ENDIF}
    ;
end;

function TrMiniSock.LookupName(const aName: string): TInAddr;
var
  HostEnt: PHostEnt;
  InAddr: TInAddr;
begin
  HostEnt := gethostbyname(PChar(aName));
  FillChar(InAddr, SizeOf(InAddr), 0);
  if HostEnt <> nil then
  begin
    with InAddr, HostEnt^ do
    begin
      {$IFDEF MSWINDOWS}
      S_un_b.s_b1 := h_addr^[0];
      S_un_b.s_b2 := h_addr^[1];
      S_un_b.s_b3 := h_addr^[2];
      S_un_b.s_b4 := h_addr^[3];
      {$ENDIF}
      {$IFDEF LINUX}
      S_un_b.s_b1 := Ord(h_addr^[0]);
      S_un_b.s_b2 := Ord(h_addr^[1]);
      S_un_b.s_b3 := Ord(h_addr^[2]);
      S_un_b.s_b4 := Ord(h_addr^[3]);
      {$ENDIF}
    end;
  end;
  Result := InAddr;
end;

function TrMiniSock.Connect(const aHost: string; const aPort: WORD): Integer;
begin
  FSocket := socket(PF_INET, SOCK_STREAM, IPPROTO_IP);
  if FSocket = INVALID_SOCKET then begin
    FLastError := _GetError;
    Result := FLastError;
    Exit;
  end;
  FAddr.sin_family := PF_INET;
  FAddr.sin_addr := LookupName(aHost);
  FAddr.sin_port := htons(aPort);
  Result := {$IFDEF MSWINDOWS} WinSock {$ENDIF}
            {$IFDEF LINUX}     Libc    {$ENDIF}
    .connect(FSocket, FAddr, SizeOf(FAddr));
  if BOOL(Result) then begin
    Result := _GetError;
    FLastError := Result;
  end;
end;

function TrMiniSock.Disconnect: Integer;
begin
  {$IFDEF MSWINDOWS}
  Result := closesocket(FSocket);
  {$ENDIF}
  {$IFDEF LINUX}
  Result := shutdown(FSocket, SHUT_RDWR);
  {$ENDIF}
  if BOOL(Result) then Result := _GetError;
end;

function TrMiniSock.ReadString(const aLen: Integer): string;
begin
  SetLength(Result, aLen);
  recv(FSocket, PChar(Result)^, aLen, 0);
  FLastError := 0;
end;

function TrMiniSock.WriteString(const aStr: string): Integer;
begin
  FLastError := 0;
  if send(FSocket, PChar(aStr)^, Length(aStr), 0) = SOCKET_ERROR
    then FLastError := _GetError;
  Result := FLastError;
end;

procedure TrMiniSock.BeforeDestruction;
begin
  inherited;
end;

{$IFDEF MSWINDOWS}
var
  vSData: TWSAData;

initialization
  WSAStartup($101, vSData);

finalization
  WSACleanup;
{$ENDIF}

end.

