(*====================================================================
    File rPeriodDlg.pas

    Components library for Delphi, Kylix and BCB++.
    Dates period choice dialog.

    Copyright  1998-2008, PSTG
    http://www.polesoft.ru
    mailto:support@polesoft.ru

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

======================================================================
$Id: rPeriodDlg.pas,v 1.3 2008/03/31 15:12:37 ariafan Exp $
======================================================================*)
unit rPeriodDlg;

{$I POLARIS.INC}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Mask, DateUtil, rUtils, rButtons, Buttons, ExtCtrls, rPeriod,
  ToolEdit, rSpeedButton, rConst, RXSpin, RXCtrls, rVCLUtils;

type
  TFormBandDate = class(TForm)
    RadioGroup1: TRadioGroup;
    SpinYear: TRxSpinEdit;
    ComboHalfYear: TComboBox;
    ComboQuart: TComboBox;
    BitBtn1: TrBitBtn;
    BitBtn2: TBitBtn;
    ComboMonth: TComboBox;
    SpinWeek: TRxSpinEdit;
    GroupBox1: TGroupBox;
    Label1: TLabel;
    Label2: TLabel;
    Bulb: TrSpeedButton;
    DateEdit1: TDateEdit;
    DateEdit2: TDateEdit;
    Label4: TLabel;
    Label5: TLabel;
    BitBtn3: TBitBtn;
    procedure DateEditExit(Sender: TObject);
    procedure ComboHalfYearChange(Sender: TObject);
    procedure SpinYearChange(Sender: TObject);
    procedure ComboQuartChange(Sender: TObject);
    procedure ComboMonthChange(Sender: TObject);
    procedure SpinWeekChange(Sender: TObject);
    procedure RadioGroup1Click(Sender: TObject);
    procedure BulbClick(Sender: TObject);
    procedure DateEdit1AcceptDate(Sender: TObject; var Date: TDateTime;
      var Action: Boolean);
    procedure BitBtn1Click(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FormDestroy(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    function FormHelp(Command: Word; Data: Integer;
      var CallHelp: Boolean): Boolean;
  private
    FPeriod: TrPeriod; //  
    DMY: TrDate;
    procedure SetCtrlEnabled;
    procedure SetCtrlValues;
    procedure RefreshDateEdits;
  public
    D1,D2: TDate;      //  
    constructor CreatePeriod(AOwner: TComponent; AD1,AD2: TDate);
  end;

var
  FormBandDate: TFormBandDate;

function EditPeriod(dPeriod: TrPeriod): Boolean;

implementation

{$R *.DFM}

function EditPeriod(dPeriod: TrPeriod): Boolean;
begin
  with TFormBandDate.CreatePeriod(Application,dPeriod.D1,dPeriod.D2) do
    try
      Result := ShowModal = mrOK;
      if Result then begin
        dPeriod.D1 := D1;
        dPeriod.D2 := D2;
      end;
    finally
      Free;
    end;
end;

{ TFormBandDate }

constructor TFormBandDate.CreatePeriod(AOwner: TComponent; AD1,AD2: TDate);
var
  i: Integer;
begin
  FPeriod := TrPeriod.Create;
  FPeriod.D1 := AD1;
  FPeriod.D2 := AD2;
  D1 := AD1;
  D2 := AD2;
  inherited Create(AOwner);
  for i:=1 to 2 do ComboHalfYear.Items.Add(HalfYearItems[i]);
  for i:=1 to 4 do ComboQuart.Items.Add(QuartItems[i]);
  for i:=1 to 12 do ComboMonth.Items.Add(LongMonthNames[i]);
  SetCtrlValues;
  RefreshDateEdits;
end;

procedure TFormBandDate.SetCtrlEnabled;
begin
  with RadioGroup1  do begin
    SpinYear.Enabled := (ItemIndex<>Ord(bdAny))and(ItemIndex<>Ord(bdWeek));
    ComboHalfYear.Enabled := (ItemIndex=Ord(bdHalfYear));
    ComboQuart.Enabled := (ItemIndex=Ord(bdQuart));
    ComboMonth.Enabled := (ItemIndex in [Ord(bdMonth),Ord(bdDay)]);
    SpinWeek.Enabled := (ItemIndex=Ord(bdWeek));
  end;
end;

procedure TFormBandDate.SetCtrlValues;
var
  n: Integer;
begin
  with DMY do
    if FPeriod.D2 = 0 then DecodeDate(SysUtils.Date, Y, M, D)
    else DecodeDate(FPeriod.D2, Y, M, D);
  with DMY do begin
    SpinYear.Value := Y;
    ComboHalfYear.ItemIndex := M div 7;
    ComboQuart.ItemIndex := (M-1) div 3;
    ComboMonth.ItemIndex := M-1;
    n := Trunc(FPeriod.D2 - EncodeDate(Y,1,1)) div 7 + 1;
    SpinWeek.Value := IIF(n=53, 1, n);
//    Trunc((FPeriod.D2-
//            (EncodeDate(Y,1,1) - DayOfWeek(EncodeDate(Y,1,1))+3))/7);
  end;
  RadioGroup1.ItemIndex := Ord(FPeriod.GetType);
  with DMY do DecodeDate(FPeriod.D1, Y, M, D);
  Bulb.Down := (DMY.D = 1) and (DMY.M = 1);
end;

procedure TFormBandDate.RefreshDateEdits;
begin
  DateEdit1.Date := FPeriod.D1;
  DateEdit2.Date := FPeriod.D2;
end;

procedure TFormBandDate.DateEditExit(Sender: TObject);
begin
  if Sender = DateEdit1 then
    FPeriod.D1 := TDateEdit(Sender).Date
  else begin
    FPeriod.D2 := TDateEdit(Sender).Date;
    if RadioGroup1.ItemIndex=5 then begin
      FPeriod.D1 := FPeriod.D2;
      DateEdit1.Date := FPeriod.D2;
    end;
  end;
  SetCtrlValues;
  SetCtrlEnabled;
end;

procedure TFormBandDate.SpinYearChange(Sender: TObject);
var w: Word;
begin
  if RadioGroup1.ItemIndex = Ord(bdYear) then
    FPeriod.AssignNum(SpinYear.AsInteger, SpinYear.AsInteger, bdYear)
  else if SpinYear.Enabled then begin
    if RadioGroup1.ItemIndex = 5 then
      with DMY do begin
        Y := SpinYear.AsInteger;
        DecodeDate(FPeriod.D1, w, M, D);
        if D > DaysPerMonth(Y, M) then D := DaysPerMonth(Y, M);
        FPeriod.D1 := EncodeDate(Y, M, D);
        DecodeDate(FPeriod.D2, w, M, D);
        if D > DaysPerMonth(Y, M) then D := DaysPerMonth(Y, M);
        FPeriod.D2 := EncodeDate(Y, M, D);
      end
    else
      RadioGroup1.OnClick(RadioGroup1);
  end;
  RefreshDateEdits;
end;

procedure TFormBandDate.ComboHalfYearChange(Sender: TObject);
begin
  with ComboHalfYear do begin
    Bulb.Down := (ItemIndex=0);
    ComboQuart.ItemIndex := ItemIndex*2+1;
    ComboMonth.ItemIndex := ItemIndex*6+5;
    FPeriod.AssignNum(ItemIndex+1, SpinYear.AsInteger, bdHalfYear);
  end;
  RefreshDateEdits;
end;

procedure TFormBandDate.ComboQuartChange(Sender: TObject);
begin
  with ComboQuart do begin
    Bulb.Down := (ItemIndex=0);
    ComboHalfYear.ItemIndex := ItemIndex div 2;
    ComboMonth.ItemIndex := ItemIndex*3+2;
    FPeriod.AssignNum(ItemIndex+1, SpinYear.AsInteger, bdQuart);
  end;
  RefreshDateEdits;
end;

procedure TFormBandDate.ComboMonthChange(Sender: TObject);
var w: Word;
begin
  if RadioGroup1.ItemIndex = Ord(bdMonth) then
    with ComboMonth do begin
      Bulb.Down := (ItemIndex=0);
      ComboHalfYear.ItemIndex := ItemIndex div 6;
      ComboQuart.ItemIndex := ItemIndex div 3;
      FPeriod.AssignNum(ItemIndex+1, SpinYear.AsInteger, bdMonth);
    end
  else begin
    with DMY do begin
      M := ComboMonth.ItemIndex+1;
      DecodeDate(FPeriod.D1, Y, w, D);
      if D > DaysPerMonth(Y, M) then D := DaysPerMonth(Y, M);
      FPeriod.D1 := EncodeDate(Y, M, D);
      DecodeDate(FPeriod.D2, Y, w, D);
      if D > DaysPerMonth(Y, M) then D := DaysPerMonth(Y, M);
      FPeriod.D2 := EncodeDate(Y, M, D);
    end;
  end;
  RefreshDateEdits;
end;

procedure TFormBandDate.SpinWeekChange(Sender: TObject);
begin
  if RadioGroup1.ItemIndex = Ord(bdWeek) then begin
    FPeriod.AssignNum(SpinWeek.AsInteger, SpinYear.AsInteger, bdWeek);
    with DMY do begin
      DecodeDate(FPeriod.D2, Y, M, D);
      ComboHalfYear.ItemIndex := M div 6;
      ComboQuart.ItemIndex := M div 3;
    end;
    SetCtrlValues;
    RefreshDateEdits;
  end;
end;

procedure TFormBandDate.RadioGroup1Click(Sender: TObject);
begin
  SetCtrlEnabled;
  DateEdit1.Enabled := RadioGroup1.ItemIndex <> 5;
  case RadioGroup1.ItemIndex of
    0: begin
         SpinYearChange(Sender);
         Bulb.Down := True;
       end;
    1: ComboHalfYearChange(Sender);
    2: ComboQuartChange(Sender);
    3: ComboMonthChange(Sender);
    4: SpinWeekChange(Sender);
    5: begin
         FPeriod.D1 := FPeriod.D2;
         with DMY do begin
           DecodeDate(FPeriod.D1, Y, M, D);
           Bulb.Down := (D=1) and (M=1);
         end;
         RefreshDateEdits;
       end;
  end;
end;

procedure TFormBandDate.BulbClick(Sender: TObject);
begin
  if Bulb.Down then begin
    with DMY do begin
      D := 1;
      M := 1;
      FPeriod.D1 := EncodeDate(SpinYear.AsInteger, M, D);
    end;
    SetCtrlValues;
    SetCtrlEnabled;
    RefreshDateEdits;
  end else Bulb.Down := True;
end;

procedure TFormBandDate.DateEdit1AcceptDate(Sender: TObject;
  var Date: TDateTime; var Action: Boolean);
begin
  if Action then begin
    TDateEdit(Sender).Date := Date;
    DateEditExit(Sender);
  end;
end;

procedure TFormBandDate.BitBtn1Click(Sender: TObject);
begin
  if FPeriod.D1 > Fperiod.D2 then begin
    rMsgBox(srPeriodErr, MB_OK+MB_ICONHAND);
    exit;
  end;
  D1 := FPeriod.D1;
  D2 := FPeriod.D2;
  ModalResult := mrOK;
end;

procedure TFormBandDate.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if (Key = VK_RETURN) and (ssCtrl in Shift) then BitBtn1.Click;
end;

procedure TFormBandDate.FormDestroy(Sender: TObject);
begin
  FPeriod.Free;
end;

procedure TFormBandDate.FormCreate(Sender: TObject);
begin
  HelpFile := srHelpFile;
end;

function TFormBandDate.FormHelp(Command: Word; Data: Integer;
  var CallHelp: Boolean): Boolean;
begin
  CallHelp := not ((Command = HELP_CONTEXTPOPUP) and (Data = HelpContext));
  Result := True;
end;

end.
