(*====================================================================
    File rSetupFields.pas

    Components library for Delphi and BCB++.
       TDBGrid   TDataset.
    Settings form for TDBGrid or TDataset fields.

    Copyright  1998-2008, PSTG
    http://www.polesoft.ru
    mailto:support@polesoft.ru

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

======================================================================
$Id: rSetupFields.pas,v 1.9 2008/09/05 09:36:30 savchenko Exp $
======================================================================*)
unit rSetupFields;

{$I POLARIS.INC}

interface  

uses Forms, Controls, Windows, Classes, Db, dbgrids, SysUtils, Registry,
     {$IFDEF USE_TRXDBGRID}RXDBCtrl,{$ENDIF}
     Graphics, Dialogs, ExtCtrls, StdCtrls, RXCombos, Buttons,
     rButtons, rConst, rSpeedButton, rDBConst, rUtils, rVCLUtils, Mask, RXSpin,
     {$IFDEF POLARIS_D6}RxStrUtils{$ELSE}StrUtils{$ENDIF};

type
  TrSFDataLink = class;

  TrSetupFields = class(TComponent)
  private
    FRegKey, FGridKey:  String;
    FDataLink: TrSFDataLink;
    FPrevDataSet: TDataSet;
    FDBGrid: TDBGrid;
    FExcludeTag: Integer;
    FEnabled: Boolean;
    CharDelim: Char;
    FRestSet: Boolean;
    FColsIsDefault: Boolean;
    FReg: TRegistry;
    function  GetDataSource: TDataSource;
    procedure SetDataSource(Value: TDataSource);
    procedure SetDBGrid(Value: TDBGrid);
    procedure SetRegKey(Value: String);
    function CheckStoreDS: Boolean;
    procedure SetDataLinkDS(DS: TDataSource);
  protected
    procedure ActiveChanged;
    procedure Notification(AComponent: TComponent;
                           Operation: TOperation); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Loaded; override;
    procedure Execute;
    procedure SetupColumns;
    property Enabled: Boolean read FEnabled;
    procedure SaveDBGridSettings(SDBGrid : TDBGrid);
    procedure RestoreDefaults;
    property RestSet: Boolean read FRestSet;
    property ColsIsDefault: Boolean read FColsIsDefault;
    property GridKey: String read FGridKey;
  published
    property RegKey: String read FRegKey write SetRegKey;
    property DataSource: TDataSource read GetDataSource write SetDataSource stored CheckStoreDS;
    property DBGrid: TDBGrid read FDBGrid write SetDBGrid;
    property ExcludeTag: Integer read FExcludeTag write FExcludeTag default 1;
  end;

  TrSFDataLink = class(TDataLink)
  private
    FrSF: TrSetupFields;
  protected
    procedure ActiveChanged; override;
  public
    constructor Create(ArSF: TrSetupFields);
    destructor Destroy; override;
  end;

  TrDualListFldDlg = class(TForm)
    GroupBox1: TGroupBox;
    SrcLabel: TLabel;
    SrcList: TListBox;
    IncludeBtn: TrBitBtn;
    IncAllBtn: TrBitBtn;
    ExcludeBtn: TrBitBtn;
    ExAllBtn: TrBitBtn;
    DstLabel: TLabel;
    DstList: TListBox;
    OKBtn: TrBitBtn;
    UpBtn: TrBitBtn;
    DownBtn: TrBitBtn;
    FontDialog1: TFontDialog;
    CancelBtn: TrBitBtn;
    DefaultBtn: TrBitBtn;
    Props: TPanel;
    Label4: TLabel;
    Label2: TLabel;
    DisplayName: TEdit;
    Label1: TLabel;
    AlignBox: TComboBox;
    Label3: TLabel;
    ColorBox: TColorComboBox;
    Label5: TLabel;
    ColWidth: TRxSpinEdit;
    FontPanel: TPanel;
    FontBtn: TrBitBtn;
    Label6: TLabel;
    FixedSpin: TRxSpinEdit;
    BitBtn1: TBitBtn;
    RowSelect: TCheckBox;
    Label7: TLabel;
    Image1: TImage;
    Editing: TCheckBox;
    procedure IncludeBtnClick(Sender: TObject);
    procedure ExcludeBtnClick(Sender: TObject);
    procedure IncAllBtnClick(Sender: TObject);
    procedure ExcAllBtnClick(Sender: TObject);
    procedure MoveSelected(List: TCustomListBox; Items: TStrings);
    procedure SetItem(List: TListBox; Index: Integer);
    function GetFirstSelection(List: TCustomListBox): Integer;
    procedure SetButtons;
    procedure DstListClick(Sender: TObject);
    procedure AlignBoxChange(Sender: TObject);
    procedure ColorBoxChange(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure UpBtnClick(Sender: TObject);
    procedure DownBtnClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FixedSpinChange(Sender: TObject);
    procedure SrcListEnter(Sender: TObject);
    procedure DstListEnter(Sender: TObject);
    procedure ColWidthChange(Sender: TObject);
    procedure FontBtnClick(Sender: TObject);
    procedure DisplayNameChange(Sender: TObject);
    procedure DisplayNameEnter(Sender: TObject);
    procedure DefaultBtnClick(Sender: TObject);
    function FormHelp(Command: Word; Data: Integer;
      var CallHelp: Boolean): Boolean;
    procedure RowSelectClick(Sender: TObject);
    procedure AlignBoxEnter(Sender: TObject);
    procedure ColWidthEnter(Sender: TObject);
    procedure DstListDragOver(Sender, Source: TObject; X, Y: Integer;
      State: TDragState; var Accept: Boolean);
    procedure DstListDragDrop(Sender, Source: TObject; X, Y: Integer);
    procedure SrcListDragOver(Sender, Source: TObject; X, Y: Integer;
      State: TDragState; var Accept: Boolean);
    procedure SrcListDragDrop(Sender, Source: TObject; X, Y: Integer);
    procedure DstListMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
  private
    CanChange : Boolean;
    function GetFontSize(ColumnIndex : Integer) : Integer;
    procedure SetFixedSpin;
  public
    FSrcGrid : TDBGrid;
    ClearFields : Boolean;
    { Public declarations }
  end;

var
  rDualListFldDlg: TrDualListFldDlg;

implementation

{$R *.DFM}

{$IFDEF USE_TRDBGRID}uses rDBGrid;{$ENDIF}

function GetFixedCols(FGrid: TCustomDBGrid): Integer;
begin
  Result := 0;
{$IFDEF USE_TRXDBGRID}
  if (FGrid <> nil) and (FGrid is TRxDBGrid) then begin
    {$IFDEF USE_TRDBGRID}
    if (FGrid is TrDBGrid) then
      Result := (FGrid as TrDBGrid).FixedCols
    else
    {$ENDIF}
      Result := (FGrid as TRxDBGrid).FixedCols;
  end;
{$ENDIF}
end;

procedure SetFixedCols(var FGrid: TCustomDBGrid; Value: Integer);
begin
{$IFDEF USE_TRXDBGRID}
  if (FGrid <> nil) and (FGrid is TRxDBGrid) then begin
    {$IFDEF USE_TRDBGRID}
    if (FGrid is TrDBGrid) then
      (FGrid as TrDBGrid).FixedCols := Value
    else
    {$ENDIF}
      (FGrid as TRxDBGrid).FixedCols := Value;
  end;
{$ENDIF}
end;

{ TrSFDataLink }

constructor TrSFDataLink.Create(ArSF: TrSetupFields);
begin
  inherited Create;
  FrSF := ArSF;
end;

destructor TrSFDataLink.Destroy;
begin
  FrSF := nil;
  inherited Destroy;
end;

procedure TrSFDataLink.ActiveChanged;
begin
  if FrSF <> nil then FrSF.ActiveChanged;
end;

{ TrSetupFields }

constructor TrSetupFields.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FDataLink := TrSFDataLink.Create(Self);
  FExcludeTag := 1;
  FEnabled := False;
  FRestSet := False;
  FColsIsDefault := True;
  FPrevDataSet := nil;
  FReg := TRegistry.Create;
end;

destructor TrSetupFields.Destroy;
begin
  FDBGrid := nil;
  FDataLink.Free;
  FDataLink := nil;
  FReg.Free;
  inherited Destroy;
end;

procedure TrSetupFields.Loaded;
begin
  inherited Loaded;
  if (FDBGrid <> nil ) then SetDataLinkDS(FDBGrid.DataSource);
end;

procedure TrSetupFields.Notification(AComponent: TComponent;Operation: TOperation);
begin
  inherited;
  if (Operation = opRemove) then
    if (FDataLink <> nil) and (FDataLink.DataSource <> nil) and (AComponent = FDataLink.DataSource) then
      FDataLink.DataSource := nil
    else if (FDBGrid <> nil) and (AComponent = FDBGrid) then
           FDBGrid := nil;
end;

procedure TrSetupFields.ActiveChanged;
{var
  i:  Integer;}
begin
  if FDataLink.Active and (FPrevDataSet <> FDataLink.DataSet) then begin
    if (FDBGrid <> nil) and FRestSet then begin
{$IFDEF USE_TRDBGRID}
      {if (FDBGrid is TrDBGrid) and (TrDBGrid(FDBGrid).RealChecked) then
        for i := FDBGrid.Columns.Count - 1 downto 1 do FDBGrid.Columns[i].Destroy
      else}
{$ENDIF}
      FDBGrid.Columns.Clear;
    end;
    SetRegKey(FRegKey);
    SetupColumns;
    {$IFDEF USE_TRDBGRID}
    if not FColsIsDefault then
      if Assigned(FDBGrid) and (fDBGrid is TrDBGrid) and Assigned(TrDBGrid(FDBGrid).OnAfterSetupColumns) then
        TrDBGrid(FDBGrid).OnAfterSetupColumns(FDBGrid, mrNone);
    {$ENDIF}
    FPrevDataSet := FDataLink.DataSet;
  end;
end;

function TrSetupFields.GetDataSource: TDataSource;
begin
  Result := FDataLink.DataSource;
end;

procedure TrSetupFields.SetDataLinkDS(DS: TDataSource);
begin
  if not (FDataLink.DataSourceFixed and (csLoading in ComponentState)) then
    FDataLink.DataSource := DS;
end;

procedure TrSetupFields.SetDataSource(Value: TDataSource);
begin
  if (FDataLink.DataSource <> Value) then begin
    if (FDBGrid <> nil) and (Value <> nil) then FDBGrid := nil;
    SetDataLinkDS(Value);
    if Value <> nil then begin
      Value.FreeNotification(Self);
      {if Value.DataSet <> nil then
        if not (csDesigning in ComponentState) then SetupColumns;}
    end;
  end;
end;

procedure TrSetupFields.SetRegKey(Value: String);
var
  r: HKEY;
  b: Boolean;
  s: string;
begin
  b := FRegKey <> Value;
  FRegKey := Value;
  if FRegKey <> '' then begin
    if FDBGrid <> nil then
      FGridKey := AddSlash(FRegKey) + FDBGrid.Name
    else FGridKey := '';
    if (DataSource <> nil) and (DataSource.DataSet <> nil) then
    begin
      if FDBGrid = nil then FGridKey := AddSlash(FRegKey);
      FGridKey := FGridKey + ';' + DataSource.DataSet.Name
    end;
    if b then begin
      FPrevDataSet := nil;
      ActiveChanged;
    end;
  end;
  SeparateRegKey(FRegKey, r, s);
  FReg.RootKey := r;
  FEnabled := FReg.KeyExists(FGridKey);
  FPrevDataSet := nil;
end;

procedure TrSetupFields.SetDBGrid(Value: TDBGrid);
begin
  if FDataLink.DataSource <> nil then SetDataLinkDS(nil);
  FDBGrid := Value;
  if Value <> nil then begin
    if not (csLoading in ComponentState) then SetDataLinkDS(FDBGrid.DataSource);
    Value.FreeNotification(Self);
//    if not (csDesigning in ComponentState) then SetupColumns;
  end else
    SetDataLinkDS(nil);
end;

procedure TrSetupFields.Execute;
var FIndex, i, j: Integer;
    FLabel, ColumnKey : String;
    FDataSet: TDataSet;
    ValueNames: TStringList;
    OldColumns: TDBGridColumns;
    OldFixedCols: Integer;
    OldRowSelect, OldEditing: Boolean;
    DualListDlg: TrDualListFldDlg;
    Exists: Boolean;
    OldColumnsState: TDBGridColumnsState;
    r: HKEY;
    s: string;
begin
  FDataSet := nil;
  OldColumnsState := csDefault;
  if (FDataLink.DataSource <> nil) and (FDataLink.DataSource.DataSet <> nil) then
     FDataSet := FDataLink.DataSource.DataSet
  else
  if (FDBGrid <> nil) and (FDBGrid.DataSource <> nil) and
     (FDBGrid.DataSource.DataSet <> nil) then
     FDataSet := FDBGrid.DataSource.DataSet;
  if (FDataSet <> nil) and (FDataSet.Active) then begin
    DualListDlg := TrDualListFldDlg.Create(Self);
    DualListDlg.FSrcGrid := FDBGrid;
    OldColumns := nil;
    OldFixedCols := 0;
    OldRowSelect := False;
    OldEditing := False;
    try
      if (FDBGrid <> nil) then begin  //  DBGrid
        OldColumns := TDBGridColumns.Create(FDBGrid, TColumn);
        OldColumnsState := FDBGrid.Columns.State;
        OldColumns.Assign(FDBGrid.Columns);
        OldFixedCols := GetFixedCols(FDBGrid);
        OldRowSelect := dgRowSelect in FDBGrid.Options;
        OldEditing := dgEditing in FDBGrid.Options;
        if FDBGrid.Columns.State = csDefault then
          FDBGrid.Columns.Assign(OldColumns);
        SetFixedCols(TCustomDBGrid(FDBGrid), OldFixedCols);
        for i := 0 to FDBGrid.Columns.Count - 1 do begin
{$IFDEF USE_TRDBGRID}
          if not ((i = 0) and (FDBGrid is TrDBGrid)
            and (TrDBGrid(FDBGrid).RealChecked)) then
          begin
{$ENDIF}
            FLabel := FDBGrid.Columns[i].Title.Caption;
            DualListDlg.DstList.Items.AddObject(FLabel,TObject(FDBGrid.Columns[i]));
{$IFDEF USE_TRDBGRID}
          end;
{$ENDIF}
        end;
      end
      else                          //  DataSource
      begin
        FIndex := 0;

        for i := 0 to FDataSet.FieldCount - 1 do
          if FDataSet.Fields[i].Visible then begin
            FLabel := FDataSet.Fields[i].DisplayLabel;
            DualListDlg.DstList.Items.AddObject(FLabel,TObject(FDataSet.Fields[i]));
            FDataSet.Fields[i].Index := FIndex;
            Inc(FIndex);
          end;
      end;
      for i := 0 to FDataSet.FieldCount - 1 do begin
        if FDataSet.Fields[i].Tag <> FExcludeTag then begin
          Exists := False;
          for j := 0 to DualListDlg.DstList.Items.Count-1 do begin
            if (FDBGrid <> nil) then begin
              if (FDataSet.Fields[i].FieldName =
                    TColumn(DualListDlg.DstList.Items.Objects[j]).FieldName) then begin
                Exists := True;
                Break;
              end
            end
            else begin
              if (FDataSet.Fields[i].FieldName =
                      TField(DualListDlg.DstList.Items.Objects[j]).FieldName) then begin
                Exists := True;
                Break;
              end;
            end;
          end;
          if not Exists then
            DualListDlg.SrcList.Items.AddObject(FDataSet.Fields[i].DisplayLabel,
                                      TObject(FDataSet.Fields[i]));
        end;
      end;
      with DualListDlg, FReg do begin
        SetButtons;
        DefaultBtn.Enabled := not FColsIsDefault;
        if (ShowModal = mrOK) and (FGridKey <> '') then begin
          FRestSet := True;
          FColsIsDefault := not FRestSet;
          SeparateRegKey(FGridKey, r, s);
          OpenKey(s, True);
          ValueNames := TStringList.Create;
          try
            GetValueNames(ValueNames);
            for i := 0 to ValueNames.Count-1 do DeleteValue(ValueNames[i]);

            if FDBGrid <> nil then begin

              WriteString(srFixedColsKey,IntToStr(FixedSpin.AsInteger));
{$IFDEF USE_TRDBGRID}
              if (FDBGrid is TrDBGrid) then
                WriteString(srDefaultRowHeight, IntToStr(TrDBGrid(FDBGrid).DefaultRowHeight));
{$ENDIF}
              WriteString(srRowSelectKey, IIF(RowSelect.Checked, '1', '0'));
              WriteString(srEditingKey, IIF(Editing.Checked, '1', '0'));

              OpenKey(s + srFieldsSubKey, True);
              GetValueNames(ValueNames);
              for i := 0 to ValueNames.Count - 1 do DeleteValue(ValueNames[i]);
              for i := 0 to DstList.Items.Count-1 do begin
                with TColumn(DstList.Items.Objects[i]) do begin
                  ColumnKey := Title.Caption + RegStringDelim + IntToStr(Ord(Alignment));
                  ColumnKey := ColumnKey + RegStringDelim + IntToStr(Width) +
                                  RegStringDelim + ColorToString(Color);

                  ColumnKey := ColumnKey + RegStringDelim;
                  if fsBold in Font.Style then ColumnKey := ColumnKey + 'B';
                  if fsItalic in Font.Style then ColumnKey := ColumnKey + 'I';
                  if fsUnderline in Font.Style then ColumnKey := ColumnKey + 'U';
                  if fsStrikeOut in Font.Style then ColumnKey := ColumnKey + 'S';
                  ColumnKey := ColumnKey + RegStringDelim + IntToStr(Font.Size) +
                                  RegStringDelim + ColorToString(Font.Color) +
                                  RegStringDelim + Font.Name;

                  ColumnKey := ColumnKey + RegStringDelim + PickList.CommaText +
                                           RegStringDelim + IntToStr(Ord(ButtonStyle)) +
                                           RegStringDelim + IntToStr(DropDownRows);
                  WriteString(FieldName, ColumnKey);
                  FEnabled := True;
                end;
              end;

            end
            else    //    DataSource
            begin

              OpenKey(s + srFieldsSubKey, True);
              for i := 0 to DstList.Items.Count-1 do begin
                with TField(DstList.Items.Objects[i]) do begin
                  ColumnKey := DisplayLabel + RegStringDelim + IntToStr(Ord(Alignment));
                  ColumnKey := ColumnKey + RegStringDelim + IntToStr(DisplayWidth);
                  WriteString(FieldName, ColumnKey);
                  FEnabled := True;
                end;
              end;
            end;
          finally
            CloseKey;
            ValueNames.Free;
          end;
        end
        else begin
          if ClearFields then
            with FReg do begin
              ModalResult := mrNone;
              SeparateRegKey(FGridKey, r, s);
              OpenKey(s, False);
              ValueNames := TStringList.Create;
              try
                GetValueNames(ValueNames);
                for i := 0 to ValueNames.Count - 1 do DeleteValue(ValueNames[i]);
                OpenKey(s + srFieldsSubKey, False);
                GetValueNames(ValueNames);
                for i := 0 to ValueNames.Count - 1 do DeleteValue(ValueNames[i]);

                if (FDBGrid <> nil) then begin
                  if FRestSet or (OldColumnsState = csDefault) then begin
                    RestoreDefaults;
{$IFDEF USE_TRDBGRID}
                    if TrDBGrid(FDBGrid).RealChecked then
                      TrDBGrid(FDBGrid).CreateNewCheckColumn;
//                    TrDBGrid(FDBGrid).CheckType := TrDBGrid(FDBGrid).CheckType;
                    TrDBGrid(FDBGrid).FixedCols := OldFixedCols;
{$ENDIF}
                  end
                  else FDBGrid.Columns.Assign(OldColumns);
                  FRestSet := False;
                  FColsIsDefault := not FRestSet;
                end;
              finally
                ValueNames.Free;
                CloseKey;
              end;
            end
          else if (FDBGrid <> nil) then begin
            {if (OldColumnsState = csDefault) then FDBGrid.Columns.Clear
            else} FDBGrid.Columns.Assign(OldColumns);
            if (OldColumnsState = csDefault) then FRestSet := True;
            SetFixedCols(TCustomDBGrid(FDBGrid), OldFixedCols);
            with FDBGrid do
              if OldRowSelect then Options := Options + [dgRowSelect]
              else Options := Options - [dgRowSelect];
            with FDBGrid do
              if OldEditing then Options := Options + [dgEditing]
              else Options := Options - [dgEditing];
          end;
        end;
        {$IFDEF USE_TRDBGRID}
        if Assigned(FDBGrid) and (fDBGrid is TrDBGrid) and Assigned(TrDBGrid(FDBGrid).OnAfterSetupColumns) then
          TrDBGrid(FDBGrid).OnAfterSetupColumns(FDBGrid, DualListDlg.ModalResult);
        {$ENDIF}
      end;
    finally
      if OldColumns <> nil then OldColumns.Free;
      DualListDlg.Free;
    end;
  end;
end;

function TrSetupFields.CheckStoreDS : Boolean;
begin
  Result := (FDBGrid = nil);
end;

procedure TrSetupFields.SetupColumns;
var
  ValueNames: TStringList;
  NewColumn: TColumn;
  FFontStyle, Int, i, j, l: Integer;
  ColumnKey, rkey: String;
  F: TField;
  FDataSet: TDataSet;
  r: HKEY;
  s: string;
begin
  SeparateRegKey(FGridKey, r, rkey);
  if (FGridKey <> '') and FReg.KeyExists(rkey) then begin
    FDataSet := nil;
    if (FDataLink.DataSource <> nil) and (FDataLink.DataSource.DataSet <> nil) then
       FDataSet := FDataLink.DataSource.DataSet;
    if (FDataSet <> nil) then begin
      ValueNames := TStringList.Create;
      try
        if FReg.OpenKey(rkey + srFieldsSubKey, False) then
          FReg.GetValueNames(ValueNames)
        else
          if FReg.OpenKey(rkey, False) then begin
            FReg.GetValueNames(ValueNames);
            for i := ValueNames.Count - 1 downto 0  do begin
              ColumnKey := FReg.ReadString(ValueNames[i]);
              if (Pos(#13, ColumnKey) = 0) and
                 (Pos(RegStringDelim, ColumnKey) = 0) then
                ValueNames.Delete(i);
            end;
          end;
        FRestSet := ValueNames.Count > 0;
        FColsIsDefault := not FRestSet;

        if FDBgrid <> nil then begin  //  DBGrid
          if FRestSet then begin
{$IFDEF USE_TRDBGRID}
            if (FDBGrid is TrDBGrid) and (TrDBGrid(FDBGrid).RealChecked) then
            begin
              if FDBGrid.Columns.State = csDefault then
                TrDBGrid(FDBGrid).CreateNewCheckColumn
              else
                for i := FDBGrid.Columns.Count - 1 downto 1 do FDBGrid.Columns[i].Destroy;
            end
            else
{$ENDIF}
              FDBGrid.Columns.Clear;
          end;

          for i := 0 to ValueNames.Count - 1 do begin
            F := FDBGrid.DataSource.DataSet.FindField(ValueNames[i]);
            if (F <> nil) and (F.Tag <> FExcludeTag) then begin
              NewColumn := FDBGrid.Columns.Add;
              NewColumn.FieldName := ValueNames[i];
              ColumnKey := FReg.ReadString(ValueNames[i]);
              if Pos(#13, ColumnKey) <> 0 then CharDelim := #13 //  
              else CharDelim := RegStringDelim;

              l := GetDelimCount(ColumnKey, [CharDelim]);
              for j := 1 to l + 1 do begin
                s := ExtractDelimited(j, ColumnKey, [CharDelim]);
                if s <> EmptyStr then
                  case j of
                    1: NewColumn.Title.Caption := s;
                    2: NewColumn.Alignment := TAlignment(StrToIntDef(s, 0));
                    3: NewColumn.Width := StrToIntDef(s, 80);
                    4: NewColumn.Color := StringToColor(s);
                    5: begin
                         FFontStyle := StrToIntDef(s, -1);
                         with NewColumn.Font do
                           case FFontStyle of
                            -1: begin
                                  Style := [];
                                  if Pos('B', s) <> 0 then Style := Style + [fsBold];
                                  if Pos('I', s) <> 0 then Style := Style + [fsItalic];
                                  if Pos('U', s) <> 0 then Style := Style + [fsUnderline];
                                  if Pos('S', s) <> 0 then Style := Style + [fsStrikeOut];
                                end;
                             0: Style := [];
                             1: Style := Style + [fsBold];
                             2: Style := Style + [fsItalic];
                             3: Style := Style + [fsBold,fsItalic];
                           end;
                       end;
                    6: NewColumn.Font.Size := StrToIntDef(s, 8);
                    7: NewColumn.Font.Color := StringToColor(s);
                    8: NewColumn.Font.Name := s;
                    9: NewColumn.PickList.CommaText := s;
                   10: NewColumn.ButtonStyle := TColumnButtonStyle(StrToIntDef(s, 0));
                   11: NewColumn.DropDownRows := StrToIntDef(s, 7);
                  end;
              end;
            end;
          end;

          FReg.OpenKey(rkey, False);
          if FReg.ValueExists(srRowSelectKey) then
            with FDBGrid do
              if FReg.ReadString(srRowSelectKey) = '1' then Options := Options + [dgRowSelect]
              else Options := Options - [dgRowSelect];

          if FReg.ValueExists(srEditingKey) then
            with FDBGrid do
              if FReg.ReadString(srEditingKey) = '1' then Options := Options + [dgEditing]
              else Options := Options - [dgEditing];

          // FixedCols
          if FRestSet then begin
            Int := 0;
            if FReg.ValueExists(srFixedColsKey) then
              Int := StrToIntDef(FReg.ReadString(srFixedColsKey),0);
            SetFixedCols(TCustomDBGrid(FDBGrid), Int);
{$IFDEF USE_TRDBGRID}
            if (FDBGrid is TrDBGrid) then
              (FDBGrid as TrDBGrid).DefaultRowHeight := StrToIntDef(FReg.ReadString(srDefaultRowHeight),17);
{$ENDIF}
          end;
        end
        else if (ValueNames.Count <> 0) then begin  //   DataSource
          for i := 0 to FDataSet.FieldCount-1 do begin
            j := ValueNames.IndexOf(FDataSet.Fields[i].FieldName);
            if j <> -1 then begin
              FDataSet.Fields[i].Visible := True;
              ColumnKey := FReg.ReadString(ValueNames[j]);
              l := Length(ColumnKey);
              if Pos(#13, ColumnKey) <> 0 then CharDelim := #13 //  
              else CharDelim := RegStringDelim;

              Int := Pos(CharDelim, ColumnKey);
              if Int = 0 then Int := l+1;
              FDataSet.Fields[i].DisplayLabel := Copy(ColumnKey, 1, Int - 1);
              ColumnKey := Copy(ColumnKey, Int + 1, l);

              Int := Pos(CharDelim, ColumnKey);
              if Int = 0 then Int := l+1;
              FDataSet.Fields[i].Alignment :=
                TAlignment(StrToIntDef(Copy(ColumnKey, 1, Int - 1), 0));
              ColumnKey := Copy(ColumnKey,Int + 1, l);

              Int := Pos(CharDelim, ColumnKey);
              if Int = 0 then Int := l+1;
              FDataSet.Fields[i].DisplayWidth := StrToIntDef(Copy(ColumnKey, 1, Int - 1), 0);
            end
            else
              FDataSet.Fields[i].Visible := False;
          end;
        end;
      finally
        FReg.CloseKey;
        ValueNames.Free;
      end;
    end;
  end;
end;

procedure TrSetupFields.RestoreDefaults;

{$IFDEF POLARIS_D4}
  procedure AddFields(Fields: TFields; Depth: Integer);
  var
    I: Integer;
    Column: TColumn;
    B: Boolean;
  begin
    Inc(Depth);
    for I := 0 to Fields.Count - 1 do begin
      {$IFDEF USE_TRXDBGRID}
      B := Fields[I].Visible and (not TrDBGrid(FDBGrid).RealChecked
        or (Fields[I].FieldName <> TrDBGrid(FDBGrid).CheckFieldName));
      {$ELSE}
      B := Fields[I].Visible;
      {$ENDIF}
      if B then begin
        Column := FDBGrid.Columns.Add;
        Column.FieldName := Fields[I].FullName;
        Column.Width := Fields[i].DisplayWidth * FDBGrid.Canvas.TextWidth('0');
        if Fields[I].DataType in [ftADT, ftArray] then
          AddFields((Fields[I] as TObjectField).Fields, Depth);
      end;
    end;
  end;
{$ELSE}
  procedure AddFields(ADataSet: TDataSet; Depth: Integer);
  var
    I: Integer;
    Column: TColumn;
    B: Boolean;
  begin
    with ADataSet do
      for I := 0 to FieldCount - 1 do begin
        {$IFDEF USE_TRXDBGRID}
        B := Fields[I].Visible and (not TrDBGrid(FDBGrid).RealChecked
          or (Fields[I].FieldName <> TrDBGrid(FDBGrid).CheckFieldName));
        {$ELSE}
        B := Fields[I].Visible;
        {$ENDIF}
        if B then begin
          Column := FDBGrid.Columns.Add;
          Column.FieldName := Fields[I].FieldName;
          Column.Width := Fields[i].DisplayWidth * FDBGrid.Canvas.TextWidth('0');
        end;
      end;
  end;
{$ENDIF}

begin
  if Assigned(FDBGrid) and Assigned(FDBGrid.DataSource) and
    Assigned(FDBGrid.Datasource.Dataset) then
  begin
    FDBGrid.Columns.BeginUpDate;
    try
      FDBGrid.Columns.Clear;
      AddFields(FDBGrid.DataSource.DataSet{$IFDEF POLARIS_D4}.Fields{$ENDIF}, 0);
    finally
      FDBGrid.Columns.EndUpDate;
    end
  end
  else
    FDBGrid.Columns.Clear;
end;

procedure TrSetupFields.SaveDBGridSettings(SDBGrid: TDBGrid);
var
  i: Integer;
  ColumnKey: String;
  ValueNames: TStringList;
  r: HKEY;
  s: string;
begin
  if SDBGrid.Columns.State <> csDefault then
    SeparateRegKey(FGridKey, r, s);
    with FReg do begin
      OpenKey(s, True);
      ValueNames := TStringList.Create;
      try
        GetValueNames(ValueNames);
        for i := 0 to ValueNames.Count - 1 do DeleteValue(ValueNames[i]);
        WriteString(srFixedColsKey, IntToStr(GetFixedCols(SDBGrid)));
  {$IFDEF USE_TRDBGRID}
        if (SDBGrid is TrDBGrid) then
          WriteString(srDefaultRowHeight, IntToStr(TrDBGrid(SDBGrid).DefaultRowHeight));
  {$ENDIF}
        WriteString(srRowSelectKey, IIF((dgRowSelect in SDBGrid.Options), '1', '0'));
        WriteString(srEditingKey, IIF((dgEditing in SDBGrid.Options), '1', '0'));

        OpenKey(s + srFieldsSubKey, True);
        GetValueNames(ValueNames);
        for i := 0 to ValueNames.Count - 1 do DeleteValue(ValueNames[i]);
        for i := 0 to SDBGrid.Columns.Count - 1 do begin
          with SDBGrid.Columns[i] do begin
            ColumnKey := Title.Caption + RegStringDelim + IntToStr(Ord(Alignment));
            ColumnKey := ColumnKey + RegStringDelim + IntToStr(Width) +
                            RegStringDelim + ColorToString(Color);

            ColumnKey := ColumnKey + RegStringDelim;
            if fsBold in Font.Style then ColumnKey := ColumnKey + 'B';
            if fsItalic in Font.Style then ColumnKey := ColumnKey + 'I';
            if fsUnderline in Font.Style then ColumnKey := ColumnKey + 'U';
            if fsStrikeOut in Font.Style then ColumnKey := ColumnKey + 'S';
            ColumnKey := ColumnKey + RegStringDelim + IntToStr(Font.Size) +
                            RegStringDelim + ColorToString(Font.Color) +
                            RegStringDelim + Font.Name;

            ColumnKey := ColumnKey + RegStringDelim + PickList.CommaText +
                                     RegStringDelim + IntToStr(Ord(ButtonStyle)) +
                                     RegStringDelim + IntToStr(DropDownRows);

            WriteString(FieldName, ColumnKey);
          end;
        end;
      finally
        CloseKey;
        ValueNames.Free;
      end;
    end;
end;

{ TrDualListFldDlg }

procedure TrDualListFldDlg.SetFixedSpin;
begin
  with FixedSpin do
    if Visible then begin
      MaxValue := DstList.Items.Count - 1;
      Enabled :=
        {$IFDEF USE_TRXDBGRID}(FSrcGrid is TRxDBGrid) and (MaxValue>0){$ELSE}False{$ENDIF};
    end;
end;

procedure TrDualListFldDlg.IncludeBtnClick(Sender: TObject);
var
  Index: Integer;
begin
  Index := GetFirstSelection(SrcList);
  MoveSelected(SrcList, DstList.Items);
  SetItem(SrcList, Index);
end;

procedure TrDualListFldDlg.ExcludeBtnClick(Sender: TObject);
var
  Index: Integer;
begin
  Index := GetFirstSelection(DstList);
  MoveSelected(DstList, SrcList.Items);
  SetItem(DstList, Index);
  DstListClick(DstList);
end;

procedure TrDualListFldDlg.IncAllBtnClick(Sender: TObject);
var
  i: Integer;
  NewColumn: TColumn;
begin
  for i := 0 to SrcList.Items.Count - 1 do begin
    if FSrcGrid <> nil then begin
      NewColumn := FSrcGrid.Columns.Add;
      NewColumn.FieldName :=
                TField(SrcList.Items.Objects[i]).FieldName;
      SrcList.Items.Objects[i] := TObject(NewColumn);
    end
    else TField(SrcList.Items.Objects[i]).Visible := True;
    DstList.Items.AddObject(SrcList.Items[I], SrcList.Items.Objects[I]);
  end;
  SrcList.Items.Clear;
  SetItem(SrcList, 0);
  SetFixedSpin;
end;

procedure TrDualListFldDlg.ExcAllBtnClick(Sender: TObject);
var
  I: Integer;
begin
  for I := 0 to DstList.Items.Count - 1 do begin
    if FSrcGrid <> nil then DstList.Items.Objects[I] :=
       TObject(TField(FSrcGrid.DataSource.DataSet.FieldByName(TColumn(DstList.Items.Objects[i]).FieldName)))
    else TField(DstList.Items.Objects[i]).Visible := False;
    SrcList.Items.AddObject(DstList.Items[I], DstList.Items.Objects[I]);
  end;
  DstList.Items.Clear;
  if FSrcGrid <> nil then begin
{$IFDEF USE_TRDBGRID}
    if (FSrcGrid is TrDBGrid) and (TrDBGrid(FSrcGrid).RealChecked) then
      for i := FSrcGrid.Columns.Count - 1 downto 1 do FSrcGrid.Columns[i].Destroy
    else
{$ENDIF}
      FSrcGrid.Columns.Clear;
  end;
  SetItem(DstList, 0);
  with FixedSpin do
    if Visible then begin
      Value := 0;
      Enabled := False;
    end;
end;

procedure TrDualListFldDlg.MoveSelected(List: TCustomListBox; Items: TStrings);
var
  I: Integer;
  NewColumn: TColumn;
begin
  for I := List.Items.Count - 1 downto 0 do
    if List.Selected[I] then
    begin
      if FSrcGrid<>nil then begin
        if List = SrcList then begin
          NewColumn := FSrcGrid.Columns.Add;
          NewColumn.FieldName :=
                    TField(List.Items.Objects[i]).FieldName;
          NewColumn.Title.Caption := List.Items[i];
          List.Items.Objects[i] := TObject(NewColumn);
        end else begin
          NewColumn := TColumn(List.Items.Objects[i]);
          List.Items.Objects[i] :=
               TObject(TField(FSrcGrid.DataSource.DataSet.FieldByName(NewColumn.FieldName)));
          NewColumn.Destroy;
        end;
      end else
        TField(List.Items.Objects[i]).Visible := List <> DstList;
      Items.AddObject(List.Items[I], List.Items.Objects[I]);
      List.Items.Delete(I);
      SetFixedSpin;
    end;
end;

procedure TrDualListFldDlg.SetButtons;
var
  SrcEmpty, DstEmpty: Boolean;
begin
  SrcEmpty := SrcList.Items.Count = 0;
  DstEmpty := DstList.Items.Count = 0;
  IncludeBtn.Enabled := not SrcEmpty;
  IncAllBtn.Enabled := not SrcEmpty;
  ExcludeBtn.Enabled := not DstEmpty;
  ExAllBtn.Enabled := not DstEmpty;
end;

function TrDualListFldDlg.GetFirstSelection(List: TCustomListBox): Integer;
begin
  for Result := 0 to List.Items.Count - 1 do
    if List.Selected[Result] then Exit;
  Result := LB_ERR;
end;

procedure TrDualListFldDlg.SetItem(List: TListBox; Index: Integer);
var
  MaxIndex: Integer;
begin
  with List do begin
    SetFocus;
    MaxIndex := List.Items.Count - 1;
    if Index = LB_ERR then Index := 0
    else if Index > MaxIndex then Index := MaxIndex;
    Selected[Index] := True;
  end;
  SetButtons;
end;

procedure TrDualListFldDlg.DstListClick(Sender: TObject);
begin
  CanChange := True;
  with DstList do
    if (Items.Count<>0) then
      if (FSrcGrid<>nil) then begin
        AlignBox.ItemIndex := Ord(TColumn(Items.Objects[ItemIndex]).Alignment);
        DisplayName.Text := TColumn(Items.Objects[ItemIndex]).Title.Caption;
        ColorBox.ColorValue := TColumn(Items.Objects[ItemIndex]).Color;
        FontPanel.Font := TColumn(Items.Objects[ItemIndex]).Font;
        ColWidth.Value := Round(TColumn(Items.Objects[ItemIndex]).Width/
                          GetFontSize(ItemIndex));
        FixedSpin.Value := GetFixedCols(FSrcGrid);
        RowSelect.Checked := dgRowSelect in FSrcGrid.Options;
        Editing.Checked := dgEditing in FSrcGrid.Options;
      end
      else
      begin
        AlignBox.ItemIndex := Ord(TField(Items.Objects[ItemIndex]).Alignment);
        DisplayName.Text := TField(Items.Objects[ItemIndex]).DisplayLabel;
        ColWidth.Value := TField(Items.Objects[ItemIndex]).DisplayWidth;
      end;
end;

procedure TrDualListFldDlg.AlignBoxChange(Sender: TObject);
begin
  if not CanChange then
    if FSrcGrid <> nil then
      TColumn(DstList.Items.Objects[DstList.ItemIndex]).Alignment :=
           TAlignment(AlignBox.ItemIndex)
    else
      TField(DstList.Items.Objects[DstList.ItemIndex]).Alignment :=
           TAlignment(AlignBox.ItemIndex);
end;

procedure TrDualListFldDlg.ColorBoxChange(Sender: TObject);
begin
  if (not CanChange) and (FSrcGrid<>nil) then
    TColumn(DstList.Items.Objects[DstList.ItemIndex]).Color := ColorBox.ColorValue;
end;

procedure TrDualListFldDlg.FormActivate(Sender: TObject);
begin
  if FSrcGrid = nil then begin
    Label3.Visible := False;
    ColorBox.Visible := False;
    FontPanel.Visible := False;
    FontBtn.Visible := False;
    Label6.Visible := False;
    FixedSpin.Visible := False;
    Label7.Visible := False;
    RowSelect.Visible := False;
    Editing.Visible := False;
  end;
  with DstList do begin
    ItemIndex := 0;
    OnClick(DstList);
  end;
  SetFixedSpin;
end;

procedure TrDualListFldDlg.UpBtnClick(Sender: TObject);
var
  Index_: Integer;
begin

  with DstList do begin
    if ItemIndex > 0 then begin
      if FSrcGrid <> nil then begin
        {$IFDEF USE_TRDBGRID}
        if (FSrcGrid is TrDBGrid) then
          Index_ := Byte((FSrcGrid as TrDBGrid).RealChecked)
        else
        {$ENDIF}
          Index_ := 0;
        FSrcGrid.Columns[ItemIndex + Index_].Index := ItemIndex + Index_ - 1
      end
      else
        TField(DstList.Items.Objects[ItemIndex]).Index := ItemIndex - 1;
      Index_ := ItemIndex;
      Items.Move(ItemIndex, ItemIndex - 1);
      ItemIndex := Index_ - 1;
      SetItem(DstList, ItemIndex);
    end else
      UpBtn.Enabled := False;
    DownBtn.Enabled := ItemIndex < Items.Count - 1;
  end;
end;

procedure TrDualListFldDlg.DownBtnClick(Sender: TObject);
var
  Index_: Integer;
begin
  with DstList do begin
    if ItemIndex < Items.Count - 1 then begin
      if FSrcGrid <> nil then begin
        {$IFDEF USE_TRDBGRID}
        if (FSrcGrid is TrDBGrid) then
          Index_ := Byte((FSrcGrid as TrDBGrid).RealChecked)
        else
        {$ENDIF}
          Index_ := 0;
        FSrcGrid.Columns[ItemIndex + Index_].Index := ItemIndex + Index_ + 1
      end
      else
        TField(Items.Objects[ItemIndex]).Index := ItemIndex + 1;
      Index_ := ItemIndex;
      Items.Move(ItemIndex, ItemIndex + 1);
      ItemIndex := Index_ + 1;
      SetItem(DstList, ItemIndex);
    end else
      DownBtn.Enabled := False;
    UpBtn.Enabled := ItemIndex > 0;
  end;
end;

procedure TrDualListFldDlg.FormCreate(Sender: TObject);
begin
  HelpFile  := srHelpFile;
  CanChange := False;
  ClearFields := False;
{$IFNDEF POLARIS_D4}
  DstList.MultiSelect := False;
{$ENDIF}
end;

procedure TrDualListFldDlg.FixedSpinChange(Sender: TObject);
begin
  SetFixedCols(TCustomDBGrid(FSrcGrid), (Sender as TRxSpinEdit).AsInteger);
end;

procedure TrDualListFldDlg.SrcListEnter(Sender: TObject);
var
  i: Integer;
begin
  with Props do
  for i := 0 to ControlCount-1 do
    if not (Controls[i] is TLabel) then
      TControl(Controls[i]).Enabled := False;
end;

procedure TrDualListFldDlg.DstListEnter(Sender: TObject);
var
  i: Integer;
begin
  with Props do
  for i := 0 to ControlCount-1 do
    if not (Controls[i] is TLabel) then
      TControl(Controls[i]).Enabled := True;
end;

procedure TrDualListFldDlg.ColWidthChange(Sender: TObject);
var
  newWidth, oldWidth: Integer;
begin
  if not CanChange then
    with DstList do
      if FSrcGrid <> nil then begin
        newWidth := ColWidth.AsInteger*GetFontSize(ItemIndex);
        oldWidth := TColumn(Items.Objects[ItemIndex]).Width;
        if Abs(newWidth - oldWidth) > GetFontSize(ItemIndex) then
          TColumn(Items.Objects[ItemIndex]).Width := newWidth;
      end else
        TField(Items.Objects[ItemIndex]).DisplayWidth := ColWidth.AsInteger;
end;

procedure TrDualListFldDlg.FontBtnClick(Sender: TObject);
begin
  FontDialog1.Font := FontPanel.Font;
  if FontDialog1.Execute then begin
    TColumn(DstList.Items.Objects[DstList.ItemIndex]).Font := FontDialog1.Font;
    FontPanel.Font := FontDialog1.Font;
  end;
end;

procedure TrDualListFldDlg.DisplayNameChange(Sender: TObject);
begin
  if not CanChange then
    with DstList do begin
      Items[ItemIndex] := (Sender as TEdit).Text;
      if FSrcGrid<>nil then
        TColumn(Items.Objects[ItemIndex]).Title.Caption :=
                          (Sender as TEdit).Text
      else
        TField(Items.Objects[ItemIndex]).DisplayLabel :=
                          (Sender as TEdit).Text;
    end;
end;

procedure TrDualListFldDlg.DisplayNameEnter(Sender: TObject);
begin
  CanChange := False;
end;

procedure TrDualListFldDlg.DefaultBtnClick(Sender: TObject);
begin
  if FSrcGrid = nil then rMsgBox(srChangesAfterRestart, MB_OK);
  ClearFields := True;
{$IFDEF USE_TRDBGRID}
  if (FSrcGrid is TrDBGrid) then TrDBGrid(FSrcGrid).DefaultRowHeight := 17;
{$ENDIF}
  Close;
end;

function TrDualListFldDlg.GetFontSize(ColumnIndex : Integer ) : Integer;
begin
  Image1.Canvas.Font := FSrcGrid.Columns[ColumnIndex].Font;
  Result := Image1.Canvas.TextWidth('0');
end;

function TrDualListFldDlg.FormHelp(Command: Word; Data: Integer;
  var CallHelp: Boolean): Boolean;
begin
  CallHelp := not ((Command = HELP_CONTEXTPOPUP) and (Data = HelpContext));
  Result := True;
end;

procedure TrDualListFldDlg.RowSelectClick(Sender: TObject);
begin
  with FSrcGrid do
    if Sender = RowSelect then begin
      if RowSelect.Checked then Options := Options + [dgRowSelect]
      else Options := Options - [dgRowSelect];
    end
    else if Sender = Editing then begin
      if Editing.Checked then Options := Options + [dgEditing]
      else Options := Options - [dgEditing];
    end
end;

procedure TrDualListFldDlg.AlignBoxEnter(Sender: TObject);
begin
  CanChange := False;
end;

procedure TrDualListFldDlg.ColWidthEnter(Sender: TObject);
begin
  CanChange := False;
end;

procedure TrDualListFldDlg.DstListDragOver(Sender, Source: TObject; X,
  Y: Integer; State: TDragState; var Accept: Boolean);
begin
  Accept := (Source = SrcList);
end;

procedure TrDualListFldDlg.DstListDragDrop(Sender, Source: TObject; X,
  Y: Integer);
begin
  IncludeBtnClick(Sender);
end;

procedure TrDualListFldDlg.SrcListDragOver(Sender, Source: TObject; X,
  Y: Integer; State: TDragState; var Accept: Boolean);
begin
  Accept := (Source = DstList);
end;

procedure TrDualListFldDlg.SrcListDragDrop(Sender, Source: TObject; X,
  Y: Integer);
begin
  ExcludeBtnClick(Sender);
end;

procedure TrDualListFldDlg.DstListMouseMove(Sender: TObject;
  Shift: TShiftState; X, Y: Integer);
begin
  if not (SrcList.Dragging or DstList.Dragging) and
     (ssLeft in Shift) then
    TControl(Sender).BeginDrag(True);
end;

end.
