(*====================================================================
    File rdatasend.pas

    Components library for Delphi and BCB++.
     
    DataSet sending & receiving components

    Copyright  2004-2008, PSTG
    http://www.polesoft.ru
    mailto:support@polesoft.ru

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

======================================================================
$Id: rdatasend.pas,v 1.6 2008/10/06 11:56:10 khmelevskiy Exp $
======================================================================*)

unit rdatasend;

{$I POLARIS.INC}

interface

uses
  Types, SysUtils, Classes, DB, DBConsts, rConst, rinterface, rUtils,
  rplugmoniker;

type
  TrGetDataSetProc = function (const aDataSetName: string): TDataSet of object;

type

  IrLocalDataServer = interface
  ['{31B1224F-DDF2-47B8-8B6F-353A5F12C60D}']
    function GetDataSet(): TDataSet;
    procedure SetDataSet(const aDataSet: TDataSet);
    function Get_GetDataSetProc(): TrGetDataSetProc;
    procedure Set_GetDataSetProc(const aDataSetProc: TrGetDataSetProc);
    //
    property DataSet: TDataSet read GetDataSet write SetDataSet;
    property OnGetDataSet: TrGetDataSetProc read Get_GetDataSetProc write Set_GetDataSetProc;
  end;

  TrLocalDataServer = class(TrInterfacedObject, IrDataServer, IrLocalDataServer)
  private
    FDataSet: TDataSet;
    FOnGetDataSet: TrGetDataSetProc;
    FRecordSize: Integer;
  public
    constructor Create(aDataSet: TDataSet); overload;

    // IrDataServer
    function Open(aDataSetName: IrMemHolder; out aFields: IrMemHolder): HRESULT; virtual;
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
    function IsBOF(out aBOF: Boolean): HRESULT; virtual;
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
    function IsEOF(out aEOF: Boolean): HRESULT; virtual;
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
    function GetRecordCount(out aCount: Integer): HRESULT; virtual;
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
    function GetRecord(out aRecord: IrMemHolder): HRESULT; virtual;
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
    function GetBlobData(aFieldNumber: Integer; out aData: IrMemHolder): HRESULT; virtual;
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
    function Next(out aEOF: Boolean): HRESULT; virtual;
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
    function Close(): HRESULT; virtual;
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}

    // IrLocalDataServer
    function GetDataSet(): TDataSet;
    procedure SetDataSet(const aDataSet: TDataSet);
    function Get_GetDataSetProc(): TrGetDataSetProc;
    procedure Set_GetDataSetProc(const aDataSetProc: TrGetDataSetProc);

    //
    property DataSet: TDataSet read FDataSet write FDataSet;
    property OnGetDataSet: TrGetDataSetProc read FOnGetDataSet write FOnGetDataSet;
  end;

  TrDataClientError = class(Exception);

  TrDataClient = class(TDataSet)
  private
    FDataServer: IrDataServer;
    FDataSetName: string;
    FRecordSize: Integer;
    FRecordBuffer: IrMemHolder;
    FDiscardReadOnly: Boolean;
    procedure CallCheckSrv(aCode: HRESULT);
    function CalcFieldNo(aField: TField): Integer;
    procedure mcheck(aErrorCode: HRESULT);
  protected
    procedure InternalOpen; override;
    procedure InternalClose; override;
    function IsCursorOpen: Boolean; override;
    function GetCanModify: Boolean; override;
    function GetRecord(Buffer: {$IFDEF POLARIS_D12}PByte{$ELSE}PChar{$ENDIF}; GetMode: TGetMode; DoCheck: Boolean): TGetResult; override;
    procedure InitRecord(Buffer: {$IFDEF POLARIS_D12}PByte{$ELSE}PChar{$ENDIF}); override;
    procedure SetBufListSize(Value: Integer); override;
    function GetRecordCount: Integer; override;
    procedure InternalHandleException; override;
    procedure InternalInitFieldDefs; override;
  public
    constructor Create(AOwner: TComponent); override;
    function GetFieldData(Field: TField; Buffer: Pointer): Boolean; overload; override;
    function GetFieldData(FieldNo: Integer; Buffer: Pointer): Boolean; overload; override;
    function CreateBlobStream(Field: TField; Mode: TBlobStreamMode): TStream; override;
    function IsSequenced: Boolean; override;
    //
    property DataServer: IrDataServer read FDataServer write FDataServer;
    property DataSetName: string read FDataSetName write FDataSetName;
  published
    property DiscardReadOnly: Boolean read FDiscardReadOnly write FDiscardReadOnly;
  end;

implementation

{ TrLocalDataServer }

constructor TrLocalDataServer.Create(aDataSet: TDataSet);
begin
  inherited Create();
  FDataSet := aDataSet;
end;

function TrLocalDataServer.Open(aDataSetName: IrMemHolder; out aFields: IrMemHolder): HRESULT;
var
  s: string;
  sl: TStringList;
  i: Integer;
  f: TField;
begin
  Result := SetError();
  try
    if Assigned(FOnGetDataSet) then DataSet := FOnGetDataSet(rs(aDataSetName));
    if FDataSet = nil then raise Exception.CreateFmt(srBadPropertyValue, ['DataSet']);
    with FDataSet do if not Active then Open();
    FRecordSize := 0;
    s := '';
    sl := TStringList.Create();
    try
      for i := 0 to FDataSet.FieldCount - 1 do begin
        f := FDataSet.Fields[i];
        sl.Values['FieldName'] := f.FieldName;
        sl.Values['DataType'] := IntToStr(Ord(f.DataType));
        sl.Values['DataSize'] := IntToStr(f.DataSize);
        sl.Values['Size'] := IntToStr(f.Size);
        if f.ReadOnly then sl.Values['ReadOnly'] := '1';
        sl.Values['DisplayLabel'] := f.DisplayLabel;
        sl.Values['Visible'] := IntToStr(Integer(f.Visible));
        if f is TNumericField then
          sl.Values['DisplayFormat'] := TNumericField(f).DisplayFormat;
        sl.Values['Tag'] := IntToStr(f.Tag);
        s := s + sl.Text + RegStringDelim;
        sl.Clear;
        if not (f is TBlobField) then Inc(FRecordSize, f.DataSize + 1);
      end;
    finally
      sl.Free();
    end;
    aFields := rs(s);
  except
    on E: Exception do Result := SetError(PE_rDataServer_Open, E);
  end;
end;

function TrLocalDataServer.GetRecord(out aRecord: IrMemHolder): HRESULT;
  //
  procedure __check(aErrorCode: HRESULT);
  begin
    if aErrorCode = S_OK then Exit;
    raise Exception.CreateFmt(srMemError, [aErrorCode]);
  end;
  //
var
  i: Integer;
  f: TField;
  p: Pointer;
begin
  Result := SetError();
  try
    if FDataSet = nil then raise Exception.CreateFmt(srBadPropertyValue, ['DataSet']);
    if not FDataSet.Active then raise Exception.Create(SDataSetClosed);
    if FRecordSize = 0 then Exit;
    aRecord := NewMemHolder();
    __check(aRecord.Alloc(FRecordSize));
    __check(aRecord.GetPtr(PChar(p)));
    for i := 0 to FDataSet.FieldCount - 1 do begin
      f := FDataSet.Fields[i];
      if not (f is TBlobField) then begin
        pByte(p)^ := Byte(f.IsNull);
        Inc(pByte(p));
        if not f.IsNull then f.GetData(p);
        Inc(DWORD(p), f.DataSize);
      end;
    end;
  except
    on E: Exception do Result := SetError(PE_rDataServer_GetRecord, E);
  end;
end;

function TrLocalDataServer.GetBlobData(aFieldNumber: Integer; out aData: IrMemHolder): HRESULT;
var
  ss: TStringStream;
begin
  Result := SetError();
  try
    if FDataSet = nil then raise Exception.CreateFmt(srBadPropertyValue, ['DataSet']);
    if not FDataSet.Active then raise Exception.Create(SDataSetClosed);
    if aFieldNumber >= FDataSet.FieldCount then raise Exception.Create(SFieldIndexError);
    if not (FDataSet.Fields[aFieldNumber] is TBlobField) then
      raise Exception.CreateFmt(SBadFieldType, [FDataSet.Fields[aFieldNumber].FieldName]);
    ss := TStringStream.Create('');
    try
      TBlobField(FDataSet.Fields[aFieldNumber]).SaveToStream(ss);
      aData := rs(ss.DataString);
    finally
      ss.Free();
    end;
  except
    on E: Exception do Result := SetError(PE_rDataServer_GetBlobData, E);
  end;
end;

function TrLocalDataServer.IsBOF(out aBOF: Boolean): HRESULT;
begin
  Result := SetError();
  try
    if FDataSet = nil then raise Exception.CreateFmt(srBadPropertyValue, ['DataSet']);
    aBOF := FDataSet.BOF;
  except
    on E: Exception do Result := SetError(PE_rDataServer_IsBOF, E);
  end;
end;

function TrLocalDataServer.IsEOF(out aEOF: Boolean): HRESULT;
begin
  Result := SetError();
  try
    if FDataSet = nil then raise Exception.CreateFmt(srBadPropertyValue, ['DataSet']);
    aEOF := FDataSet.EOF;
  except
    on E: Exception do Result := SetError(PE_rDataServer_IsEOF, E);
  end;
end;

function TrLocalDataServer.Next(out aEOF: Boolean): HRESULT;
begin
  Result := SetError();
  try
    if FDataSet = nil then raise Exception.CreateFmt(srBadPropertyValue, ['DataSet']);
    with FDataSet do begin
      if not EOF then Next();
      aEOF := EOF;
    end;
  except
    on E: Exception do Result := SetError(PE_rDataServer_Next, E);
  end;
end;

function TrLocalDataServer.GetRecordCount(out aCount: Integer): HRESULT;
begin
  Result := SetError();
  try
    if FDataSet = nil then raise Exception.CreateFmt(srBadPropertyValue, ['DataSet']);
    aCount := iif(FDataSet.IsUniDirectional, -1, FDataSet.RecordCount);
  except
    on E: Exception do Result := SetError(PE_rDataServer_GetRecordCount, E);
  end;
end;

function TrLocalDataServer.Close(): HRESULT;
begin
  Result := SetError();
  try
    if FDataSet = nil then raise Exception.CreateFmt(srBadPropertyValue, ['DataSet']);
    FDataSet.Close();
  except
    on E: Exception do Result := SetError(PE_rDataServer_Close, E);
  end;
end;

function TrLocalDataServer.GetDataSet: TDataSet;
begin
  Result := FDataSet;
end;

procedure TrLocalDataServer.SetDataSet(const aDataSet: TDataSet);
begin
  FDataSet := aDataSet;
end;

function TrLocalDataServer.Get_GetDataSetProc: TrGetDataSetProc;
begin
  Result := FOnGetDataSet;
end;

procedure TrLocalDataServer.Set_GetDataSetProc(const aDataSetProc: TrGetDataSetProc);
begin
  FOnGetDataSet := aDataSetProc;
end;

{ TrDataClient }

constructor TrDataClient.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  SetUniDirectional(True);
  ObjectView := False;
end;

procedure TrDataClient.CallCheckSrv(aCode: HRESULT);
var
  m: IrMemHolder;
begin
  if aCode = S_OK then Exit;
  if FDataServer.GetLastErrorMsg(m) <> S_OK
    then raise TrDataClientError.CreateFmt('Error %d', [aCode])
    else raise TrDataClientError.Create(rs(m));
end;

procedure TrDataClient.InternalOpen;
  //
  type TrFieldRec = record
    FieldName: string;
    DataType: TFieldType;
    DataSize: Integer;
    Size: Integer;
    ReadOnly: Boolean;
    DisplayLabel: string;
    Visible: Boolean;
    DisplayFormat: string;
    Tag: Integer;
  end;
  //
var
  vFields: IrMemHolder;
  s, vsf: string;
  sl: TStringList;
  i, i1, i2: Integer;
  f: TField;
  fda: array of TrFieldRec;
begin
  if FDataServer = nil then raise TrDataClientError.CreateFmt(srBadPropertyValue, ['DataServer']);
  CallCheckSrv(FDataServer.Open(rs(FDataSetName), vFields));
  vsf := rs(vFields);
  if vsf = '' then raise TrDataClientError.Create(SInvalidFieldSize);
  FRecordSize := 0;
  sl := TStringList.Create();
  try
    i1 := 1; i2 := 1;
    while i2 <= Length(vsf) do begin
      if vsf[i2] = RegStringDelim then begin
        sl.Text := Copy(vsf, i1, i2 - i1);
        s := sl.Values['FieldName'];
        if s = '' then raise TrDataClientError.Create(SFieldNameMissing);
        SetLength(fda, Length(fda) + 1);
        with fda[Length(fda) - 1] do begin
          FieldName := s;
          DataType := TFieldType(StrToIntDef(sl.Values['DataType'], 0));
          DataSize := StrToIntDef(sl.Values['DataSize'], 0);
          Size := StrToIntDef(sl.Values['Size'], 0);
          ReadOnly := not FDiscardReadOnly and (sl.Values['ReadOnly'] = '1');
          DisplayLabel := sl.Values['DisplayLabel'];
          Visible := sl.Values['Visible'] <> '0';
          DisplayFormat := sl.Values['DisplayFormat'];
          Tag := StrToIntDef(sl.Values['Tag'], 0);
        end;
        i1 := i2 + 1;
      end;
      Inc(i2);
    end; // while
  finally
    sl.Free();
  end;
  Fields.Clear;
  FieldDefs.Clear;
  for i := 0 to Length(fda) - 1 do
    with fda[i] do begin
      if (DataType = ftString) and (Size = 0) then
        Size := DataSize - 1;
      FieldDefs.Add(FieldName, DataType, Size, False);
    end;
  CreateFields;
  BindFields(True);
  for i := 0 to Length(fda) - 1 do begin
    f := Fields[i];
    //f.Name := Self.Name + f.FieldName;      Exception
    f.ReadOnly := fda[i].ReadOnly;
    f.DisplayLabel := fda[i].DisplayLabel;
    f.Visible := fda[i].Visible;
    if f is TNumericField then
      TNumericField(f).DisplayFormat := fda[i].DisplayFormat;
    f.Tag := fda[i].Tag;
    if not (f is TBlobField) then Inc(FRecordSize, f.DataSize + 1);
  end;
end;

procedure TrDataClient.InternalClose;
begin
  if FDataServer = nil then raise TrDataClientError.CreateFmt(srBadPropertyValue, ['DataServer']);
  CallCheckSrv(FDataServer.Close());
  Fields.Clear;
  FieldDefs.Clear;
end;

function TrDataClient.IsCursorOpen: Boolean;
begin
  Result := True;
end;

function TrDataClient.GetCanModify: Boolean;
begin
  Result := False;
end;

procedure TrDataClient.mcheck(aErrorCode: HRESULT);
begin
  if aErrorCode = S_OK then Exit;
  raise TrDataClientError.CreateFmt(srMemError, [aErrorCode]);
end;

function TrDataClient.GetRecord(Buffer: {$IFDEF POLARIS_D12}PByte{$ELSE}PChar{$ENDIF}; GetMode: TGetMode; DoCheck: Boolean): TGetResult;
var
  vEOF: Boolean;
  vRecord: IrMemHolder;
  i: Integer;
begin
  if BOF
    then CallCheckSrv(FDataServer.IsEOF(vEOF))
    else CallCheckSrv(FDataServer.Next(vEOF));
  if vEOF then Result := grEOF
  else begin
    CallCheckSrv(FDataServer.GetRecord(vRecord));
    if vRecord = nil then i := 0
    else mcheck(vRecord.GetSize(i));
    if i <> FRecordSize then raise TrDataClientError.Create(SInvalidFieldSize);
    FRecordBuffer := vRecord;
    Result := grOK;
  end;
end;

function TrDataClient.CalcFieldNo(aField: TField): Integer;
var
  i: Integer;
begin
  Result := 0;
  for i := 0 to Fields.Count - 1 do
    if aField.FieldName = Fields[i].FieldName then begin
      Result := i + 1;
      Break;
    end;
end;

function TrDataClient.GetFieldData(Field: TField; Buffer: Pointer): Boolean;
var
  n: Integer;
begin
  Result := False;
  n := CalcFieldNo(Field);
  if n > 0 then Result := GetFieldData(n, Buffer);
end;

function TrDataClient.GetFieldData(FieldNo: Integer; Buffer: Pointer): Boolean;
var
  f, f1: TField;
  i: Integer;
  p: Pointer;
begin
  Result := False;
  if FRecordBuffer = nil then Exit;
  f := Fields[FieldNo - 1];
  if f is TBlobField then Exit;
  mcheck(FRecordBuffer.GetPtr(PChar(p)));
  for i := 0 to FieldNo - 2 do begin
    f1 := Fields[i];
    if not (f1 is TBlobField) then Inc(DWORD(p), f1.DataSize + 1);
  end;
  Result := not Boolean(pByte(p)^);
  if not Result or (nil = Buffer) then Exit;
  Inc(pByte(p));
  Move(p^, Buffer^, f.DataSize);
end;

function TrDataClient.CreateBlobStream(Field: TField; Mode: TBlobStreamMode): TStream;
var
  vData: IrMemHolder;
begin
  CallCheckSrv(FDataServer.GetBlobData(CalcFieldNo(Field) - 1, vData));
  Result := TStringStream.Create(rs(vData));
end;

procedure TrDataClient.InitRecord(Buffer: {$IFDEF POLARIS_D12}PByte{$ELSE}PChar{$ENDIF});
begin
  // nop
end;

procedure TrDataClient.SetBufListSize(Value: Integer);
begin
  // nop
end;

function TrDataClient.IsSequenced: Boolean;
begin
  Result := False;
end;

function TrDataClient.GetRecordCount: Integer;
begin
  CallCheckSrv(FDataServer.GetRecordCount(Result));
end;

procedure TrDataClient.InternalHandleException;
begin
  // nop
end;

procedure TrDataClient.InternalInitFieldDefs;
begin
  // nop
end;

end.

